SystemOrganization addCategory: #'SmallWiki-Kernel-Action'!
SystemOrganization addCategory: #'SmallWiki-Kernel-Server'!
SystemOrganization addCategory: #'SmallWiki-Kernel-Tests'!
SystemOrganization addCategory: #'SmallWiki-Kernel-Structure'!
SystemOrganization addCategory: #'SmallWiki-Kernel-Security'!
SystemOrganization addCategory: #'SmallWiki-Kernel-Storage'!
SystemOrganization addCategory: #'SmallWiki-Kernel-Template'!
SystemOrganization addCategory: #'SmallWiki-Kernel-Visitor'!

TestSuite subclass: #SWSmallWikiTestSuite
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Tests'!

!SWSmallWikiTestSuite commentStamp: '<historical>' prior: 0!
One of the main benefits of SmallWiki is that it is extensively tested. Before reporting any bugs you should always run the tests in order to verify if there is something wrong with your set-up. Maintenance, porting or extending should go together with running the existing tests and writing new ones to further improve the quality of the code. All the major releases of SmallWiki have to pass all of the provided test-cases. 

The following table is built automatically, while putting together the SmallWiki documentation. This information is taken from the authors current development environment and therefore should be always accurate:

| SmallWiki: | \texttt{[ VersionString ]}
| VisualWorks: | \texttt{[ SystemUtils version ]}
| Time stamp: | \texttt{[ Timestamp now ]}
| Test results: | \texttt{[ | suite result | suite := SmallWikiTestSuite new. result := [ suite run ] ifCurtailed: [  ]. result ]}!

!SWSmallWikiTestSuite methodsFor: 'initialization' stamp: ''!
initialize
	| testclasses testselectors |
	testclasses := TestCase allSubclasses select: [ :class |
		class environment == SmallWiki ].
	testselectors := testclasses 
		inject: Set new
		into: [ :set :class |
			set addAll: (class testSelectors collect: [ :selector | class selector: selector ]);
			yourself ].
	self addTests: testselectors! !

!SWSmallWikiTestSuite class methodsFor: 'instance-creation' stamp: ''!
new
	^super new
		initialize! !

Object subclass: #SWSecurityInformation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Security'!

!SWSecurityInformation commentStamp: '<historical>' prior: 0!
I am an abstract entity representig the security-informatin in the system. My sole resposibility is to check where I have a certain permission. There is also the possibility to assert permissions, what raises an UnauthorizedError exception.!

!SWSecurityInformation methodsFor: 'testing' stamp: ''!
hasPermission: aPermission
	"If the receiver has got aPremission return true, else return false."

	self subclassResponsibility! !

!SWSecurityInformation methodsFor: 'testing' stamp: 'chbu 10/22/2003 09:43'!
assertPermission: aPermission
	"Assert that aPermission is valid in the receiver, throws a UnauthorizedError if
	the permission is missing."

	(self hasPermission: aPermission)
		ifFalse: [ SWUnauthorizedError signal"With: self" ]! !

!SWSecurityInformation methodsFor: 'as yet unclassified' stamp: 'chbu 10/23/2003 18:10'!
copy

	^super copy postCopy! !

!SWSecurityInformation class methodsFor: 'instance-creation' stamp: ''!
new
	^super new
		initialize! !

!SWSecurityInformation methodsFor: 'initialization' stamp: ''!
initialize! !

SWSecurityInformation subclass: #SWRole
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Security'!

!SWRole commentStamp: '<historical>' prior: 0!
I represent a role in the system. Multiple permissions might get assigned to any role. Permission will be granted if any of the permission grants the permission.!

SWRole subclass: #SWBasicRole
	instanceVariableNames: 'name permissions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Security'!

!SWBasicRole methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: self name;
		nextPut: $).! !

!SWBasicRole methodsFor: 'accessing-permissions' stamp: ''!
add: aPermission
	permissions add: aPermission! !

!SWBasicRole methodsFor: 'testing' stamp: ''!
hasPermission: aPermisson
	^self permissions
		includes: aPermisson! !

!SWBasicRole methodsFor: 'accessing-permissions' stamp: ''!
addAll: aCollection
	permissions addAll: aCollection! !

!SWBasicRole methodsFor: 'accessing-permissions' stamp: ''!
remove: aPermission
	^permissions remove: aPermission ifAbsent: [ nil ]! !

!SWBasicRole methodsFor: 'copying' stamp: ''!
postCopy
	super postCopy.
	permissions := permissions copy.! !

!SWBasicRole methodsFor: 'accessing' stamp: ''!
name
	^name! !

!SWBasicRole methodsFor: 'accessing-permissions' stamp: ''!
do: aBlock
	^permissions do: aBlock! !

!SWBasicRole methodsFor: 'initialization' stamp: ''!
initialize
	super initialize.
	permissions := Set new.! !

!SWBasicRole methodsFor: 'comparing' stamp: ''!
hash
	^name hash! !

!SWBasicRole methodsFor: 'accessing' stamp: ''!
permissions
	^permissions! !

!SWBasicRole methodsFor: 'private' stamp: ''!
name: aString
	name := aString! !

!SWBasicRole methodsFor: 'comparing' stamp: ''!
= aRole
	^self class = aRole class 
		and: [ name = aRole name ]! !

!SWBasicRole class methodsFor: 'instance creation' stamp: ''!
name: aString
	^self new
		name: aString;
		yourself! !

!SWRole methodsFor: 'accessing' stamp: ''!
permissions
	^self subclassResponsibility! !

!SWRole methodsFor: 'accessing' stamp: ''!
name
	^self subclassResponsibility! !

SWRole subclass: #SWAdminRole
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Security'!

!SWAdminRole methodsFor: 'comparing' stamp: ''!
= aRole
	^self class = aRole class! !

!SWAdminRole methodsFor: 'accessing' stamp: ''!
name
	^'administrator'! !

!SWAdminRole methodsFor: 'accessing' stamp: 'chbu 10/22/2003 17:04'!
permissions
	^SWStructure allSubclasses
		inject: Set new
		into: [ :collection :each | collection addAll: each permissions; yourself ]! !

!SWAdminRole methodsFor: 'testing' stamp: ''!
hasPermission: aPermission
	^true! !

!SWAdminRole methodsFor: 'comparing' stamp: ''!
hash
	^self class hash! !

SWSecurityInformation subclass: #SWUser
	instanceVariableNames: 'username password roles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Security'!

!SWUser commentStamp: '<historical>' prior: 0!
I represent a user and its password in the system. Multiple roles might be assigned to any user. Permission will be granted if any of the roles grants the permission.

Note, that there is no read-accessor for the password, but the password might still get accessed when using an inspector. This weak mechanism should be replaced with hashed passwords.!

!SWUser methodsFor: 'testing' stamp: ''!
isAnonymous
	^self = self class anonymous! !

!SWUser methodsFor: 'accessing-roles' stamp: ''!
add: aRole
	roles remove: aRole ifAbsent: [ ].
	roles add: aRole! !

!SWUser class methodsFor: 'instance creation' stamp: ''!
username: anUsername password: aPassword
	^self new
		username: anUsername;
		password: aPassword;
		yourself! !

!SWUser methodsFor: 'comparing' stamp: ''!
hash
	^username hash! !

!SWUser methodsFor: 'accessing' stamp: ''!
username
	^username! !

!SWUser methodsFor: 'update' stamp: ''!
updateRoles: aCollection
	"Create a new user with aCollection of roles updated."

	^aCollection 
		inject: self 
		into: [ :user :role | user updateRole: role ]! !

!SWUser methodsFor: 'testing' stamp: ''!
validatePassword: aString
	"Returns true if the password could be validated or false in the opposite 
	case. Note: If the passwort of ther receiving user is nil it will validate with 
	everything!!"

	^password isNil 
		or: [ password = aString ]! !

!SWUser methodsFor: 'accessing-roles' stamp: ''!
do: aBlock
	^roles do: aBlock! !

!SWUser methodsFor: 'initialize' stamp: ''!
initialize
	super initialize.
	roles := Set new! !

!SWUser class methodsFor: 'instance creation' stamp: ''!
anonymous
	^self
		username: 'anonymous'
		password: nil! !

!SWUser methodsFor: 'private' stamp: ''!
username: aString
	username := aString! !

!SWUser methodsFor: 'private' stamp: ''!
password: aString
	password := aString! !

!SWUser methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: self username;
		nextPut: $).! !

!SWUser methodsFor: 'comparing' stamp: ''!
= anUser
	^self class = anUser class 
		and: [ username = anUser username ]! !

!SWUser methodsFor: 'accessing-roles' stamp: ''!
remove: aRole
	^roles remove: aRole ifAbsent: [ nil ]! !

!SWUser methodsFor: 'update' stamp: ''!
updateRole: aRole
	"Create a new user with aRole updated."

	^(roles includes: aRole)
		ifTrue: [ self copy add: aRole; yourself ]
		ifFalse: [ self ]! !

!SWUser methodsFor: 'accessing' stamp: ''!
roles
	^roles! !

!SWUser methodsFor: 'copying' stamp: ''!
postCopy
	super postCopy.
	roles := roles copy.! !

!SWUser methodsFor: 'testing' stamp: ''!
hasPermission: aPermission
	^roles anySatisfy: [ :role |
		role hasPermission: aPermission ]! !

SWSecurityInformation subclass: #SWPermission
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Security'!

!SWPermission commentStamp: '<historical>' prior: 0!
I represent a permission in the system and I am the basic entity for the permission management. A permission will be granted if I am equal to the given permission. 

Permission will be usually used in conjuction with User>>hasPermission: aPermission or User>>assertPermission.!

!SWPermission methodsFor: 'copying' stamp: ''!
copy
	"Answer the receiver, because Permission's should be unique."

	^self! !

!SWPermission methodsFor: 'accessing' stamp: ''!
name
	^name! !

!SWPermission methodsFor: 'comparing' stamp: ''!
hash
	^name hash! !

!SWPermission methodsFor: 'testing' stamp: ''!
hasPermission: aPermission
	^aPermission = self! !

!SWPermission methodsFor: 'copying' stamp: ''!
shallowCopy
	"Answer the receiver, because Permission's should be unique."

	^self! !

!SWPermission class methodsFor: 'instance creation' stamp: ''!
name: aString
	^self new
		name: aString;
		yourself.! !

!SWPermission methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: self name;
		nextPut: $).! !

!SWPermission methodsFor: 'private' stamp: ''!
name: aString
	name := aString! !

!SWPermission methodsFor: 'comparing' stamp: ''!
= aPermission
	^self class = aPermission class 
		and: [ name = aPermission name ]! !

Object subclass: #SWResponse
	instanceVariableNames: 'stream status type headers cookies server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Server'!

!SWResponse class methodsFor: 'status-clienterror' stamp: ''!
statusPaymentRequired
	"Request should be retried with proper charge-to header."
	^402! !

!SWResponse methodsFor: 'accessing-cookies' stamp: ''!
cookies: aStream
	cookies := aStream! !

!SWResponse class methodsFor: 'instance creation' stamp: ''!
server: aServer
	^self new
		server: aServer;
		yourself! !

!SWResponse methodsFor: 'configuration' stamp: 'chbu 10/22/2003 17:31'!
defaultStream
	^SWHtmlWriteStream on: String new! !

!SWResponse class methodsFor: 'status-clienterror' stamp: ''!
statusUnauthorized
	"Request should be retried with proper authorization header. This is the 
	response which triggers the browser to pop up the dialog requesting your 
	username and password."
	^401! !

!SWResponse methodsFor: 'accessing' stamp: ''!
server
	^server! !

!SWResponse methodsFor: 'accessing-headers' stamp: ''!
headers: aDictionary
	headers := aDictionary! !

!SWResponse class methodsFor: 'status-success' stamp: ''!
statusAccepted
	"Request accepted for asynchronous processing."
	^202! !

!SWResponse class methodsFor: 'status-redirection' stamp: ''!
statusSeeOther
	"Same as found, but a different method may be used to access the 
	document; details about the method are sent in the message body."
	^303! !

!SWResponse methodsFor: 'configuration' stamp: ''!
defaultCookies
	^Dictionary new! !

!SWResponse methodsFor: 'actions' stamp: ''!
redirectTo: aString
	self status: self class statusFound.
	self headerAt: 'Location' put: aString! !

!SWResponse methodsFor: 'accessing' stamp: ''!
type
	type isNil
		ifTrue: [ type := self defaultType ].
	^type! !

!SWResponse class methodsFor: 'status-success' stamp: ''!
statusOk	
	"Request was processed without any error conditions."
	^200! !

!SWResponse class methodsFor: 'status-clienterror' stamp: ''!
statusForbidden
	"Authorization will not help."
	^403! !

!SWResponse class methodsFor: 'status-redirection' stamp: ''!
statusFound
	"Same as move, except that linking to the found address doesn't make much 
	sense, since the document URL is expected to change. This is the code that 
	the httpd returns for a cgi script whose output contained a Location: header."
	^302! !

!SWResponse methodsFor: 'accessing-headers' stamp: ''!
headers
	^headers! !

!SWResponse class methodsFor: 'status-clienterror' stamp: ''!
statusBadRequest
	"Impossible request or syntax error."
	^400! !

!SWResponse class methodsFor: 'status-redirection' stamp: ''!
statusMoved
	"Browsers with link editing capabilities should automatically link to the new 
	reference. The response contains one or more header lines of the form URI: url 
	string CrLf which specify alternative addresses for the object in question. The 
	string is an optional comment field."
	^301! !

!SWResponse methodsFor: 'accessing-cookies' stamp: ''!
cookieRemove: aKey
	^cookies at: aKey put: nil! !

!SWResponse methodsFor: 'accessing-headers' stamp: ''!
headerAt: aKey put: aValue
	headers at: aKey put: aValue! !

!SWResponse methodsFor: 'accessing' stamp: ''!
status
	status isNil 
		ifTrue: [ status := self defaultStatus ].
	^status! !

!SWResponse class methodsFor: 'status-redirection' stamp: ''!
statusNotModified
	"Use the local copy if you cached it. Often seen when using the HEAD 
	method, rather than the GET method."
	^304! !

!SWResponse class methodsFor: 'status-servererror' stamp: ''!
statusInternalError
	"A rather meaningless catch-all message that indicates that the site admin 
	goofed on their CGI program."
	^500! !

!SWResponse methodsFor: 'accessing-cookies' stamp: ''!
cookieAt: aKey put: aValue
	^cookies at: aKey put: aValue! !

!SWResponse class methodsFor: 'status-success' stamp: ''!
statusCreated
	"Following a POST command, this indicates success, but the text of the response
	line indicates the URL of the new document."
	^201! !

!SWResponse class methodsFor: 'status-success' stamp: ''!
statusPartialInformation
	"Returned information may be cached or private."
	^203! !

!SWResponse methodsFor: 'accessing' stamp: ''!
status: anInteger
	status := anInteger! !

!SWResponse class methodsFor: 'instance creation' stamp: ''!
new
	^super new 
		initialize! !

!SWResponse methodsFor: 'configuration' stamp: ''!
defaultStatus
	^self class statusOk! !

!SWResponse methodsFor: 'initialize' stamp: ''!
initialize
	headers := self defaultHeaders.
	cookies := self defaultCookies.
	stream := self defaultStream.! !

!SWResponse methodsFor: 'accessing-cookies' stamp: ''!
cookies
	^cookies! !

!SWResponse methodsFor: 'accessing' stamp: ''!
stream
	^stream! !

!SWResponse class methodsFor: 'status-servererror' stamp: ''!
statusTimedOut
	"Not in the HTTP spec, but implemented by some HTTP servers."
	^502! !

!SWResponse methodsFor: 'configuration' stamp: ''!
defaultType
	^'text/html'! !

!SWResponse methodsFor: 'actions' stamp: ''!
unauthorized
	self status: self class statusUnauthorized.
	self headerAt: 'WWW-Authenticate' put: 'Basic realm="SmallWiki"'! !

!SWResponse methodsFor: 'private' stamp: ''!
server: aServer
	server := aServer.
	stream server: aServer.! !

!SWResponse methodsFor: 'configuration' stamp: 'chbu 10/24/2003 16:48'!
defaultHeaders
	^Dictionary new
		at: 'Cache-Control' put: 'no-cache';
		at: 'X-Wiki-Engine' put: SWSmallWiki versionString;
		at: 'X-Wiki-Copyright' put: SWSmallWiki copyrightString;
		yourself! !

!SWResponse class methodsFor: 'status-clienterror' stamp: ''!
statusNotFound
	"A document with that URL doesn't exist."
	^404! !

!SWResponse methodsFor: 'accessing' stamp: ''!
stream: aWriteStream
	stream := aWriteStream! !

!SWResponse methodsFor: 'accessing' stamp: ''!
type: aString
	type := aString! !

!SWResponse methodsFor: 'testing' stamp: ''!
isRedirect
	^(status == self class statusFound)
		or: [ status == self class statusMoved 
		or: [ status == self class statusNotModified 
		or: [ status == self class statusSeeOther ] ] ]! !

!SWResponse class methodsFor: 'status-servererror' stamp: ''!
statusNotImplemented
	"Another rather ambiguous message, typically meaning that you tried to 
	execute something that was not executable, or POST to someting that 
	was not a CGI program, or something similar."
	^501! !

!SWResponse class methodsFor: 'status-success' stamp: ''!
statusNoResponse
	"Used for scripts that don't return a visible result."
	^204! !

Object subclass: #SWAction
	instanceVariableNames: 'structure request response server html'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWAction commentStamp: 'chbu 10/22/2003 08:57' prior: 0!
Actions are instantiated by a structure and they are initialized with that structure and the current request using the constructor method #request:structure:. Actions have basically two tasks: performing the action itself and initiating or doing the GUI rendering. Actions do also represent the context in which a page is rendered as they know about their structure, the request, the response, and the security status.

Action Protocol
This part of the action is used to handle the requests. The message #execute is called by the structure after initializing the required instance variables. It checks the security permissions of the current user, evaluates the callbacks and starts the rendering by calling #render on itself. The running action might use the accessors to manipulate and mediate with the current environment. It is usually not necessary to override the message #execute, use the callback mechanism described in Chapter \ref{HTML and Callbacks} on page \pageref{HTML and Callbacks} instead. 

Rendering Protocol
The rendering process is started from the message #render at the end of #execute. The message #render fetches the collection of templates of the associated structure and starts generating the XHTML output. It asks each template to render the content they want to emit into the \texttt{<head>...</head>} part of the output. Afterwards the body part \texttt{<body>...</body>} is generated and again every template might contribute its content into that part.
As explained on page \pageref{Template} in chapter \ref{Template}, there is always an instance of \texttt{TemplateBodyContent} available calling back the message #renderContent of the action: override this message to let your action render its user-interface. You should not change the state of any component inside the rendering protocol, as an action is unable to know when and how many times it is actually called.!

!SWAction methodsFor: 'testing' stamp: ''!
isEnabled
	^true! !

!SWAction methodsFor: 'accessing' stamp: ''!
user
	^self request user! !

SWAction subclass: #SWEditAction
	instanceVariableNames: 'title target exception'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWEditAction methodsFor: 'configuration' stamp: ''!
defaultTarget
	^structure url! !

!SWEditAction methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:21'!
hasError
	^exception notNil! !

!SWEditAction methodsFor: 'action' stamp: 'chbu 11/19/2003 19:37'!
save
	"Do not override this message, instead use the messages priveateSave and 
	shouldSave to check your state and to do the actual saving. If necessary a new
	version of the current structure will be generated automatically and you will be
	redirected to the right url."

	self shouldSave ifTrue: [
		self privatePreSave.
		[ self privateSave ]
			ensure: [ self privatePostSave ] ].
	self hasError ifFalse: [
		response redirectTo: self target ]! !

!SWEditAction methodsFor: 'utilities' stamp: ''!
privatePreSave
	structure nextVersion.! !

!SWEditAction methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:22'!
renderError
	exception isNil ifFalse: [
		html tableRow: [
			html tableHeading: 'Error:'.
			html tableData: [
				html spanNamed: #error with: exception messageText ] ] ]! !

!SWEditAction methodsFor: 'rendering' stamp: 'chbu 11/12/2003 20:07'!
renderSummary
	html tableRow: [
		html attributeAt: 'width' put: '100'.
		html tableHeading: 'Modified:'.
		html tableData: structure timestamp asString]! !

!SWEditAction methodsFor: 'rendering' stamp: ''!
renderFields
	html tableRow: [
		html tableHeading: 'Title:'.
		html tableData: [ 
			html attributeAt: 'style' put: 'width: 100%'.
			html textInputWithValue: self title callback: #title: ] ]! !

!SWEditAction methodsFor: 'action' stamp: 'chbu 11/19/2003 19:35'!
shouldSave
	"You have to override this message and return false, if you like to stop the saving 
	mechanism, e.g. in case of invalid user input. If you return false, no new version will 
	be generated and no dependents will be called."

	^self hasError not! !

!SWEditAction methodsFor: 'rendering' stamp: ''!
renderButton
	html tableRowWith: nil with: [
		html submitButtonWithAction: #save text: 'Save' ]! !

!SWEditAction methodsFor: 'rendering' stamp: 'chbu 11/19/2003 19:37'!
renderContent
	self renderForm: [
		html hiddenInputWithValue: self target callback: #target:.
		html attributeAt: #width put: '100%'.
		html table: [
			self renderError.
			self renderSummary. 
			self renderFields.
			self renderButton ] ]! !

!SWEditAction methodsFor: 'accessing' stamp: ''!
target: aUrlString
	target := aUrlString! !

!SWEditAction methodsFor: 'accessing' stamp: ''!
target
	target isNil ifTrue: [ 
		target := self defaultTarget ].
	^target! !

!SWEditAction methodsFor: 'utilities' stamp: ''!
privatePostSave
	self response redirectTo: self url.
	self changed: #edit.! !

!SWEditAction methodsFor: 'accessing' stamp: 'chbu 11/19/2003 19:36'!
title: aString
	[ self structure renameTo: aString. target := nil ]
		on: SWDuplicatedStructure
		do: [ :err | self exception: err ].
	title := aString! !

!SWEditAction methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:21'!
exception: anException
	exception := anException! !

SWEditAction subclass: #SWPageEdit
	instanceVariableNames: 'wiki document'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWPageEdit class methodsFor: 'accessing' stamp: ''!
title
	^'Edit'! !

!SWPageEdit methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:21'!
parserException: anException
	self exception: anException.
	self wiki streamContents: [ :input |
		wiki := String streamContents: [ :output |
			output nextPutAll: (input next: anException parameter position - 1).
			output nextPutAll: anException messageText; nextPutAll: ' -> '.
			output nextPutAll: input upToEnd ] ]! !

!SWPageEdit methodsFor: 'accessing' stamp: 'chbu 10/22/2003 17:18'!
wiki
	wiki isNil 
		ifTrue: [ wiki := (SWVisitorRendererWiki render: structure) contents ].
	^wiki! !

!SWPageEdit methodsFor: 'rendering' stamp: ''!
renderSyntaxHelp
	| syntax |
	syntax := structure root resolveTo: '/Information/Syntax'.
	syntax isNil ifFalse: [
		html tableRow: [
			html tableHeading: 'Syntax:'.
			html tableData: [ html anchorWithUrl: syntax url do: syntax title ] ] ]! !

!SWPageEdit methodsFor: 'rendering' stamp: 'chbu 11/19/2003 19:32'!
renderFields
	super renderFields.
	self renderSyntaxHelp.
	self renderDocument.! !

!SWPageEdit methodsFor: 'utilities' stamp: 'chbu 11/19/2003 19:32'!
shouldSave
	^super shouldSave
		and: [ document notNil ]! !

!SWPageEdit methodsFor: 'accessing' stamp: 'chbu 11/19/2003 19:38'!
wiki: aString
	[
		[ document := SWWikiParser parse: (wiki := aString copyWithout: Character lf) readStream for: self
		]	on: SmaCCParserError
			do: [ :error | error return: (self exception: error) ]
	]	on: SWUnauthorizedError
		do: [ :error | self exception: error ]! !

!SWPageEdit methodsFor: 'utilities' stamp: ''!
privateSave
	super privateSave.
	structure document: document.! !

!SWPageEdit methodsFor: 'accessing' stamp: ''!
document: aDocument
	document := aDocument! !

!SWPageEdit methodsFor: 'accessing' stamp: ''!
document
	document isNil
		ifTrue: [ document := structure document ].
	^document! !

!SWPageEdit methodsFor: 'rendering' stamp: ''!
renderDocument
	html tableRow: [
		html attributeAt: #valign put: #top.
		html tableHeading: 'Document:'.
		html tableData: [
			html attributeAt: 'wrap' put: 'virtual'.
			html attributeAt: 'rows' put: '20'.
			html attributeAt: 'style' put: 'width: 100%'.
			html textAreaWithValue: self wiki callback: #wiki: ] ].! !

!SWEditAction methodsFor: 'accessing' stamp: ''!
title
	title isNil
		ifTrue: [ title := structure title ].
	^title! !

!SWEditAction methodsFor: 'action' stamp: ''!
executePermission
	self assertPermission: structure class permissionEdit.! !

!SWEditAction methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:21'!
exception
	^exception! !

SWEditAction subclass: #SWResourceEdit
	instanceVariableNames: 'data mime embed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWResourceEdit class methodsFor: 'accessing' stamp: ''!
title
	^'Edit'! !

!SWResourceEdit methodsFor: 'accessing' stamp: ''!
mime
	mime isNil ifTrue: [ 
		mime := structure mimetype.
		mime isEmpty ifTrue: [
			mime := 'unknown' ] ].
	^mime! !

!SWResourceEdit methodsFor: 'rendering' stamp: 'svp 11/10/2003 15:54'!
renderFields
	super renderFields.
	html tableRow: [
 		html tableHeading: 'Data:'.
		html tableData: [ html fileUploadWithCallback: #data:mime: ] ].
	html tableRow: [
		html tableHeading: 'Embedding:'.
		html tableData: [ 
			html label: [
				html checkboxWithValue: structure isEmbedded callback: #embed:.
				html text: 'Enabled' ] ] ].! !

!SWResourceEdit methodsFor: 'rendering' stamp: ''!
renderForm: aBlock
	html formMultipart: aBlock.! !

!SWResourceEdit methodsFor: 'configuration' stamp: ''!
defaultTarget
	^request
		fieldAt: 'target'
		ifAbsent: [ super defaultTarget ]! !

!SWResourceEdit methodsFor: 'accessing' stamp: ''!
data
	^data! !

!SWResourceEdit methodsFor: 'accessing' stamp: ''!
embed
	embed isNil
		ifTrue: [ embed := false ].
	^embed! !

!SWResourceEdit methodsFor: 'accessing' stamp: ''!
embed: aBoolean
	embed := aBoolean! !

!SWResourceEdit methodsFor: 'rendering' stamp: ''!
renderSummary
	super renderSummary.
	html tableRow: [
		html tableHeading: [ html text: 'Size:' ].
		html tableData: [ html text: structure data size printString , ' Byte' ] ].
	html tableRow: [
		html tableHeading: [ html text: 'Mimetype:' ].
		html tableData: [ html text: self mime ] ].! !

!SWResourceEdit methodsFor: 'accessing' stamp: 'svp 11/10/2003 16:10'!
data: aDataString mime: aMimeString

	self data: aDataString.
	self mime: aMimeString.! !

!SWResourceEdit methodsFor: 'accessing' stamp: ''!
mime: aString
	mime := aString! !

!SWResourceEdit methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:21'!
hasData
	^data notNil and: [ data notEmpty ]! !

!SWResourceEdit methodsFor: 'accessing' stamp: ''!
data: aString
	data := aString! !

!SWResourceEdit methodsFor: 'utilities' stamp: 'chbu 11/19/2003 19:30'!
privateSave
	super privateSave.
	structure embed: self embed.
	self hasData ifTrue: [
		structure mimetype: self mime.
		structure data: self data ].! !

!SWEditAction methodsFor: 'utilities' stamp: 'chbu 11/19/2003 19:37'!
privateSave! !

!SWAction class methodsFor: 'accessing' stamp: ''!
urlFor: aStructure
	^aStructure url, '?action=', self name! !

!SWAction class methodsFor: 'accessing' stamp: ''!
title
	"Return a short string describing the action."

	^nil! !

!SWAction methodsFor: 'accessing' stamp: ''!
request
	^request! !

SWAction subclass: #SWHistoryAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWHistoryAction methodsFor: 'rendering' stamp: ''!
renderContent
	structure versions 
		do: [ :item | 
			html table: [ self renderVersion: item ].
			html paragraph: [ self renderVersionButtons: item version ] ] 
		separatedBy: [ html horizontalRule ]! !

!SWHistoryAction methodsFor: 'utilities' stamp: ''!
revert: anInteger
	self structure versionRevert: anInteger.
	self changed: #version.! !

!SWHistoryAction methodsFor: 'action' stamp: ''!
executePermission
	self assertPermission: structure class permissionHistory.! !

SWHistoryAction subclass: #SWPageHistory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWPageHistory class methodsFor: 'accessing' stamp: ''!
title
	^'History'! !

!SWPageHistory methodsFor: 'rendering' stamp: ''!
renderVersion: aStructure
	super renderVersion: aStructure.
	html tableRow: [
		html attributeAt: #valign put: #top.
		html tableHeading: 'Document:'.
		html tableData: [ self renderDocument: aStructure ] ]! !

!SWPageHistory methodsFor: 'rendering' stamp: 'chbu 10/22/2003 17:20'!
renderDocument: aStructure
	| full contracted |
	full := (SWVisitorRendererWiki render: aStructure) contents.
	contracted := String streamContents: [ :stream |
		stream nextPutAll: (full copyFrom: 1 to: (256 min: full size)).
		stream nextPutAll: '...' ].
 	html text: contracted! !

!SWHistoryAction methodsFor: 'utilities' stamp: ''!
restore: anInteger
	self structure versionRestore: anInteger.
	self changed: #version.! !

!SWHistoryAction methodsFor: 'rendering' stamp: ''!
renderVersionButtons: anInteger 
	html anchorWithAction: [ :action | action restore: anInteger ] to: self url do: 'Restore'; space.
	html anchorWithAction: [ :action | action revert: anInteger ] to: self url do: 'Revert'; space.
	html anchorWithAction: [ :action | action truncate: anInteger ] to: self url do: 'Truncate'; space! !

SWHistoryAction subclass: #SWResourceHistory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWResourceHistory class methodsFor: 'accessing' stamp: ''!
title
	^'History'! !

!SWResourceHistory methodsFor: 'rendering' stamp: ''!
renderVersion: aStructure
	super renderVersion: aStructure.
	html tableRow: [
		html tableHeading: 'Size:'.
		html tableData: aStructure data size printString, ' Byte' ].
	html tableRow: [
		html tableHeading: 'Mimetype:'.
		html tableData: aStructure mimetype ].! !

!SWHistoryAction methodsFor: 'rendering' stamp: 'chbu 11/12/2003 20:07'!
renderVersion: aStructure
	html tableRow: [
		html tableHeading: 'Title:'.
		html tableData: aStructure title ].
	html tableRow: [
		html tableHeading: 'Modified:'.
		html tableData: aStructure timestamp asString ].
	html tableRow: [
		html tableHeading: 'Version:'.
		html tableData: aStructure version ]! !

!SWHistoryAction methodsFor: 'utilities' stamp: ''!
truncate: anInteger
	self structure versionTruncate: anInteger.
	self changed: #version.! !

!SWAction methodsFor: 'action' stamp: ''!
executePermission
	"Override this message to check permission before anything inside this action is 	
	executed. By default an action might be used by all users, so no permissions are
	asserted."! !

!SWAction methodsFor: 'rendering' stamp: ''!
renderContent
	"Override this message to customize the output of this action. Do not change
	the state of the component in this message."! !

!SWAction methodsFor: 'notification' stamp: ''!
changed: aSymbol
	self changed: aSymbol with: self! !

!SWAction methodsFor: 'accessing' stamp: ''!
structure
	^structure! !

!SWAction methodsFor: 'accessing' stamp: ''!
server
	^server! !

!SWAction methodsFor: 'accessing' stamp: ''!
response
	^response! !

!SWAction methodsFor: 'rendering' stamp: ''!
renderHead
	self structure templates do: [ :each |
		each renderHeadWith: self on: html ]! !

!SWAction methodsFor: 'accessing-heading' stamp: ''!
heading
	"Return the full heading of the receiver, containing the title of the receiver and
	the one of the structure to be handled. Do not override this message, instead 
	have a look at #headingAction and #headingStructure."

	^String streamContents: [ :stream |
		self headingAction isNil ifFalse: [ 
			stream nextPutAll: self headingAction.
			self headingStructure isNil
				ifFalse: [ stream space ] ].
 		self headingStructure isNil
			ifFalse: [ stream nextPutAll: self headingStructure ] ]! !

!SWAction methodsFor: 'accessing' stamp: ''!
html
	^html! !

!SWAction methodsFor: 'accessing-heading' stamp: ''!
headingStructure
	"Return the title of the current structure to be used in the heading. Do override 
	this message, if you want to provide something different. This might return nil,
	if it is not appropriate."

	^structure isNil 
		ifTrue: [ nil ]
		ifFalse: [ structure title ]! !

!SWAction class methodsFor: 'class initialization' stamp: ''!
initialize! !

SWAction subclass: #SWSessionAction
	instanceVariableNames: 'target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWSessionAction methodsFor: 'accessing' stamp: ''!
target: anUrl
	target := anUrl! !

SWSessionAction subclass: #SWLogout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWLogout methodsFor: 'accessing' stamp: ''!
heading
	^self class title , ' from ' , self structure root title! !

!SWLogout methodsFor: 'action' stamp: ''!
execute
	self executeCallback.
	self executeLogout.! !

!SWLogout methodsFor: 'action' stamp: ''!
executeLogout
	self response cookieRemove: 'authorization'.
	self response redirectTo: self target.
	self changed: #logout.! !

!SWLogout class methodsFor: 'accessing' stamp: ''!
title
	^'Logout'! !

!SWSessionAction methodsFor: 'rendering' stamp: ''!
renderForm: aBlock
	super renderForm: [
		html hiddenInputWithValue: self target callback: #target:.
		aBlock value ]! !

SWSessionAction subclass: #SWLogin
	instanceVariableNames: 'username password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWLogin methodsFor: 'accessing' stamp: ''!
username
	username isNil
		ifTrue: [ username := String new ].
	^username! !

!SWLogin methodsFor: 'rendering' stamp: ''!
renderForm
	self renderForm: [
		html table: [
			html tableRow: [
				html tableHeading: 'Username:'.
				html tableData: [ html textInputWithValue: self username callback: #username: ] ].
			html tableRow: [
				html tableHeading: 'Password:'.
				html tableData: [ html passwordInputWithValue: String new callback: #password: ] ].
			html tableRow: [
				html tableHeading: nil.
				html tableData: [ html submitButton: 'Login' ] ] ] ].! !

!SWLogin class methodsFor: 'accessing' stamp: ''!
title
	^'Login'! !

!SWLogin methodsFor: 'accessing' stamp: ''!
password
	^password! !

!SWLogin methodsFor: 'action' stamp: ''!
executeLogin
	self response cookieAt: 'authorization' put: self authentication.
	self response redirectTo: self target.
	self changed: #login.! !

!SWLogin methodsFor: 'testing' stamp: ''!
isInvalid
	^self isSubmitted 
		and: [ self isAccepted not ]! !

!SWLogin methodsFor: 'accessing' stamp: ''!
password: aString
	password := aString! !

!SWLogin methodsFor: 'accessing' stamp: ''!
heading
	^self class title , ' to ' , self structure root title! !

!SWLogin methodsFor: 'action' stamp: 'svp 11/8/2003 14:26'!
execute

	self executeCallback.
	self isValid
		ifTrue: [ self executeLogin ]
		ifFalse: [ self render ].! !

!SWLogin methodsFor: 'action' stamp: ''!
authentication
	| stream |
	stream := WriteStream on: String new.
	stream nextPutAll: self username; nextPut: $:; nextPutAll: self password.
	^stream contents! !

!SWLogin methodsFor: 'rendering' stamp: ''!
renderInvalid
	html paragraph: 'There were errors with your submission. The username and password you entered ', 
		'did not match any accounts in our file. Please try again.'! !

!SWLogin methodsFor: 'rendering' stamp: ''!
renderContent
	self isInvalid 
		ifTrue: [ self renderInvalid ].
	self renderForm.! !

!SWLogin methodsFor: 'testing' stamp: ''!
isSubmitted
	^self username notNil 
		and: [ self password notNil ]! !

!SWLogin methodsFor: 'accessing' stamp: ''!
username: aString
	username := aString! !

!SWLogin methodsFor: 'testing' stamp: ''!
isValid
	^self isSubmitted 
		and: [ self isAccepted ]! !

!SWLogin methodsFor: 'testing' stamp: ''!
isAccepted
	^(request server userAt: self username ifAbsent: [ ^false ]) 
		validatePassword: self password! !

!SWSessionAction methodsFor: 'accessing' stamp: ''!
target
	target isNil
		ifTrue: [ target := structure url ].
	^target! !

!SWAction methodsFor: 'callback' stamp: 'chbu 10/27/2003 22:17'!
performCallback: aBlockOrSymbol withArguments: anArray
	^aBlockOrSymbol isSymbol
		ifTrue: [ self perform: aBlockOrSymbol withArguments: (anArray copyFrom: 2 to: aBlockOrSymbol numArgs + 1) ]
		ifFalse: [ aBlockOrSymbol valueWithArguments: (anArray copyFrom: 1 to: aBlockOrSymbol numArgs) ]! !

SWAction subclass: #SWViewAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWViewAction methodsFor: 'testing' stamp: ''!
isIndexable
	"All the view actions should be indexed by search-engines."

	^true! !

!SWViewAction methodsFor: 'action' stamp: ''!
executePermission
	self assertPermission: structure class permissionView.! !

!SWViewAction methodsFor: 'accessing-heading' stamp: ''!
headingAction
	^nil! !

SWViewAction subclass: #SWPageView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWPageView class methodsFor: 'accessing' stamp: ''!
title
	^'View'! !

!SWPageView methodsFor: 'rendering' stamp: ''!
renderContent
	self structure defaultVisitor 
		render: self structure document with: self on: html! !

SWViewAction subclass: #SWMimeView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWMimeView methodsFor: 'rendering' stamp: ''!
render
	response type: structure mimetype.
	response stream: structure stream.! !

!SWAction methodsFor: 'rendering' stamp: ''!
render
	"This message starts the basic html rendering of a page. Unless you do not want the 
	templates and the html addendum to be rendered, do not override this message."

	html doctype: '"-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"'.
	html attributeAt: 'xmlns' put: 'http://www.w3.org/1999/xhtml'.
	html attributeAt: 'xml:lang' put: 'en'.
	html html: [
		html head: [ self renderHead ].
		html body: [ self renderBody ] ]! !

SWAction subclass: #SWRecentChanges
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWRecentChanges methodsFor: 'rendering' stamp: 'chbu 11/19/2003 19:31'!
renderContent
	| changes keys |
	changes := OrderedCollection new.
	SWVisitorStructure
		block: [ :each | changes addAll: each versions ]
		start: structure.
	changes := changes 
		groupBy: [ :item | item timestamp asDate ] having: [:a | true].
	keys := changes keys
		asSortedCollection: [ :date1 :date2 | date1 > date2 ].
	keys := keys copyFrom: 1 to: (10 min: keys size).
	keys do: [ :key |
		self renderDate: key changes: (changes at: key) ]! !

!SWRecentChanges methodsFor: 'accessing-heading' stamp: ''!
headingStructure
	^'to ' , super headingStructure! !

!SWRecentChanges methodsFor: 'rendering' stamp: ''!
renderDate: aDate changes: aCollection 
	| structures |
	structures := aCollection 
				asSortedCollection: [:item1 :item2 | item1 timestamp < item2 timestamp].
	html heading: aDate printString level: 2.
	html unorderedList: 
			[structures do: 
					[:item | 
					html listItem: 
							[html render: item timestamp asTime.
							html render: ' - '.
							html anchorWithUrl: item url do: item title]]]! !

!SWRecentChanges class methodsFor: 'accessing' stamp: ''!
title
	^'Changes'! !

!SWAction methodsFor: 'initialization' stamp: ''!
initialize! !

!SWAction class methodsFor: 'instance creation' stamp: ''!
request: aRequest structure: aStructure
	^self new
		request: aRequest;
		structure: aStructure;
		yourself! !

!SWAction methodsFor: 'callback' stamp: 'chbu 11/12/2003 20:07'!
processFormCallback
	| keys object value mime |

	keys := request fields keys 
		select: [ :key | ('form-callback-' charactersExactlyMatching: key) = 14 ].
	keys := keys collect: [ :key | (key copyFrom: 15 to: key size) asNumber ].
	keys asSortedCollection do: [ :key | 
		object := self request server callback get: key.
		object isNil ifFalse: [
			value := request fieldAt: 'form-callback-' , key asString ifAbsent: [ nil ].
			mime := request fieldAt: 'mime-form-callback-' , key asString ifAbsent: [ String new ].
			self performCallback: object withArguments: (Array with: self with: value with: mime) ] ].
	^keys notEmpty! !

!SWAction methodsFor: 'notification' stamp: ''!
changed: aSymbol with: anObject
	structure changed: aSymbol with: anObject.! !

!SWAction methodsFor: 'action' stamp: ''!
executeCallback
	"Override this message to provide your own way of evaluating callbacks. This implementation
	only executes the anchor-callback when there are no form-callbacks being executed. This 
	prevents from accidently executing form and anchor callbacks at the same time, what is usually
	not intended."

	self processFormCallback
		ifFalse: [ self processAnchorCallback ]! !

!SWAction methodsFor: 'accessing-heading' stamp: ''!
headingAction
	"Return the title of the reveiver to be used in the heading. Do override this
	message, if you want to provide something different. This might return nil,
	if it is not appropriate."

	^self class title! !

!SWAction methodsFor: 'testing' stamp: ''!
isIndexable
	"Override this message and return true to tell search-engines to index the 
	contents of this actions."

	^false! !

!SWAction methodsFor: 'security' stamp: ''!
hasPermission: aPermission 
	^self user hasPermission: aPermission! !

!SWAction methodsFor: 'security' stamp: ''!
assertPermission: aPermission
	"Assert the presence of a Permission in the current session. If no such permission
	is present an \texttt{UnauthorizedError} is thrown and an error page will be renderered
	instead of the one of the current action."

	self user assertPermission: aPermission! !

!SWAction methodsFor: 'tools' stamp: ''!
createChildFromClass: aClass titled: aString
	self createChildFromClass: aClass titled: aString creator: nil! !

!SWAction methodsFor: 'rendering' stamp: ''!
renderForm: aBlock	
	html form: [
		"html hidden: self class name named: 'action'.	"
		aBlock value ]! !

!SWAction class methodsFor: 'instance creation' stamp: ''!
new
	^super new
		initialize;
		yourself! !

!SWAction methodsFor: 'rendering' stamp: ''!
renderBody
	html divNamed: #container with: [
		self structure templates do: [ :each |
			each renderBodyWith: self on: html ] ]! !

!SWAction methodsFor: 'testing' stamp: ''!
shouldRender
	^response isRedirect not! !

SWAction subclass: #SWErrorAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWErrorAction methodsFor: 'rendering' stamp: ''!
renderContent
	self renderProblem.
	self renderSolution.! !

SWErrorAction subclass: #SWErrorUnauthorized
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWErrorUnauthorized methodsFor: 'rendering' stamp: ''!
renderProblem
	html paragraph: [
		html text: 'You are not authorized to execute the requested action on '.
		html emphasis: request url.
		html text: '.' ]! !

!SWErrorUnauthorized class methodsFor: 'accessing' stamp: ''!
title
	^'Unauthorized'! !

!SWErrorUnauthorized methodsFor: 'rendering' stamp: 'chbu 10/22/2003 17:10'!
renderSolution
	html paragraph: [
		html text: 'You can try one of the following options:'.
		html unorderedList: [
			html listItem: [
				html anchorWithUrl: (SWLogin urlFor: structure) do: 'Logging in'; text: ' with your username and password.'].
			html listItem: [
				html text: 'Jump to the '; anchorWithUrl: self root url do: self root title; text: ' homepage.' ] ] ]! !

!SWErrorAction methodsFor: 'rendering' stamp: ''!
renderProblem! !

!SWErrorAction class methodsFor: 'testing' stamp: ''!
isListable
	^false! !

!SWErrorAction methodsFor: 'accessing-heading' stamp: ''!
headingStructure
	^nil! !

!SWErrorAction methodsFor: 'action' stamp: ''!
executeCallback
	"We do not want any callback to be processed!!"! !

!SWErrorAction methodsFor: 'utilities' stamp: ''!
root
	^structure root! !

!SWErrorAction methodsFor: 'rendering' stamp: ''!
renderSolution! !

SWErrorAction subclass: #SWErrorNotFound
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWErrorNotFound methodsFor: 'rendering' stamp: ''!
renderProblem
	html paragraph: [
		html text: 'The requested object '.
		html emphasis: request url.
		html text: ' was not found on this server.' ]! !

!SWErrorNotFound class methodsFor: 'accessing' stamp: ''!
title
	^'Not Found'! !

!SWErrorNotFound methodsFor: 'rendering' stamp: ''!
renderSolution
	html paragraph: [
		html text: 'You can try one of the these options:'.
		html unorderedList: [
			html listItem: 'Click on a shortcut in the menu.'.
			html listItem: [ html text: 'Jump to the '; anchorWithUrl: self root url do: self root title; text: ' homepage.'].
			html listItem: 'If you typed the page address, make sure that it is spelled correctly.'.
			html listItem: 'If you clicked a link from another page, please inform the webmaster of this site of his mistake.' ] ]! !

!SWAction methodsFor: 'action' stamp: ''!
execute
	"Usually it is not necessary to override this message, instead use the provided callback 
	mechanism. Still there are rare cases where you need to have full control over the 
	execution process; but do not generate any output in here, use #renderContent
	instead. The rendering is called automatically, if there hasn't been a redirect response 
	created while checking the permissions or while executing the callbacks."

	self executePermission.
	self executeCallback.
	self shouldRender
		ifTrue: [ self render ].! !

SWAction subclass: #SWInvisibleAction
	instanceVariableNames: 'redirect'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWInvisibleAction methodsFor: 'accessing' stamp: ''!
redirect: aString
	redirect := aString! !

!SWInvisibleAction methodsFor: 'utilites' stamp: ''!
redirectTo: anUrl
	self redirect isEmpty
		ifFalse: [ ^response redirectTo: self redirect ].
	(anUrl isNil or: [ anUrl isEmpty ])
		ifFalse: [ ^response redirectTo: anUrl ].
	response redirectTo: structure url.! !

!SWInvisibleAction methodsFor: 'accessing' stamp: ''!
redirect
	redirect isNil
		ifTrue: [ redirect := String new ].
	^redirect! !

SWInvisibleAction subclass: #SWRedirectAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWRedirectAction methodsFor: 'accessing' stamp: ''!
target
	self subclassResponsibility! !

SWRedirectAction subclass: #SWParentStructure
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWParentStructure methodsFor: 'accessing' stamp: ''!
target
	^structure parent! !

!SWParentStructure class methodsFor: 'accessing' stamp: ''!
title
	^'Parent'! !

!SWRedirectAction methodsFor: 'testing' stamp: ''!
isEnabled
	^self target notNil! !

SWRedirectAction subclass: #SWPreviousStructure
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWPreviousStructure methodsFor: 'accessing' stamp: ''!
target
	^structure previous! !

!SWPreviousStructure class methodsFor: 'accessing' stamp: ''!
title
	^'Prevoius'! !

SWRedirectAction subclass: #SWNextStructure
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWNextStructure methodsFor: 'accessing' stamp: ''!
target
	^structure next! !

!SWNextStructure class methodsFor: 'accessing' stamp: ''!
title
	^'Next'! !

!SWRedirectAction methodsFor: 'action' stamp: ''!
execute
	self isEnabled
		ifTrue: [ response redirectTo: self target url ]
		ifFalse: [ response redirectTo: self structure url ]! !

!SWInvisibleAction methodsFor: 'rendering' stamp: ''!
render
	"We do not want anything to be rendered as we are invisible, so we override with 
	an empty message here. We assume that subclasses do a redirect somewhere
	after they performed an action."! !

SWAction subclass: #SWSearch
	instanceVariableNames: 'expression results root ignoreCase'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWSearch methodsFor: 'accessing' stamp: ''!
results: aCollection
	results := aCollection! !

!SWSearch methodsFor: 'accessing-calculated' stamp: ''!
startStructure
	^self root 
		ifTrue: [ structure root ]
		ifFalse: [ structure ]! !

!SWSearch methodsFor: 'rendering' stamp: ''!
renderButton
	html tableRowWith: nil with: [
		html submitButtonWithAction: #executeSearch text: 'search' ]! !

!SWSearch methodsFor: 'initialization' stamp: 'chbu 10/27/2003 22:11'!
initialize
	super initialize.
	self expression: String new.
	self results: Array new.
	self ignoreCase: true.
	self root: false.! !

!SWSearch class methodsFor: 'accessing' stamp: ''!
title
	^'Search'! !

!SWSearch methodsFor: 'accessing' stamp: ''!
results
	^results! !

!SWSearch methodsFor: 'rendering' stamp: ''!
renderContent
	self renderForm: [
		html table: [
			self renderExpression.
			self renderOptions.
			self renderButton ] ].
	self results isEmpty ifFalse: [
		self renderResults ].! !

!SWSearch methodsFor: 'accessing' stamp: ''!
root
	^root! !

!SWSearch methodsFor: 'accessing-calculated' stamp: 'chbu 10/27/2003 22:03'!
fullExpression
	^String streamContents: [ :stream |
		stream nextPut: $*; nextPutAll: self expression withBlanksTrimmed; nextPut: $* ]! !

!SWSearch methodsFor: 'rendering' stamp: ''!
renderResults
	html orderedList: [
		self results do: [ :each |
			html listItem: [
				html anchorWithUrl: each url do: each title ] ] ]! !

!SWSearch methodsFor: 'rendering' stamp: ''!
renderExpression
	html tableRow: [
		html attributeAt: #width: put: 100.
		html tableHeading: 'Search for:'.
		html tableData: [
			html attributeAt: #size put: 50.
			html textInputWithValue: self expression callback: #expression: ] ]! !

!SWSearch methodsFor: 'accessing' stamp: ''!
ignoreCase
	^ignoreCase! !

!SWSearch methodsFor: 'accessing' stamp: ''!
ignoreCase: aBoolean
	ignoreCase := aBoolean! !

!SWSearch methodsFor: 'accessing' stamp: 'chbu 10/27/2003 22:18'!
expression: aString
	expression := aString! !

!SWSearch methodsFor: 'rendering' stamp: ''!
renderOptions
	html tableRow: [
		html attributeAt: #valign put: #top.
		html tableHeading: 'Options:'.
		html tableData: [
			html label: [
				html checkboxWithValue: self ignoreCase callback: #ignoreCase:.
				html text: 'Ignore character case' ].
			html break.
			html label: [
				html checkboxWithValue: self root callback: #root:.
				html text: 'Start in the root-folder' ] ] ]! !

!SWSearch methodsFor: 'accessing-heading' stamp: ''!
headingStructure
	^'in ' , super headingStructure! !

!SWSearch methodsFor: 'execute' stamp: 'chbu 11/19/2003 19:29'!
executeSearch
	results := self expression isEmpty
		ifTrue: [ Array new ]
		ifFalse: [ self performSearch ]! !

!SWSearch methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:20'!
performSearch	
	| collection |
	collection := Set new.
	SWVisitorText
		block: [ :string :parent |
			(self fullExpression match: string "ignoreCase: self ignoreCase")
				ifTrue: [ collection add: parent ] ]
		start: self startStructure.
	^collection! !

!SWSearch methodsFor: 'accessing' stamp: ''!
root: aBoolean
	root := aBoolean! !

!SWSearch methodsFor: 'accessing' stamp: ''!
expression
	^expression! !

!SWAction methodsFor: 'notification' stamp: ''!
changed
	self changed: #action! !

!SWAction methodsFor: 'callback' stamp: 'chbu 10/27/2003 22:18'!
processAnchorCallback
	| key blockOrSymbol |
	key := request fieldAt: 'callback'.
	key isNil ifFalse: [
		blockOrSymbol := self request server callback get: key asNumber.
		blockOrSymbol isNil ifFalse: [
			self performCallback: blockOrSymbol withArguments: (Array with: self) ] ].! !

SWAction subclass: #SWFolderEdit
	instanceVariableNames: 'name type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWFolderEdit methodsFor: 'tools' stamp: 'chbu 10/22/2003 17:13'!
filteredStructures
	^SWStructure allStructures
		select: [ :class | self hasPermission: class permissionAdd ]! !

!SWFolderEdit methodsFor: 'utilities' stamp: ''!
moveUp: aStructure
	self assertPermission: self structure class permissionMove.
	self structure children moveUp: aStructure ifError: [ ^self ].
	self changed: #move.! !

!SWFolderEdit methodsFor: 'utilities' stamp: ''!
remove: aStructure
	self assertPermission: aStructure class permissionRemove.
	self structure nextVersion.
	self structure remove: aStructure.
	self changed: #remove.! !

!SWFolderEdit methodsFor: 'accessing' stamp: 'chbu 10/22/2003 17:13'!
type
	type isNil
		ifTrue: [ type := SWPage ].
	^type! !

!SWFolderEdit methodsFor: 'rendering' stamp: ''!
renderAdd: aCollection
	self renderForm: [
		html table: [
			html tableRow: [
				html tableHeading: 'Title:'.
				html tableData: [ html textInputWithValue: self name callback: #name: ] ].
			html tableRow: [
				html tableHeading: 'Type:'.
				html tableData: [ 
					html selectFromList: self filteredStructures selected: self type callback: #type:.
					html submitButtonWithAction: #add text: 'Add'  ] ] ] ]! !

!SWFolderEdit class methodsFor: 'accessing' stamp: ''!
title
	^'Contents'! !

!SWFolderEdit methodsFor: 'utilities' stamp: ''!
moveDown: aStructure
	self assertPermission: self structure class permissionMove.
	self structure children moveDown: aStructure ifError: [ ^self ].
	self changed: #move.! !

!SWFolderEdit methodsFor: 'utilities' stamp: ''!
add
	| child |
	self assertPermission: self type permissionAdd.
	self name isEmpty
		ifTrue: [ self name: self type title ].
	self name: (self structure uniqueTitle: self name).
	child := self type title: self name.
	self structure 
		nextVersion;
		add: child;
		changed: #add! !

!SWFolderEdit methodsFor: 'accessing' stamp: 'chbu 10/27/2003 22:00'!
name: aString
	name := aString withBlanksTrimmed! !

!SWFolderEdit methodsFor: 'rendering' stamp: ''!
renderChildrenPosition: aStructure 
	(self hasPermission: aStructure class permissionMove) 
		ifTrue: [
			html anchorWithAction: [ :action | action moveUp: aStructure ] to: self url do: 'up'; space.
			html anchorWithAction: [ :action | action moveDown: aStructure ] to: self url do: 'down' ]
		ifFalse: [ html text: 'up down' ]! !

!SWFolderEdit methodsFor: 'accessing' stamp: ''!
type: aClass
	type := aClass! !

!SWFolderEdit methodsFor: 'utilities' stamp: ''!
copy: aStructure
	self assertPermission: aStructure class permissionCopy.
	self structure nextVersion.
	self structure copy: aStructure.
	self changed: #add.! !

!SWFolderEdit methodsFor: 'rendering' stamp: ''!
renderContent
	| children structures |
	structures := self filteredStructures.
	structures isEmpty ifFalse: [
		html heading: 'Add Children' level: 2.
		self renderAdd: structures. ].
	children := self structure children.
	children isEmpty ifFalse: [
		html heading: 'Edit Children' level: 2.
		self renderChildren: children ].! !

!SWFolderEdit methodsFor: 'accessing' stamp: ''!
name
	name isNil
		ifTrue: [ name := String new ].
	^name! !

!SWFolderEdit methodsFor: 'rendering' stamp: 'chbu 11/12/2003 20:07'!
renderChildren: aCollection 
	html attributeAt: 'width' put: '100%'.
	html table: [
		html tableRow: [
			html tableHeading: 'Title'.
			html tableHeading: 'Type'.
			html tableHeading: 'Modified'.
			html tableHeading: 'Version'.
			html tableHeading: 'Position'.
			html tableHeading: 'Commands' ].
		aCollection do: [ :child | 
			html tableRow: [
				html tableData: [
					(self hasPermission: child class permissionView) 
						ifTrue: [ html anchorWithUrl: child url do: child title ]
						ifFalse: [ html text: child title ] ].
				html tableData: child class title.
				html tableData: child timestamp asString.
				html tableData: child version asString.
				html tableData: [ self renderChildrenPosition: child ].
				html tableData: [ self renderChildrenCommands: child ] ] ] ]! !

!SWFolderEdit methodsFor: 'rendering' stamp: ''!
renderChildrenCommands: aStructure 
	(self hasPermission: aStructure class permissionCopy) 
		ifTrue: [ html anchorWithAction: [ :action | action copy: aStructure ] to: self url do: 'copy'; space ]
		ifFalse: [ html text: 'copy'; space ].
	(self hasPermission: aStructure class permissionRemove) 
		ifTrue: [ html anchorWithAction: [ :action | action remove: aStructure ] to: self url do: 'remove'; space ]
		ifFalse: [ html text: 'remove'; space ]! !

SWAction subclass: #SWTemplateEdit
	instanceVariableNames: 'state selectedAvailable selectedChoosen'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Action'!

!SWTemplateEdit methodsFor: 'accessing' stamp: ''!
selectedAvailable: aClass
	selectedAvailable := aClass! !

!SWTemplateEdit methodsFor: 'rendering' stamp: ''!
renderIntro
	self hasLocalSettings
		ifTrue: [ self renderIntroLocal ]
		ifFalse: [ self renderIntroNonLocal ]! !

!SWTemplateEdit methodsFor: 'as yet unclassified' stamp: 'svp 11/10/2003 16:33'!
repositoryFrom: anUrlString 

	^[anUrlString asUrl retrieveContents contents] 
		on: Error
		do: [ :err | String new]! !

!SWTemplateEdit methodsFor: 'accessing-state' stamp: ''!
state: aSymbol
	state := aSymbol asSymbol! !

!SWTemplateEdit methodsFor: 'actions' stamp: ''!
templateDown
	| template other |
	template := self selectedChoosen.
	template isNil ifFalse: [
		self structure templates last == template ifFalse: [
			other := self structure templates after: template.
			self structure templates remove: template.
			self structure templates add: template after: other ] ]! !

!SWTemplateEdit methodsFor: 'as yet unclassified' stamp: 'chbu 10/22/2003 09:24'!
settingsFrom
	^self structure parents reverse 
		detect: [ :each | each localProperties includes: #internalTemplates ]
		ifNone: [ self structure root ]! !

!SWTemplateEdit methodsFor: 'rendering' stamp: ''!
renderForm: aBlock
	super renderForm: [
		html hiddenInputWithValue: self state callback: #state:.
		aBlock value ]! !

!SWTemplateEdit methodsFor: 'accessing-state' stamp: ''!
stateSettings
	^#renderSettings! !

!SWTemplateEdit methodsFor: 'accessing-state' stamp: ''!
state
	state isNil
		ifTrue: [ state := self stateTemplates ].
	^state! !

!SWTemplateEdit methodsFor: 'rendering' stamp: ''!
renderIntroLocal
	html paragraph: [ 
		html text: 'The current structure defines its own template and inherits it to all its children. To '.
		html text: 'get the parent template applied, you have to '.
		html anchorWithAction: #makeSettingsNonLocal to: self url do: 'remove'.
		html text: ' the current settings first.' ].
	self renderStates.! !

!SWTemplateEdit methodsFor: 'accessing-state' stamp: ''!
stateTemplates
	^#renderTemplates! !

!SWTemplateEdit methodsFor: 'accessing' stamp: 'chbu 10/22/2003 17:58'!
templatesAvailable
	| templates |
	templates := SWTemplate withAllSubclasses 
		reject: [ :each | each title isNil ].
	^templates 
		asSortedCollection: [ :x :y | x title <= y title ]! !

!SWTemplateEdit methodsFor: 'tools' stamp: ''!
shouldRemove: aTemplate
	^aTemplate notNil
		and: [ aTemplate class isRemoveable
			or: [ self structure templates contains: [ :each | 
				aTemplate ~= each and: [ aTemplate class = each class ] ] ] ]! !

!SWTemplateEdit methodsFor: 'accessing' stamp: ''!
selectedAvailable
	^selectedAvailable! !

!SWTemplateEdit methodsFor: 'repository' stamp: 'chbu 10/22/2003 17:58'!
repositoryIndex
	| result stream |
	result := OrderedCollection new.
	stream := (self repositoryFrom: self repositoryPath)
		readStream.
	[ stream atEnd ] whileFalse: [
		result add: (stream upTo: Character space) -> (stream upTo: Character lf) ].
	^result! !

!SWTemplateEdit methodsFor: 'rendering' stamp: ''!
renderStates
	html unorderedList: [
		html listItem: [ 
			html anchorWithAction: [ :action | action state: self stateTemplates ] to: self url do: 'Templates' ].
		html listItem: [
			html anchorWithAction: [ :action | action state: self stateSettings ] to: self url do: 'Settings' ].
		html listItem: [
			html anchorWithAction: [ :action | action state: self stateStylesheet ] to: self url do: 'Stylesheet' ] ]! !

!SWTemplateEdit methodsFor: 'accessing-state' stamp: ''!
stateStylesheet
	^#renderStylesheet! !

!SWTemplateEdit methodsFor: 'actions' stamp: ''!
makeSettingsLocal
	| copyied |
	copyied := self settingsFrom templates collect: [ :each | each copy ].
	structure templates: copyied.
	structure stylesheet: self settingsFrom stylesheet! !

!SWTemplateEdit methodsFor: 'actions' stamp: ''!
executePermission
	self assertPermission: structure class permissionTemplate.! !

!SWTemplateEdit methodsFor: 'actions' stamp: ''!
templateUp
	| template other |
	template := self selectedChoosen.
	template isNil ifFalse: [
		self structure templates first == template ifFalse: [
			other := self structure templates before: template.
			self structure templates remove: template.
			self structure templates add: template before: other ] ]! !

!SWTemplateEdit methodsFor: 'accessing' stamp: ''!
selectedChoosen: aTemplate
	selectedChoosen := aTemplate! !

!SWTemplateEdit methodsFor: 'actions' stamp: ''!
makeSettingsNonLocal
	structure templates: nil.
	structure stylesheet: nil.! !

!SWTemplateEdit methodsFor: 'rendering-pages' stamp: ''!
renderSettings
	html heading: 'Settings' level: 2.
	self renderForm: [
		html attributeAt: #width put: '100%'.
		html table: [
			self structure templates do: [ :each |
				each renderConfigWith: self on: html ].
		html tableRowWith: nil with: [ html submitButton: 'Save' ] ] ]! !

!SWTemplateEdit class methodsFor: 'accessing' stamp: ''!
title
	^'Template'! !

!SWTemplateEdit methodsFor: 'repository' stamp: ''!
repositoryPath
	^'http://www.iam.unibe.ch/~scg/smallwiki/index.txt'! !

!SWTemplateEdit methodsFor: 'actions' stamp: ''!
templateRemove
	| template |
	template := self selectedChoosen.
	(self shouldRemove: template)
		ifTrue: [ self structure templates remove: template ]! !

!SWTemplateEdit methodsFor: 'tools' stamp: ''!
hasLocalSettings
	^self structure == self settingsFrom! !

!SWTemplateEdit methodsFor: 'rendering' stamp: ''!
renderContent
	self renderIntro.
	self hasLocalSettings
		ifTrue: [ self perform: self state ]! !

!SWTemplateEdit methodsFor: 'rendering-pages' stamp: ''!
renderStylesheet
	| index |
	index := self repositoryIndex.
	html heading: 'Stylesheet' level: 2.
	index isEmpty ifFalse: [ 
		self renderForm: [
			html paragraph: [
				html 
					selectFromList: index selected: nil 
					callback: [ :action :association | structure stylesheet: '@import "' , association key , '";' ]
					labels: [ :association | association value ].
				html submitButton: 'Include' ] ] ].
	self renderForm: [
		html paragraph: [
			html 
				attributeAt: 'wrap' put: 'virtual';
				attributeAt: 'cols' put: '70';
				attributeAt: 'rows' put: '20'.
			html
				textAreaWithValue: self structure stylesheet 
				callback: [ :action :value | structure stylesheet:  value ].
			html paragraph: [ html submitButton: 'Save' ] ] ]! !

!SWTemplateEdit methodsFor: 'rendering-pages' stamp: ''!
renderTemplates
	html heading: 'Templates' level: 2.
	self renderForm: [
		html table: [
			html tableRow: [
				html tableData: [
					html attributeAt: #size put: 10.
					html
						selectFromList: self templatesAvailable
						selected: self selectedAvailable 
						callback: #selectedAvailable:
						labels: [ :each | each title ] ].
				html tableData: [
					html attributeAt: #size put: 10.
					html
						selectFromList: self structure templates 
						selected: self selectedChoosen 
						callback: #selectedChoosen:
						labels: [ :each | each class title ] ] ].
			html tableRow: [
				html attributeAt: #align put: #center.
				html attributeAt: #colspan put: 2.
				html tableData: [
					html submitButtonWithAction: #templateAdd text: 'add'.
					html submitButtonWithAction: #templateRemove text: 'remove'.
					html submitButtonWithAction: #templateUp text: 'up'.
					html submitButtonWithAction: #templateDown text: 'down' ] ] ] ]! !

!SWTemplateEdit methodsFor: 'accessing' stamp: ''!
selectedChoosen
	^selectedChoosen! !

!SWTemplateEdit methodsFor: 'rendering' stamp: ''!
renderIntroNonLocal
	html paragraph: [ 
		html text: 'The current structure does not define its own template, but inherits the one of '.
		html anchorWithUrl: (self class urlFor: self settingsFrom) do: self settingsFrom title.
		html text: '. To redefine this template at this place, you have to '.
		html anchorWithAction: #makeSettingsLocal to: self url do: 'load it'.
		html text: ' into the current structure first.' ]! !

!SWTemplateEdit methodsFor: 'actions' stamp: ''!
templateAdd
	| template |
	self selectedAvailable isNil ifFalse: [
		template := self selectedAvailable new.
		self structure templates add: template.
		self selectedChoosen: template ]! !

!SWAction methodsFor: 'accessing' stamp: ''!
url
	^self class urlFor: self structure! !

!SWAction methodsFor: 'tools' stamp: ''!
createChildFromClass: aClass titled: aString creator: aStructure
	| parent child url |
	self assertPermission: aClass permissionAdd.
	parent := self structure defaultAddTarget.
	child := aClass title: (parent uniqueTitle: aString).
	parent nextVersion; add: child; changed: #add.
	url := aClass defaultEditAction urlFor: child.
	aStructure isNil ifFalse: [
		url := url , '&target=' , aStructure url ].
	response redirectTo: url! !

!SWAction methodsFor: 'private' stamp: ''!
structure: aStructure
	structure := aStructure.! !

!SWAction class methodsFor: 'testing' stamp: ''!
isListable
	^self title notNil! !

!SWAction methodsFor: 'private' stamp: ''!
request: aRequest
	request := aRequest.
	response := aRequest response.
	server := aRequest server.
	html := response stream.! !

Object subclass: #SWRequest
	instanceVariableNames: 'username password user cookies fields headers url urlparsed response server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Server'!

!SWRequest methodsFor: 'private-accessing' stamp: ''!
server: aServer
	server := aServer! !

!SWRequest methodsFor: 'accessing-user' stamp: ''!
updateRoles: aCollection
	"Update the roles of the user performing the current request. The roles are given in 
	aCollection and might be nil, if nothing should be updated. See User>>updateRoles:
	for additional information."

	aCollection isNil
		ifFalse: [ user := user updateRoles: aCollection ].
	^user! !

!SWRequest methodsFor: 'accessing' stamp: ''!
server
	^server! !

!SWRequest class methodsFor: 'instance-creation' stamp: ''!
server: aServer
	^self new
		server: aServer;
		yourself! !

!SWRequest methodsFor: 'accessing-cookies' stamp: ''!
cookieAt: aKey ifAbsent: aBlock
	^cookies 
		at: aKey 
		ifAbsent: aBlock! !

!SWRequest methodsFor: 'private-accessing' stamp: ''!
user: anUser
	user := anUser! !

!SWRequest methodsFor: 'accessing-user' stamp: 'svp 11/10/2003 11:16'!
updateAuthentication

	(self headerIncludes: 'authorization') 
		ifTrue: [ self updateAuthenticationFromHeader ].
	(self cookieIncludes: 'authorization')
		ifTrue: [ self updateAuthenticationFromCookies ].
	self updateUser.! !

!SWRequest methodsFor: 'accessing-headers' stamp: ''!
headerAt: aKey ifAbsent: aBlock
	^headers 
		at: aKey asUppercase 
		ifAbsent: aBlock! !

!SWRequest methodsFor: 'accessing-fields' stamp: ''!
fieldIncludes: aKey
	^fields 
		includesKey: aKey! !

!SWRequest methodsFor: 'accessing-cookies' stamp: ''!
cookies: aDictionary
	cookies := aDictionary! !

!SWRequest methodsFor: 'accessing-fields' stamp: ''!
fieldAt: aKey
	^self 
		fieldAt: aKey 
		ifAbsent: [ nil ]! !

!SWRequest methodsFor: 'accessing-cookies' stamp: ''!
cookies
	^cookies! !

!SWRequest methodsFor: 'accessing-user' stamp: ''!
updateAuthenticationFromCookies
	| stream |
	username := password := nil.
	stream := (self cookieAt: 'authorization') readStream.
	username := stream upTo: $:.
	stream atEnd 
		ifFalse: [ password := stream upToEnd ]! !

!SWRequest methodsFor: 'private' stamp: 'chbu 10/23/2003 18:07'!
basicAuthentication: aString
	| readStream |
	readStream := (Base64MimeConverter mimeDecodeToChars: aString readStream) 
		upToEnd asString readStream.
	username := readStream upTo: $:.
	readStream atEnd 
		ifFalse: [ password := readStream upToEnd ]! !

!SWRequest methodsFor: 'accessing-fields' stamp: ''!
fields: aDictionary
	fields := aDictionary! !

!SWRequest methodsFor: 'private' stamp: ''!
updateUser
	user := self server userAt: self username.
	(user validatePassword: self password)
		ifFalse: [ user := self server userAnonymous ]! !

!SWRequest methodsFor: 'private-accessing' stamp: ''!
response: aResponse
	response := aResponse! !

!SWRequest methodsFor: 'accessing-headers' stamp: ''!
headers: aDictionary
	headers := aDictionary! !

!SWRequest methodsFor: 'accessing-user' stamp: ''!
updateAuthenticationFromHeader
	| readStream method |
	username := password := nil.
	readStream := (self headerAt: 'authorization') readStream.
	method := readStream upTo: $ .
	readStream atEnd ifFalse: [
		(method sameAs: 'basic') ifTrue: [ 
			self basicAuthentication: readStream upToEnd ] ]! !

!SWRequest methodsFor: 'accessing-fields' stamp: ''!
fieldAt: aKey ifAbsent: aBlock
	^fields 
		at: aKey 
		ifAbsent: aBlock! !

!SWRequest methodsFor: 'accessing-user' stamp: ''!
user
	user isNil 
		ifTrue: [ user := server userAnonymous ].
	^user! !

!SWRequest methodsFor: 'accessing-url' stamp: ''!
url: aString
	| stream part |
	url := aString.
	urlparsed := OrderedCollection new.
	stream := aString readStream.
	[ stream atEnd ] whileFalse: [
		part := stream upTo: $/.
		part isEmpty 
			ifFalse: [ urlparsed add: part ] ]! !

!SWRequest methodsFor: 'accessing-authentification' stamp: ''!
username
	^username! !

!SWRequest methodsFor: 'accessing-cookies' stamp: ''!
cookieIncludes: aKey
	^cookies 
		includesKey: aKey! !

!SWRequest methodsFor: 'accessing-url' stamp: ''!
urlAtEnd
	^urlparsed isEmpty! !

!SWRequest methodsFor: 'accessing-headers' stamp: ''!
headerAt: aKey
	^self 
		headerAt: aKey 
		ifAbsent: [ nil ]! !

!SWRequest methodsFor: 'accessing-authentification' stamp: ''!
password
	^password! !

!SWRequest methodsFor: 'testing' stamp: ''!
isAuthenticated
	^username notNil 
		and: [ password notNil ]! !

!SWRequest methodsFor: 'accessing-url' stamp: ''!
url
	^url! !

!SWRequest methodsFor: 'accessing-url' stamp: ''!
urlNext
	^urlparsed removeFirst! !

!SWRequest methodsFor: 'accessing' stamp: 'chbu 10/22/2003 17:29'!
response
	response isNil 
		ifTrue: [ response := SWResponse server: self server ].
	^response! !

!SWRequest methodsFor: 'accessing-headers' stamp: ''!
headers
	^headers! !

!SWRequest methodsFor: 'accessing-fields' stamp: ''!
fields
	^fields! !

!SWRequest methodsFor: 'accessing-headers' stamp: ''!
headerIncludes: aKey
	^headers 
		includesKey: aKey asUppercase! !

!SWRequest methodsFor: 'accessing-cookies' stamp: ''!
cookieAt: aKey
	^self 
		cookieAt: aKey 
		ifAbsent: [ nil ]! !

Object subclass: #SWTemplate
	instanceVariableNames: ''
	classVariableNames: 'SWExpandTable'
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplate commentStamp: '<historical>' prior: 0!
Templates are used to render common parts of wiki-pages. They are defined within a collection hold in the root of the wiki and in combination with a selected Stylesheet, they provide the look-and-feel of the wiki. As the templates are hold in the property manager of the structure, they are shared within all childrens of a folder unless there is a new definition.!

SWTemplate subclass: #SWTemplateBody
	instanceVariableNames: 'id title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplateBody commentStamp: '<historical>' prior: 0!
The class \texttt{TemplateBody} should be subclassed in most of the cases to create a new template component. Don't forget to implement the message #title on the class-side to return a string describing this class. The title is also used by default to identify the associated CSS-id and to render it into the body part. The messages #defaultId and #defaultTitle might be used to change this behaviour. The user is always able to edit the id and title from within the template-editor in the web-browser to customise the template to his needs and to the applied stylesheet.
!

!SWTemplateBody methodsFor: 'configuration' stamp: ''!
defaultTitle
	^self class title! !

SWTemplateBody subclass: #SWTemplateBodyTitle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplateBodyTitle class methodsFor: 'configuration' stamp: ''!
title
	^'Title'! !

!SWTemplateBodyTitle methodsFor: 'rendering' stamp: ''!
renderBodyWith: anAction on: html
	self renderDivFor: anAction on: html with: nil! !

!SWTemplateBodyTitle methodsFor: 'configuration' stamp: ''!
defaultTitle
	^'%r'! !

SWTemplateBody subclass: #SWTemplateBodySearch
	instanceVariableNames: 'expression root ignoreCase'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplateBodySearch methodsFor: 'accessing' stamp: ''!
expression: aString
	expression := aString! !

!SWTemplateBodySearch methodsFor: 'rendering' stamp: 'svp 11/10/2003 14:03'!
renderBodyWith: anAction on: html 
	self renderDivFor: anAction on: html with: [
		html attributeAt: #action put: (SWSearch urlFor: anAction structure).
		html form: [
			html textInputWithValue: (self expand: self expression for: anAction) callback: #expression:.
			html hiddenInputWithValue: self ignoreCase callback: #ignoreCase:.
			html hiddenInputWithValue: self root callback: #root:.
			html submitButtonWithAction: #executeSearch text: 'Search' ] ]! !

!SWTemplateBodySearch methodsFor: 'accessing' stamp: ''!
expression
	^expression! !

!SWTemplateBodySearch methodsFor: 'accessing' stamp: ''!
root
	^root! !

!SWTemplateBodySearch methodsFor: 'accessing' stamp: ''!
root: aBoolean
	root := aBoolean! !

!SWTemplateBodySearch methodsFor: 'rendering' stamp: ''!
renderConfigWith: anAction on: html
	super renderConfigWith: anAction on: html.
	self renderConfigString: anAction on: html label: 'Expression' selector: #expression.
	self renderConfigBoolean: anAction on: html label: 'Ignore character case' selector: #ignoreCase.
	self renderConfigBoolean: anAction on: html label: 'Start in the root-folder' selector: #root.! !

!SWTemplateBodySearch methodsFor: 'accessing' stamp: ''!
ignoreCase: aBoolean
	ignoreCase := aBoolean! !

!SWTemplateBodySearch methodsFor: 'accessing' stamp: ''!
ignoreCase
	^ignoreCase! !

!SWTemplateBodySearch class methodsFor: 'configuration' stamp: ''!
title
	^'Search'! !

!SWTemplateBodySearch methodsFor: 'initialization' stamp: ''!
initialize
	super initialize.
	self expression: 'Search %t ...'.
	self ignoreCase: true.	
	self root: false.! !

SWTemplateBody subclass: #SWTemplateBodySession
	instanceVariableNames: 'showRoles showUser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplateBodySession methodsFor: 'accessing' stamp: ''!
showRoles: aBoolean
	showRoles := aBoolean! !

!SWTemplateBodySession methodsFor: 'rendering' stamp: 'chbu 10/22/2003 19:38'!
renderBodyWith: anAction on: html 
	| user |
	user := anAction user.
	self renderDivFor: anAction on: html with: [
		html div: [
			user isAnonymous 
				ifTrue: [ html anchorWithUrl: (SWLogin urlFor: anAction structure) do: 'Login' ]
				ifFalse: [ html anchorWithUrl: (SWLogout urlFor: anAction structure) do: 'Logout' ] ].
		user isAnonymous ifFalse: [
			self showUser ifTrue: [
				html div: [
					html heading: 'User' level: 2.
					html divClass: #list with: [
						html divClass: #listItem with: user username ] ] ].
			self showRoles ifTrue: [
				html div: [
					html heading: 'Roles' level: 2.
					html divClass: #list with: [
						user roles 
							do: [ :each | html divClass: #listItem with: each name ]
							separatedBy: [ html spanClass: #listSeparator with: nil ] ] ] ] ] ]! !

!SWTemplateBodySession methodsFor: 'accessing' stamp: ''!
showUser
	^showUser! !

!SWTemplateBodySession methodsFor: 'rendering' stamp: ''!
renderConfigWith: anAction on: html
	super renderConfigWith: anAction on: html.
	self renderConfigBoolean: anAction on: html label: 'Show User' selector: #showUser.
	self renderConfigBoolean: anAction on: html label: 'Show Roles' selector: #showRoles.! !

!SWTemplateBodySession methodsFor: 'accessing' stamp: ''!
showRoles
	^showRoles! !

!SWTemplateBodySession methodsFor: 'initialization' stamp: ''!
initialize
	super initialize.
	self showUser: true.
	self showRoles: true.! !

!SWTemplateBodySession methodsFor: 'accessing' stamp: ''!
showUser: aBoolean
	showUser := aBoolean! !

!SWTemplateBodySession class methodsFor: 'configuration' stamp: ''!
title
	^'Session'! !

SWTemplateBody subclass: #SWTemplateBodyPath
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplateBodyPath class methodsFor: 'configuration' stamp: ''!
title
	^'Path'! !

!SWTemplateBodyPath methodsFor: 'rendering' stamp: ''!
renderBodyWith: anAction on: html 
	self renderDivFor: anAction on: html with: [
		html divClass: #list with: [
			anAction structure parents 
				do: [ :each | html divClass: #listItem with: [ html anchorWithUrl: each url do: each title ] ]
				separatedBy: [ html spanClass: #listSeparator with: nil ] ] ]! !

!SWTemplateBody methodsFor: 'accessing' stamp: ''!
title
	^title! !

!SWTemplateBody methodsFor: 'configuration' stamp: ''!
defaultId
	^self class title asLowercase! !

SWTemplateBody subclass: #SWTemplateBodyReferences
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplateBodyReferences class methodsFor: 'configuration' stamp: ''!
title
	^'References'! !

!SWTemplateBodyReferences methodsFor: 'rendering' stamp: 'chbu 11/19/2003 19:42'!
renderBodyWith: anAction on: html
	| references |
	references := anAction structure references.
	references isEmpty ifFalse: [
		self renderDivFor: anAction on: html with: [
			html divClass: #list with: [
				references
					do: [ :each | html divClass: #listItem with: [ html anchorWithUrl: each url do: each title ] ]
					separatedBy: [ html spanClass: #listSeparator with: nil ] ] ] ]! !

SWTemplateBody subclass: #SWTemplateBodyW3C
	instanceVariableNames: 'showPictures showHtmlValidator showCssValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplateBodyW3C methodsFor: 'accessing' stamp: ''!
showCssValidator
	^showCssValidator! !

!SWTemplateBodyW3C methodsFor: 'accessing' stamp: ''!
showPictures: aBoolean
	showPictures := aBoolean! !

!SWTemplateBodyW3C methodsFor: 'accessing' stamp: ''!
showHtmlValidator: anObject
	showHtmlValidator := anObject! !

!SWTemplateBodyW3C methodsFor: 'rendering' stamp: ''!
renderConfigWith: anAction on: html
	super renderConfigWith: anAction on: html.
	self renderConfigBoolean: anAction on: html label: 'Show Pictures' selector: #showPictures.
	self renderConfigBoolean: anAction on: html label: 'HTML Validator' selector: #showHtmlValidator.
	self renderConfigBoolean: anAction on: html label: 'CSS Validator' selector: #showCssValidator.! !

!SWTemplateBodyW3C methodsFor: 'rendering' stamp: ''!
renderBodyWith: anAction on: html 
	self renderDivFor: anAction on: html with: [
		html divClass: #list with: [
			self showHtmlValidator ifTrue: [
				html divClass: #listItem with: [
					html anchorWithUrl: 'http://validator.w3.org/check/referer' do: [
					self showPictures 
						ifTrue: [ html image: 'http://www.w3.org/Icons/valid-xhtml10' alt: 'Valid XHTML!!' size: 88@31 ]
						ifFalse: [ html text: 'xhtml' ] ] ].
				self showCssValidator 
					ifTrue: [ html spanClass: #listSeparator with: nil ] ].
			self showCssValidator ifTrue: [
				html divClass: #listItem with: [
					html anchorWithUrl: 'http://jigsaw.w3.org/css-validator/check/referer' do: [
					self showPictures 
						ifTrue: [ html image: 'http://jigsaw.w3.org/css-validator/images/vcss' alt: 'Valid CSS!!' size: 88@31 ]
						ifFalse: [ html text: 'css' ] ] ] ] ] ]! !

!SWTemplateBodyW3C class methodsFor: 'configuration' stamp: ''!
title
	^'Validator'! !

!SWTemplateBodyW3C methodsFor: 'accessing' stamp: ''!
showHtmlValidator
	^showHtmlValidator! !

!SWTemplateBodyW3C methodsFor: 'accessing' stamp: ''!
showPictures
	^showPictures! !

!SWTemplateBodyW3C methodsFor: 'accessing' stamp: ''!
showCssValidator: anObject
	showCssValidator := anObject! !

!SWTemplateBodyW3C methodsFor: 'initialization' stamp: ''!
initialize
	super initialize.
	self showPictures: true.
	self showHtmlValidator: true.
	self showCssValidator: true.! !

!SWTemplateBody methodsFor: 'accessing' stamp: ''!
id
	^id! !

!SWTemplateBody methodsFor: 'initialization' stamp: ''!
initialize
	super initialize.
	self id: self defaultId.
	self title: self defaultTitle.! !

SWTemplateBody subclass: #SWTemplateBodyContents
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplateBodyContents class methodsFor: 'configuration' stamp: ''!
title
	^'Contents'! !

!SWTemplateBodyContents class methodsFor: 'configuration' stamp: ''!
isRemoveable
	^false! !

!SWTemplateBodyContents methodsFor: 'configuration' stamp: ''!
defaultTitle
	^'%a'! !

!SWTemplateBodyContents methodsFor: 'rendering' stamp: ''!
renderBodyWith: anAction on: html
	self renderDivFor: anAction on: html with: [
		anAction renderContent ]! !

!SWTemplateBodyContents methodsFor: 'rendering' stamp: ''!
renderHeadWith: anAction on: html
	html style: anAction structure stylesheet! !

SWTemplateBody subclass: #SWTemplateBodyCustom
	instanceVariableNames: 'document message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplateBodyCustom methodsFor: 'initialization' stamp: 'chbu 10/22/2003 09:53'!
initialize
	super initialize.
	self document: (SWParagraph new
		add: (SWText newText: 'Last modified: ');
		add: (SWCode newCode: 'structure timestamp');
		yourself)! !

!SWTemplateBodyCustom class methodsFor: 'configuration' stamp: ''!
title
	^'Custom'! !

!SWTemplateBodyCustom methodsFor: 'tools' stamp: 'chbu 10/22/2003 17:56'!
parseSource: aString with: anAction 
	self 
		document: SWDocument new;
		message: nil.
	[ self document: (SWWikiParser parse: aString readStream for: anAction) ] 
		on: SmaCCParserError
		do: [ :error | error return: (self message: error messageText) ]! !

!SWTemplateBodyCustom methodsFor: 'accessing' stamp: ''!
document: aDocument
	document := aDocument! !

!SWTemplateBodyCustom methodsFor: 'accessing' stamp: ''!
message
	^message! !

!SWTemplateBodyCustom methodsFor: 'rendering' stamp: 'chbu 10/22/2003 17:56'!
renderBodyWith: anAction on: html
	self renderDivFor: anAction on: html with: [
		self document isNil
			ifTrue: [ html spanNamed: #error with: self message ]
			ifFalse: [ SWVisitorRendererHtml render: self document with: anAction on: html ] ]! !

!SWTemplateBodyCustom methodsFor: 'accessing' stamp: ''!
message: aString
	message := aString! !

!SWTemplateBodyCustom methodsFor: 'accessing' stamp: 'chbu 10/22/2003 17:56'!
source
	^String streamContents: [ :stream |
		SWVisitorRendererWiki render: self document on: stream ]! !

!SWTemplateBodyCustom methodsFor: 'accessing' stamp: ''!
document
	^document! !

!SWTemplateBodyCustom methodsFor: 'rendering' stamp: ''!
renderConfigWith: anAction on: html
	super renderConfigWith: anAction on: html.
	self renderConfigCustom: anAction on: html label: 'Source' do: [
		html attributeAt: #rows put: 6; attributeAt: #style put: 'width: 100%'.
		html 
			textAreaWithValue: self source
			callback: [ :action :value | self parseSource: value with: action ] ]! !

!SWTemplateBody methodsFor: 'accessing' stamp: ''!
id: aString
	id := aString! !

!SWTemplateBody methodsFor: 'accessing' stamp: ''!
title: aString 
	title := aString! !

!SWTemplateBody methodsFor: 'rendering-tools' stamp: ''!
renderDivFor: anAction on: html with: aBlock
	html divNamed: self id with: [
		self title isEmpty
			ifFalse: [ html heading: (self expand: self title for: anAction) level: 1 ].
		html render: aBlock ]! !

SWTemplateBody subclass: #SWTemplateBodyActions
	instanceVariableNames: 'actions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplateBodyActions methodsFor: 'rendering' stamp: ''!
renderConfigActions: anAction on: html
	| listed selected |
	listed := selected := nil.
	html layoutTable: [
		html tableRowWith: [
			html attributeAt: #size put: 6.
			html 
				selectFromList: self listActions 
				selected: nil 
				callback: [ :action :value | listed := value ] 
				labels: [ :class | class name , ' (' , class title , ')' ].
			html attributeAt: #size put: 6.
			html 
				selectFromList: self actions 
				selected: nil 
				callback: [ :action :value | selected := value ] 
				labels: [ :class | class name , ' (' , class title , ')' ] ].
		html tableRow: [
			html attributeAt: #align put: #center.
			html tableData: [
				html submitButtonWithAction: [ listed isNil ifFalse: [ self actions add: listed ] ] text: 'add'.
				html submitButtonWithAction: [ self actions remove: selected ifAbsent: [ ] ] text: 'remove'.
				html submitButtonWithAction: [ self actions moveUp: selected ifError: [ ] ] text: 'up'.
				html submitButtonWithAction: [ self actions moveDown: selected ifError: [ ] ] text: 'down' ] ] ]! !

!SWTemplateBodyActions methodsFor: 'accessing' stamp: ''!
actions
	^actions! !

!SWTemplateBodyActions methodsFor: 'tools' stamp: ''!
listActions
	^self allActions
		reject: [ :each | self actions includes: each ]! !

!SWTemplateBodyActions methodsFor: 'tools' stamp: 'svp 11/10/2003 13:59'!
allActions
	^SWAction withAllSubclasses asArray
		select: [ :each | each isListable ]! !

!SWTemplateBodyActions methodsFor: 'rendering' stamp: ''!
renderConfigWith: anAction on: html
	super renderConfigWith: anAction on: html.
	html tableRow: [
		html tableData: 'Actions'.
		html tableData: [
			self renderConfigActions: anAction on: html ] ]! !

!SWTemplateBodyActions methodsFor: 'configuration' stamp: 'chbu 10/22/2003 17:55'!
defaultActions
	^OrderedCollection new
		add: SWPageView;
		add: SWPageEdit;
		add: SWFolderEdit;
		add: SWResourceEdit;
		add: SWPageHistory;
		add: SWResourceHistory;
		add: SWRecentChanges;
		add: SWSearch;
		add: SWTemplateEdit;
		yourself! !

!SWTemplateBodyActions methodsFor: 'rendering' stamp: ''!
renderBodyWith: anAction on: html 
	self renderDivFor: anAction on: html with: [
		html divClass: #list with: [
			(self actionsFor: anAction) 
				do: [ :each | html divClass: #listItem with: [ html anchorWithUrl: (each urlFor: anAction structure) do: each title ] ]
				separatedBy: [ html spanClass: #listSeparator with: nil ] ] ]! !

!SWTemplateBodyActions methodsFor: 'initialization' stamp: ''!
initialize
	super initialize.
	self actions: self defaultActions.! !

!SWTemplateBodyActions methodsFor: 'accessing' stamp: ''!
actions: anOrderedCollection
	actions := anOrderedCollection! !

!SWTemplateBodyActions class methodsFor: 'configuration' stamp: ''!
title
	^'Actions'! !

!SWTemplateBodyActions methodsFor: 'tools' stamp: 'chbu 10/22/2003 17:55'!
actionsFor: anAction 
	| available sorted |
	available := anAction structure class actions 
		select: [ :each | each isListable ].
	sorted := self actions 
		inject: OrderedCollection new
		into: [ :collection :action | 
			(available includes: action) 
				ifTrue: [ collection add: action ].
			collection ].
	^sorted select: [ :each | 
		[ (each request: anAction request structure: anAction structure) executePermission. true ] 
			on: SWUnauthorizedError
			do: [ :err | err return: false ] ]! !

!SWTemplateBody methodsFor: 'rendering' stamp: ''!
renderBodyWith: anAction on: html
	"Override this message in all subclasses to render the body-part of the template. Do all the rendering within aBlock passed to the message #renderDivFor:on:with: to ensure that the XHTML environment is properly set-up and that the design can be specified using css-stylesheets."! !

!SWTemplateBody methodsFor: 'rendering' stamp: ''!
renderConfigWith: anAction on: html
	"If you override this message in your subclasses don't forget to call super, as there are the default properties for the title and the css-id rendered in here."

	self renderConfigTitle: anAction on: html.
	self renderConfigString: anAction on: html label: 'Id' selector: #id.
	self renderConfigString: anAction on: html label: 'Title' selector: #title.! !

!SWTemplate methodsFor: 'initialization' stamp: ''!
initialize! !

!SWTemplate methodsFor: 'rendering' stamp: ''!
renderConfigWith: anAction on: html
	"This message is called when the configuration form of the receiver should be rendered. This message is solely called from the \texttt{TemplateEdit} action to let the user specify his settings. The default implementation is empty."! !

!SWTemplate class methodsFor: 'accessing' stamp: ''!
isRemoveable
	^true! !

SWTemplate subclass: #SWTemplateHead
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplateHead commentStamp: '<historical>' prior: 0!
The class \texttt{TemplateHead} should be used for templates only rendering to the header of the output-file. If you want to render to the head and to the body use \texttt{TemplateBody} as a superclass instead.!

SWTemplateHead subclass: #SWTemplateHeadTitle
	instanceVariableNames: 'title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplateHeadTitle methodsFor: 'initialization' stamp: ''!
initialize
	super initialize.
	self title: '%r: %a'.! !

!SWTemplateHeadTitle methodsFor: 'rendering' stamp: ''!
renderConfigWith: anAction on: html
	self renderConfigTitle: anAction on: html.
	self renderConfigString: anAction on: html label: 'Title' selector: #title.! !

!SWTemplateHeadTitle methodsFor: 'rendering' stamp: ''!
renderHeadWith: anAction on: html
	html title: (self expand: self title for: anAction)! !

!SWTemplateHeadTitle methodsFor: 'accessing' stamp: ''!
title: aString
	title := aString! !

!SWTemplateHeadTitle methodsFor: 'accessing' stamp: ''!
title
	^title! !

!SWTemplateHeadTitle class methodsFor: 'configuration' stamp: ''!
title
	^'Metatag Title'! !

SWTemplateHead subclass: #SWTemplateHeadNavigation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplateHeadNavigation methodsFor: 'rendering' stamp: ''!
renderHeadWith: action on: html
	self renderMetaLink: 'contents' to: action structure root on: html.
	self renderMetaLink: 'chapter' to: action structure parent on: html.
	self renderMetaLink: 'prev' to: action structure previous on: html.
	self renderMetaLink: 'next' to: action structure next on: html.
	self renderMetaLink: 'start' to: action structure first on: html.
	self renderMetaLink: 'end' to: action structure last on: html.! !

!SWTemplateHeadNavigation class methodsFor: 'configuration' stamp: ''!
title
	^'Metatag Navigational'! !

SWTemplateHead subclass: #SWTemplateHeadMeta
	instanceVariableNames: 'encoding keywords description author index follow'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Template'!

!SWTemplateHeadMeta methodsFor: 'accessing' stamp: ''!
follow: aBoolean
	follow := aBoolean! !

!SWTemplateHeadMeta methodsFor: 'accessing' stamp: ''!
description
	^description! !

!SWTemplateHeadMeta methodsFor: 'rendering' stamp: ''!
renderConfigWith: anAction on: html
	self renderConfigTitle: anAction on: html.
	self renderConfigString: anAction on: html label: 'Encoding' selector: #encoding.
	self renderConfigString: anAction on: html label: 'Description' selector: #description.
	self renderConfigString: anAction on: html label: 'Keywords' selector: #keywords.
	self renderConfigString: anAction on: html label: 'Author' selector: #author.
	self renderConfigBoolean: anAction on: html label: 'Index' selector: #index.
	self renderConfigBoolean: anAction on: html label: 'Follow' selector: #follow.! !

!SWTemplateHeadMeta methodsFor: 'accessing' stamp: ''!
follow
	^follow! !

!SWTemplateHeadMeta methodsFor: 'rendering' stamp: ''!
robotMetaWith: anAction
	^String streamContents: [ :stream |
		self index & anAction isIndexable
			ifFalse: [ stream nextPutAll: 'no' ].
		stream nextPutAll: 'index,'.
		self follow
			ifFalse: [ stream nextPutAll: 'no' ].
		stream nextPutAll: 'follow' ]! !

!SWTemplateHeadMeta class methodsFor: 'configuration' stamp: ''!
title
	^'Metatag Common'! !

!SWTemplateHeadMeta methodsFor: 'accessing' stamp: ''!
keywords: aString
	keywords := aString! !

!SWTemplateHeadMeta methodsFor: 'accessing' stamp: ''!
description: aString
	description := aString! !

!SWTemplateHeadMeta methodsFor: 'accessing' stamp: ''!
encoding: aString
	encoding := aString! !

!SWTemplateHeadMeta methodsFor: 'accessing' stamp: ''!
encoding
	^encoding! !

!SWTemplateHeadMeta methodsFor: 'accessing' stamp: ''!
author
	^author! !

!SWTemplateHeadMeta methodsFor: 'accessing' stamp: ''!
author: aString
	author := aString! !

!SWTemplateHeadMeta methodsFor: 'accessing' stamp: ''!
keywords
	^keywords! !

!SWTemplateHeadMeta methodsFor: 'rendering' stamp: ''!
renderHeadWith: anAction on: html
	self encoding notEmpty
		ifTrue: [ html metaEncoding: self encoding ].
	self keywords notEmpty
		ifTrue: [ html meta: 'keywords' content: (self expand: self keywords for: anAction) ].
	self description notEmpty
		ifTrue: [ html meta: 'description' content: (self expand: self description for: anAction) ].
	self author notEmpty
		ifTrue: [ html meta: 'author' content: (self expand: self author for: anAction) ].
	html meta: 'robots' content: (self robotMetaWith: anAction)! !

!SWTemplateHeadMeta methodsFor: 'initialization' stamp: ''!
initialize
	super initialize.
	self encoding: 'text/html; charset=iso-8859-1'.
	self description: String new.
	self keywords: '%t'.
	self author: String new.
	self index: true.
	self follow: true.! !

!SWTemplateHeadMeta methodsFor: 'accessing' stamp: ''!
index
	^index! !

!SWTemplateHeadMeta methodsFor: 'accessing' stamp: ''!
index: aBoolean
	index := aBoolean! !

!SWTemplateHead methodsFor: 'rendering' stamp: ''!
renderMetaLink: aString to: aStructure on: html
	aStructure isNil ifFalse: [ 
		html link: aString to: aStructure url title: aStructure title ]! !

!SWTemplate class methodsFor: 'accessing' stamp: 'chbu 10/27/2003 20:07'!
expandTable
	^SWExpandTable ifNil: [ SWExpandTable := Dictionary new
		at: $a	put: [ :action | action heading ];
		at: $h	put: [ :action | action server host ];
		at: $i	put: [ :action | action server ip ];
		at: $l	put: [ :action | action structure url ];	
		at: $m	put: [ :action | action structure timestamp ];
		at: $p	put: [ :action | action server port ];
		at: $r	put: [ :action | action structure root title ];
		at: $t	put: [ :action | action structure title ];
		at: $u	put: [ :action | action user username ];	
		at: $v	put: [ :action | action structure version ];
		yourself
	]! !

!SWTemplate methodsFor: 'rendering' stamp: ''!
renderHeadWith: anAction on: html
	"This message is called when the action should render its content to the head-part of the resulting XHTML document. The default implementation is empty."! !

!SWTemplate methodsFor: 'private' stamp: ''!
expand: aString for: anAction
	"Expand aString within the context of anAction. This is often used to let the user specify dynamic parts within the settings of the templates. Currently the following tags are supported:

\begin{itemize}
\item \texttt{\%a} the title of the action
\item \texttt{\%h} the host-name of the server
\item \texttt{\%i} the ip-number of the server
\item \texttt{\%l} the the url of the structure
\item \texttt{\%m} the modification time of the structure
\item \texttt{\%p} the port-number of the server
\item \texttt{\%r} the title of the root structure
\item \texttt{\%t} the title of the structure
\item \texttt{\%u} the name of the current user
\item \texttt{\%v} the version-number of the structure
\end{itemize}"

	| read |
	read := aString readStream.
	^String streamContents: [ :write |
		[ read atEnd ] whileFalse: [
			write nextPutAll: (read upTo: $%).
			read atEnd ifFalse: [
				write nextPutAll: (self expandChar: read next for: anAction) ] ] ]! !

!SWTemplate methodsFor: 'rendering-config' stamp: ''!
renderConfigBoolean: anAction on: html label: aString selector: aSymbol
	self renderConfigCustom: anAction on: html label: String new do: [
		html label: [
			html
				checkboxWithValue: (self perform: aSymbol)
				callback: [ :action :value | self perform: (aSymbol , ':') asSymbol with: value ].
			html text: aString ] ]! !

!SWTemplate methodsFor: 'rendering-config' stamp: ''!
renderConfigCustom: anAction on: html label: aString do: aBlock
	html tableRow: [
		html attributeAt: #width put: 100.
		html cssClass: #label; tableData: aString.
		html cssClass: #field; tableData: aBlock ]! !

!SWTemplate class methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:25'!
defaultStylesheet
	^'@import "http://www.iam.unibe.ch/~scg/smallwiki/standard_blue/style.css";'! !

!SWTemplate methodsFor: 'rendering-config' stamp: ''!
renderConfigString: anAction on: html label: aString selector: aSymbol
	self renderConfigCustom: anAction on: html label: aString do: [
		html attributeAt: #style put: 'width: 100%'.
		html 
			textInputWithValue: (self perform: aSymbol) 
			callback: [ :action :value | self perform: (aSymbol , ':') asSymbol with: value ] ]! !

!SWTemplate class methodsFor: 'instance-creation' stamp: ''!
new
	^super new
		initialize; 
		yourself! !

!SWTemplate methodsFor: 'rendering' stamp: ''!
renderBodyWith: anAction on: html
	"This message is called when the action should render its content to the body-part of the resulting XHTML document. The default implementation is empty."! !

!SWTemplate methodsFor: 'rendering-config' stamp: ''!
renderConfigTitle: anAction on: html
	html tableRow: [
		html attributeAt: 'colspan' put: 2.
		html tableData: [
			html heading: self class title level: 3 ] ]! !

!SWTemplate methodsFor: 'private' stamp: 'chbu 11/12/2003 20:06'!
expandChar: aCharacter for: anAction
	| block |
	block := self class expandTable
		at: aCharacter 
		ifAbsent: [ [ :action | String with: aCharacter ] ].
	^(block value: anAction)
		asString! !

!SWTemplate class methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:25'!
defaultTemplates
	^OrderedCollection new
		add: SWTemplateHeadMeta new;
		add: SWTemplateHeadTitle new;
		add: SWTemplateHeadNavigation new;
		add: SWTemplateBodyTitle new;
		add: SWTemplateBodyPath new;
		add: SWTemplateBodyActions new;
		add: SWTemplateBodySession new;
		add: SWTemplateBodyContents new;
		yourself! !

!SWTemplate class methodsFor: 'accessing' stamp: ''!
title
	^nil! !

Object subclass: #SWStorage
	instanceVariableNames: 'server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Storage'!

!SWStorage commentStamp: '<historical>' prior: 0!
The abstract storage class provides a protocol to all kinds of storage mechanism implementing persistence in a wiki. It takes care of the notification of changes. Subclasses should implement one of the message #changed or #changed: to make the given structure persistent.!

!SWStorage class methodsFor: 'cleaning-up' stamp: ''!
stopThreads
	self allGeneralInstancesDo: [ :storage |
		storage stopThread ]! !

!SWStorage methodsFor: 'notification' stamp: ''!
changed: aStructure
	"Everytime a structure inside the wiki-tree changes this message is called. Override
	it to provide a storage mechanism."

	self changed! !

!SWStorage methodsFor: 'notification' stamp: ''!
changed
	"This message is called whenever something changed inside the wiki
	structure, if you need to know what exactly happened override #changed:
	instead."

	self subclassResponsibility! !

SWStorage subclass: #SWSnapshotStorage
	instanceVariableNames: 'thread delay lastchange lastsnapshot'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Storage'!

!SWSnapshotStorage commentStamp: '<historical>' prior: 0!
The class \texttt{SnapshotStorage} provides an interface to make snapshots of wikis on a regular bases. With the implementation of the \texttt{ImageStorage} as a concrete implementation this is the most secure and most widely used storage mechanism.

!

!SWSnapshotStorage methodsFor: 'accessing' stamp: ''!
delay: aDelayInSeconds 
	"Set the delay in seconds between the snapshots."

	delay := aDelayInSeconds! !

!SWSnapshotStorage methodsFor: 'thread' stamp: ''!
stopThread
	self isThreadRunning 
		ifTrue: [ thread terminate ].
	thread := nil! !

!SWSnapshotStorage methodsFor: 'accessing' stamp: ''!
lastsnapshot
	"Return the timestamp when the last successfull snapshot has been made."

	^lastsnapshot! !

!SWSnapshotStorage methodsFor: 'thread' stamp: ''!
threadWait
	self isThreadRunning 
		ifTrue: [ self delaySemaphore wait ]! !

!SWSnapshotStorage methodsFor: 'testing' stamp: ''!
isThreadRunning
	"Return if the storage thread is running."

	^thread notNil! !

!SWSnapshotStorage methodsFor: 'accessing' stamp: ''!
lastchange
	"Return the timestamp of the last change of the whole wiki."

	^lastchange! !

!SWSnapshotStorage methodsFor: 'notification' stamp: 'chbu 10/22/2003 17:43'!
changed
	lastchange := TimeStamp now! !

!SWSnapshotStorage methodsFor: 'configuration' stamp: 'chbu 3/22/2004 21:35'!
defaultDelay
	^10 * 60 "sec"! !

!SWSnapshotStorage methodsFor: 'private' stamp: ''!
delaySemaphore
	^Delay forSeconds: self delay! !

!SWSnapshotStorage methodsFor: 'thread' stamp: ''!
threadSnapshot
	self isSnapshotNeeded 
		ifTrue: [ self snapshot ].
	self threadWait! !

!SWSnapshotStorage methodsFor: 'testing' stamp: ''!
isChanged
	"Return true if the wiki has been changed since the last snapshot."

	^self lastchange asSeconds > self lastsnapshot asSeconds! !

!SWSnapshotStorage methodsFor: 'snapshot' stamp: ''!
privatePreSnapshot
	"Override this message with code that should be executed before doing
	the actual snapshot."! !

SWSnapshotStorage subclass: #SWImageStorage
	instanceVariableNames: 'filename'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Storage'!

!SWImageStorage methodsFor: 'tools' stamp: 'svp 11/11/2003 11:30'!
backup
	| originalFiles backupFiles |

	originalFiles := (OrderedCollection new: 2)
		add: (self filename, '.image');
		add: (self filename, '.changes');
		yourself.

	backupFiles := (OrderedCollection new: 2)
		add: (self filename, '.', self defaultBackupExtension, '.image');
		add: (self filename, '.', self defaultBackupExtension, '.changes');
		yourself. 

	originalFiles with: backupFiles do: 
		[ :of :bf |
		(FileStream isAFileNamed: of) ifTrue:
			[(FileStream isAFileNamed: bf) ifTrue:
				[FileDirectory deleteFilePath: (FileDirectory default fullNameFor: bf)].
			FileDirectory default copyFileNamed: of toFileNamed: bf]]! !

!SWImageStorage methodsFor: 'accessing' stamp: ''!
filename: aString
	filename := aString! !

!SWImageStorage methodsFor: 'accessing' stamp: ''!
filename
	filename isNil 
		ifTrue: [ filename := self defaultFilename ].
	^filename! !

!SWImageStorage methodsFor: 'snapshot' stamp: 'svp 11/11/2003 11:01'!
privateSnapshotImage

	| newName |
	newName := self filename, '.image'.
	(SourceFiles at: 2) ifNotNil:
		[Smalltalk saveChangesInFileNamed: (Smalltalk fullNameForChangesNamed: newName)].
	Smalltalk saveImageInFileNamed: (Smalltalk fullNameForImageNamed: newName).
! !

!SWImageStorage methodsFor: 'snapshot' stamp: ''!
privateSnapshot
	[ self privateSnapshotImage ]
		on: Notification
		do: [ :err | err resume ]! !

!SWImageStorage methodsFor: 'snapshot' stamp: ''!
privatePreSnapshot
	super privatePreSnapshot.
	self backup.! !

!SWImageStorage methodsFor: 'configuration' stamp: 'svp 11/11/2003 11:14'!
defaultBackupExtension
	^'backup'! !

!SWImageStorage methodsFor: 'configuration' stamp: 'chbu 3/22/2004 21:35'!
defaultFilename
	^'smallwikiSnapshot'! !

!SWSnapshotStorage methodsFor: 'accessing' stamp: ''!
server: aServer 
	server notNil ifTrue: [self stopThread].
	super server: aServer.
	server notNil ifTrue: [self startThread]! !

!SWSnapshotStorage methodsFor: 'thread' stamp: ''!
threadBlock
	^[ [ [ self threadSnapshot ] repeat ] 
		ifCurtailed: [ thread := nil ] ]! !

!SWSnapshotStorage methodsFor: 'initialize' stamp: ''!
initialize
	super initialize.
	delay := self defaultDelay.
	lastchange := self defaultTimestamp.
	lastsnapshot := self defaultTimestamp.! !

!SWSnapshotStorage methodsFor: 'testing' stamp: ''!
isSnapshotNeeded
	"Return true if a snapshot is needed."

	^self isValid and: [ self isChanged ]! !

!SWSnapshotStorage methodsFor: 'testing' stamp: 'chbu 10/22/2003 17:43'!
isExpired
	"Return true if the delay has been expired since the last snapshot."

	^TimeStamp now asSeconds - self lastsnapshot asSeconds > self delay! !

!SWSnapshotStorage methodsFor: 'snapshot' stamp: ''!
privateSnapshot
	"Override this message to do the actual snapshot."

	self subclassResponsibility! !

!SWSnapshotStorage methodsFor: 'accessing' stamp: ''!
delay
	^delay! !

!SWSnapshotStorage methodsFor: 'configuration' stamp: 'chbu 3/22/2004 21:33'!
defaultPriority
	^Processor userBackgroundPriority! !

!SWSnapshotStorage methodsFor: 'snapshot' stamp: 'chbu 10/22/2003 17:42'!
privatePostSnapshot
	"Override this message with code that should be executed after doing the
	actual snapshot."

	lastsnapshot := TimeStamp now! !

!SWSnapshotStorage methodsFor: 'configuration' stamp: 'chbu 10/22/2003 09:46'!
defaultTimestamp
	^TimeStamp fromSeconds: 0! !

!SWSnapshotStorage methodsFor: 'snapshot' stamp: ''!
snapshot
	"Do not override this message, that simply calls the messages #privatePreSnapshot,
	#privateSnapshot and #privatePostSnapshot in order to save the wiki structure as a
	whole."

	self privatePreSnapshot.
	self privateSnapshot.
	self privatePostSnapshot.! !

SWSnapshotStorage subclass: #SWSIXXStorage
	instanceVariableNames: 'directoryRoot loadFilename'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Storage'!

!SWSIXXStorage methodsFor: 'filenames' stamp: 'svp 11/7/2003 01:54'!
directoryCreate: aDirectory

	aDirectory assureExistence
! !

!SWSIXXStorage methodsFor: 'utility-streams' stamp: ''!
sixxSettings
	"If you get an exception here, make sure that you have loaded SIXX
	0.1f alpha at least."

	SixxSetting useEnvironment: true! !

!SWSIXXStorage methodsFor: 'filenames' stamp: 'chbu 11/16/2003 12:29'!
directoryTime
	| stream |
	stream := WriteStream on: String new.
	TimeStamp now time hours printOn: stream paddedWith: $0 to: 2 base: 10.
	^stream contents! !

!SWSIXXStorage methodsFor: 'utility-streams' stamp: 'svp 11/11/2003 14:28'!
readStreamFrom: aFilename do: aBlock
	| stream result |
	self sixxSettings.
	stream := SixxReadStream on: (FileStream readOnlyFileNamed: aFilename).
	result := [ aBlock value: stream ]
		ensure: [ stream close ].
	^result! !

!SWSIXXStorage methodsFor: 'accessing' stamp: ''!
loadFilename
	^loadFilename! !

!SWSIXXStorage methodsFor: 'filenames' stamp: 'svp 11/7/2003 01:53'!
directory
	^self directoryRoot.
! !

!SWSIXXStorage methodsFor: 'accessing' stamp: ''!
loadFilename: aFilename
	loadFilename := aFilename! !

!SWSIXXStorage methodsFor: 'initialization' stamp: ''!
initialize	
	super initialize.
	directoryRoot := self directoryDefault.! !

!SWSIXXStorage methodsFor: 'filenames' stamp: 'svp 11/7/2003 01:47'!
directoryDefault
	^FileDirectory default! !

!SWSIXXStorage methodsFor: 'accessing' stamp: ''!
directoryRoot: aFilename
	directoryRoot := aFilename! !

!SWSIXXStorage methodsFor: 'accessing' stamp: ''!
directoryRoot
	^directoryRoot! !

!SWSIXXStorage methodsFor: 'snapshot' stamp: 'svp 11/7/2003 01:52'!
privateSnapshot
	| directory |
	self 
		directoryCreate: (directory := self directory);
		writeStreamTo: (loadFilename := directory fullPathFor: 'root.xml')
		do: [ :stream |
			stream contextDictionary at: #directory put: directory.
			stream nextPut: self server root ]! !

!SWSIXXStorage methodsFor: 'filenames' stamp: ''!
directoryDate
	^Date today 
		printFormat: #(3 2 1 $- 3 1)! !

!SWSIXXStorage methodsFor: 'utility-streams' stamp: 'svp 11/11/2003 14:40'!
writeStreamTo: aFilename do: aBlock
	| stream result |
	self sixxSettings.
	result := nil.
	stream := SixxWriteStream on: (FileStream forceNewFileNamed: aFilename).
	result := [ aBlock value: stream ]
		ensure: [ stream close ].
	^result! !

!SWSnapshotStorage methodsFor: 'thread' stamp: 'chbu 3/22/2004 21:32'!
startThread

	self isThreadRunning 
		ifFalse: [
				thread := self threadBlock.
				thread forkAt: self defaultPriority.
				]
! !

!SWStorage methodsFor: 'initialize' stamp: ''!
initialize! !

!SWStorage methodsFor: 'accessing' stamp: ''!
server: aServer
	server := aServer! !

!SWStorage methodsFor: 'accessing' stamp: ''!
server
	^server! !

!SWStorage methodsFor: 'testing' stamp: ''!
isValid
	^server notNil! !

!SWStorage class methodsFor: 'instance creation' stamp: ''!
new
	^super new 
		initialize! !

Object subclass: #SWServer
	instanceVariableNames: 'host ip port root storage callback users'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Server'!

!SWServer commentStamp: '<historical>' prior: 0!
The server class has been designed to be subclassed and to provide a common interface to different server implementations. A server might get started using the messages #start, #startOn:, #startOn:host:ip: or by simply instantiating using the message #new, configuring and starting manually. Please note that the server is not a singleton, so there might be multiple instances running within the same image.

=server := SwazooServer startOn: 8080.

The instance variable \texttt{root} represents the root-entity of the wiki-tree, what is usually a folder. When starting a new server, a default configuration will be created. Take care that you don't accidentally call the write-accessor for the root on a running wiki, as all the subentries will be destroyed immediately without the possibility of going back. If you want to have a look at the model of your wiki, evaluate the following expression:

=server root inspect

The default server has no automatic storage mechanism assigned; this is basically useful when developing for SmallWiki and saving the image manually. If you use the wiki in a production environment make sure that you assign a working storage-strategy and test extensively that it is fitting your needs. If you develop other storage strategies, please let us know as we are interested to integrate them into the main-distribution.

=server storage: ImageStorage new.     " fast and secure persistence "
=server storage: SIXXStorage new.	     " slow dump-out using xml "
=server storage: nil.                  " no persistence "

The responsibility to pass the request to the root node of the wiki is taken by the server. Also there will be caught all kinds of exceptions and being displayed as a stack-dump on the client side. The link \emph{Open Debugger} can be used to open the debugger in VisualWorks within the context that caused the error and investigate the problem further.!

!SWServer methodsFor: 'initialize' stamp: ''!
initialize
	users := Dictionary new.
	self callback: self defaultCallbackCache.
	self root: self defaultRoot.
	self userAddAll: self defaultUsers.! !

!SWServer methodsFor: 'serving-private' stamp: ''!
emitException: anException request: aRequest
	self 
		emitException: anException 
		response: aRequest response! !

!SWServer methodsFor: 'accessing-users' stamp: ''!
userAdd: anUser
	"Add a new user to the receiver. Any user with the same username will be
	overridden."

	^users 
		at: anUser username 
		put: anUser! !

SWServer subclass: #SWKom
	instanceVariableNames: 'comanche'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Server'!

!SWKom methodsFor: 'initialize' stamp: 'svp 11/7/2003 01:26'!
initialize

	super initialize.
	ip := self class defaultIp.
	host := self class defaultHost.! !

!SWKom class methodsFor: 'examples' stamp: 'svp 11/11/2003 10:57'!
startOn: portNumber host: hostName ip: ipAddress
	"This example starts a basic server that serves nothing but
	SmallWiki content.  It also logs the requests to the Transcript.  For
	more advanced configurations, you'll likely want to build
	your own Comanche module assembly."

	| ma inst |
	inst := self new
			ip: ipAddress;
			host: hostName;
			yourself.
	ma := ModuleAssembly core.
	ma logTo: (KomLogger on: Transcript).
	ma addPlug: [ :request | inst process: request ].
	inst comanche: (HttpService on: portNumber named: 'SmallWiki').
	inst comanche plug: ma rootModule; start.
	^inst
! !

!SWKom methodsFor: 'accessing' stamp: 'svp 11/7/2003 01:28'!
port

	^self comanche isNil
		ifTrue: [port]
		ifFalse: [self comanche portNumber]! !

!SWKom methodsFor: 'request handling' stamp: 'svp 11/6/2003 22:32'!
prepareRequestUrl: aHTTPRequest
	^aHTTPRequest url! !

!SWKom methodsFor: 'accessing' stamp: 'svp 11/7/2003 01:29'!
port: aNumber

	self comanche isNil
		ifTrue: [port := aNumber]
		ifFalse: [self comanche portNumber: aNumber]! !

!SWKom methodsFor: 'accessing' stamp: 'svp 11/7/2003 01:15'!
comanche: anHttpService

	comanche := anHttpService! !

!SWKom methodsFor: 'request handling' stamp: 'svp 11/8/2003 00:57'!
prepareResponse: swResponse
	| komResponse |
	komResponse := HttpResponse new
		contentType:	swResponse type;
		status:			(HttpResponse statusSymbolFor: swResponse status);
		contents:		swResponse stream contents readStream;
		yourself.
	self prepareResponseHeader: komResponse from: swResponse.
	swResponse cookies isEmpty ifFalse: [
		self prepareResponseCookie: komResponse from: swResponse ].
	^komResponse! !

!SWKom methodsFor: 'request handling' stamp: 'svp 11/8/2003 00:59'!
prepareResponseHeader: komResponse from: swResponse

	swResponse headers associationsDo:
		[ :assn |
		komResponse fields at: assn key put: assn value]

! !

!SWKom methodsFor: 'serving' stamp: 'svp 11/7/2003 01:30'!
isRunning

	^self comanche isRunning! !

!SWKom methodsFor: 'request handling' stamp: 'chbu 2/17/2004 18:27'!
process: request
	"Accept a Comanche HttpRequest, convert it into a SmallWiki
	request, process it, then convert the SmallWiki response into 
	a Comanche response and answer that."

	| swRequest |

	swRequest := self prepareRequest: request.

	"Note, don't try and use the SmallWiki exception handling logic (like
	the super implementation does), Comanche can deal with exceptions 
	signaled from here"
	self root process: swRequest.  

	^self prepareResponse: swRequest response.

! !

!SWKom methodsFor: 'request handling' stamp: 'chbu 2/17/2004 18:03'!
prepareRequestFields: aHTTPRequest
	| fields |
	fields := Dictionary new.
	aHTTPRequest getFields keysAndValuesDo: 
		[ :key :value |
		fields at: key put: value].
	aHTTPRequest postFields keysAndValuesDo: 
		[ :key :value |
		fields at: key put: value first].
	aHTTPRequest multipartFormFieldsDo:
		[ :chunk |
		fields at: chunk fieldName put:
			(String streamContents: [ :strm | chunk saveToStream: strm]).
		fields at: 'mime-', chunk fieldName put:
			(chunk contentType).
		].
	^fields! !

!SWKom methodsFor: 'request handling' stamp: 'svp 11/10/2003 11:15'!
prepareResponseCookie: response from: aResponse
	| stream  |

	aResponse cookies keysAndValuesDo: [ :key :value |
		stream := WriteStream on: String new.
		stream nextPutAll: key; nextPut: $=.
		value ifNotNil: [stream nextPutAll: value].
		stream nextPutAll: '; path=/'.
		value ifNil: [ stream nextPutAll: '; Max-Age=0; Expires=Sun, 09 Mar 2003 14:43:19 GMT' ].
		response addCookies: (Array with: stream contents) ]! !

!SWKom methodsFor: 'serving' stamp: 'svp 11/7/2003 01:20'!
start
	"We are just going to plug the receiver into some Comanche
	module hierarchy, therefore, this message doesn't make sense.

	See the class side for example server configurations"

	self comanche start! !

!SWKom methodsFor: 'serving' stamp: 'svp 11/7/2003 01:30'!
isServing

	^self comanche isRunning! !

!SWKom methodsFor: 'request handling' stamp: 'svp 11/6/2003 22:32'!
prepareRequestHeaders: aHTTPRequest
	| headers |
	headers := Dictionary new.
	aHTTPRequest header keysAndValuesDo: [ :item :value |
		headers at: item name put: value ].
	^headers! !

!SWKom methodsFor: 'accessing' stamp: 'svp 11/7/2003 01:14'!
comanche

	^comanche! !

!SWKom methodsFor: 'serving' stamp: 'svp 11/7/2003 01:20'!
stop
	"We are just going to plug the receiver into some Comanche
	module hierarchy, therefore, this message doesn't make sense.

	See the class side for example server configurations"

	self comanche stop! !

!SWKom methodsFor: 'request handling' stamp: 'svp 11/10/2003 11:13'!
prepareRequest: aHTTPRequest
	^(SWRequest server: self)
		headers: (self prepareRequestHeaders: aHTTPRequest);
		fields: (self prepareRequestFields: aHTTPRequest);
		cookies: (self prepareRequestCookies: aHTTPRequest);
		url: (self prepareRequestUrl: aHTTPRequest);
		updateAuthentication;
		yourself! !

!SWKom methodsFor: 'request handling' stamp: 'svp 11/6/2003 22:31'!
prepareRequestCookies: aHTTPRequest
	| cookies |
	cookies := Dictionary new.
	aHTTPRequest cookies keysAndValuesDo: [ :key :value |
		cookies at: key put: value ].
	^cookies! !

!SWServer methodsFor: 'serving' stamp: ''!
stop
	"Stop the web-server of the receiver."

	self subclassResponsibility! !

!SWServer methodsFor: 'accessing-users' stamp: ''!
users
	^users! !

!SWServer methodsFor: 'configuration-pages' stamp: 'chbu 10/22/2003 09:33'!
defaultDocumentInformation
	^SWDocument new
		add: (SWParagraph new 
			add: (SWText newText: 'A Wiki is a collaborative software to do content management. Although there are a lot of different Wiki implementations available today, they all lack the possibility to be extended
and to adapt to the needs of their users. SmallWiki is a new and fully object-oriented Wiki framework in Smalltalk, that has got a lot of unit-tests included.');
			yourself);
		add: (SWParagraph new);
		add: (SWParagraph new
			add: (SWCode newCode: SWFolder new defaultChildrenListCode);
			yourself);
		yourself! !

!SWServer methodsFor: 'serving' stamp: ''!
isServing
	"Return true if the receivers web-server is up and running."

	self subclassResponsibility! !

!SWServer methodsFor: 'accessing' stamp: ''!
root: aStructure
	root isNil 
		ifFalse: [ root removeDependent: self ].
	root := aStructure.
	root isNil
		ifFalse: [ root addDependent: self ]! !

!SWServer class methodsFor: 'server' stamp: ''!
start
	^self
		startOn: self defaultPort
		host: self defaultHost
		ip: self defaultIp! !

!SWServer methodsFor: 'configuration-security' stamp: 'chbu 10/22/2003 17:37'!
defaultUserAnonymous
	^(SWUser anonymous)
		add: self defaultRoleAnonymous;
		yourself! !

!SWServer methodsFor: 'serving-private' stamp: ''!
process: aRequest 
	"Start the chain of responsibilities in the root of the wiki. Any unhandled exceptions 
	thrown while processing the request will be displayed as a stack-trace within the
	browser of the client."

	[ self root process: aRequest ] 
		on: Error
		do: [ :exception | exception return: (self emitException: exception request: aRequest) ]! !

!SWServer methodsFor: 'accessing-server' stamp: ''!
port
	^port! !

!SWServer methodsFor: 'configuration-security' stamp: ''!
defaultUsers
	^OrderedCollection new
		add: self defaultUserAnonymous;
		add: self defaultUserAdministrator;
		yourself! !

!SWServer methodsFor: 'serving-private' stamp: ''!
emitException: anException response: aResponse
	aResponse initialize; server: self.
	self 
		emitException: anException 
		on: aResponse stream! !

!SWServer methodsFor: 'accessing-users' stamp: ''!
userIncludes: aString
	"Return true if the receiver has got a user with the given username."

	^users 
		includesKey: aString! !

!SWServer methodsFor: 'configuration-pages' stamp: 'chbu 10/22/2003 09:33'!
defaultDocumentRoot
	^SWDocument new
		add: (SWParagraph new 
			add: (SWText newText: 'Welcome to SmallWiki, a new Wiki-Wiki implementation in Smalltalk. Below you can find a list with the contents of the root folder of this Wiki. By default this list is generated automatically, but you might replace it with your own welcome-message and page-index.');
			yourself);
		add: (SWParagraph new);
		add: (SWHeader newText: 'Contents of this Site' level: 1);
		add: (SWParagraph new
			add: (SWCode newCode: SWFolder new defaultChildrenListCode);
			yourself);
		yourself! !

!SWServer class methodsFor: 'configuration' stamp: ''!
defaultPort
	^8080! !

!SWServer methodsFor: 'accessing' stamp: ''!
callback
	^callback! !

!SWServer methodsFor: 'accessing-users' stamp: ''!
userAddAll: aCollection
	aCollection do: [ :user |
		self userAdd: user ]! !

!SWServer methodsFor: 'configuration-pages' stamp: 'chbu 10/22/2003 09:33'!
defaultPageInformation
	^(SWFolder title: 'Information')
		document: self defaultDocumentInformation;
		add: self defaultPageIntroduction;
		add: self defaultPageSyntax;
		add: self defaultPageCopyright;
		yourself! !

!SWServer methodsFor: 'accessing' stamp: 'chbu 10/27/2003 22:06'!
callback: aCallbackCache
	callback := aCallbackCache! !

!SWServer methodsFor: 'configuration-pages' stamp: 'chbu 10/22/2003 17:38'!
defaultPageCopyright
	^SWPage
		title: 'Copyright'
		parse: SWSmallWiki licenseString! !

!SWServer methodsFor: 'serving-private' stamp: ''!
emitContext: aContext on: html
	html heading: aContext printString level: 2.
	html attributeAt: 'border' put: '0'.
	html attributeAt: 'width' put: '100%'.
	html table: [
		html tableRow: [
			html attributeAt: 'width' put: '25%'.
			html tableData: 'self'.
			html tableData: aContext receiver printString ].
		aContext localScope localTemps do: [ :item |
			html tableRow: [
				html tableData: item key.
				html tableData: item value printString ] ] ]! !

!SWServer methodsFor: 'accessing-users' stamp: ''!
userRemove: aString
	"Remove the user with the given username from the receiver."

	^users 
		removeKey: aString! !

!SWServer methodsFor: 'serving' stamp: ''!
start
	"Start the web-server of the receiver."

	self subclassResponsibility! !

!SWServer methodsFor: 'configuration-security' stamp: 'chbu 10/22/2003 17:37'!
defaultRoleAnonymous
	^(SWBasicRole name: 'anonymous')
		addAll: (SWStructure allSubclasses collect: [ :class | class permissionAdd ]);
		addAll: (SWStructure allSubclasses collect: [ :class | class permissionEdit ]);
		addAll: (SWStructure allSubclasses collect: [ :class | class permissionView ]);
		yourself! !

!SWServer methodsFor: 'configuration-security' stamp: 'chbu 10/22/2003 17:37'!
defaultUserAdministrator
	^(SWUser username: 'admin' password: 'smallwiki')
		add: self defaultRoleAdministrator;
		yourself! !

!SWServer class methodsFor: 'configuration' stamp: ''!
defaultHost
	^'localhost'! !

!SWServer methodsFor: 'accessing' stamp: ''!
root
	^root! !

!SWServer methodsFor: 'accessing-users' stamp: ''!
userAt: aString
	"Return the user with aString as name, if there is no such user the default
	anonymous user is returned."

	^self
		userAt: aString
		ifAbsent: [ self userAnonymous ]! !

!SWServer methodsFor: 'configuration' stamp: 'chbu 10/22/2003 09:32'!
defaultCallbackCache
	^SWFifoCache new! !

!SWServer methodsFor: 'serving' stamp: ''!
restart
	"Restart the web-server of the receiver."

	self stop.
	self start.! !

!SWServer methodsFor: 'accessing-server' stamp: ''!
host
	^host! !

!SWServer methodsFor: 'accessing-server' stamp: ''!
ip
	^ip! !

!SWServer methodsFor: 'accessing-users' stamp: ''!
userAnonymous
	^self userAt: 'anonymous'! !

!SWServer methodsFor: 'configuration' stamp: 'chbu 10/22/2003 09:32'!
defaultRoot
	"Return the default wiki that will be used when setting up a new server."

	^(SWFolder title: 'SmallWiki')
		document: self defaultDocumentRoot;
		add: self defaultPageInformation;
		yourself! !

!SWServer class methodsFor: 'configuration' stamp: ''!
defaultIp
	^'127.0.0.1'! !

!SWServer methodsFor: 'accessing' stamp: ''!
storage
	^storage! !

!SWServer class methodsFor: 'instance-creation' stamp: ''!
new
	^super new 
		initialize! !

!SWServer methodsFor: 'configuration-security' stamp: 'chbu 10/22/2003 17:37'!
defaultRoleAdministrator
	^SWAdminRole new! !

!SWServer class methodsFor: 'server' stamp: 'chbu 10/24/2003 16:13'!
startOn: anInteger host: aHostString ip: anIpString
	^self new
		port: anInteger;
		host: aHostString;
		ip: anIpString;
		start.! !

!SWServer methodsFor: 'accessing-server' stamp: ''!
host: aString
	host := aString! !

!SWServer methodsFor: 'accessing-users' stamp: ''!
userAt: aString ifAbsent: anExceptionBlock
	"Return the user with aString as name, if there is no such user anExceptionBlock
	is evaluated."

	^users
		at: aString
		ifAbsent: anExceptionBlock! !

!SWServer methodsFor: 'accessing-users' stamp: ''!
roles
	^users 
		inject: Set new
		into: [ :set :user | set addAll: user roles; yourself ]! !

!SWServer class methodsFor: 'as yet unclassified' stamp: 'svp 11/11/2003 14:47'!
defaultWorkspace
	" SWServer defaultWorkspace "

	^ Workspace new contents: 'SmallWiki
Welcome to SmallWiki, a new Wiki-Wiki implementation in Smalltalk. Below you can find some expressions to configure the wiki to your needs. 

Server
Use the following code to create your wiki for the first time. I would suggest to remove that line afterwards, so you don''t accidentally loose your data.

	server := SWKom startOn: 8080.

If you like to change the server parameters use the following messages (you may also use the normal Comanche protocols or the StarBrowser):

	server host: ''localhost''.
	server ip: ''*''.
	server port: 8080.

And don''t forget to restart the sever afterwards:

	server stop.
	server start.

Storage
By default there is no automatic storage process running and all the objects are just kept in your image. It is important that you select a proper storage mechanism, when using SmallWiki in a production environment:

	server storage: SWImageStorage new.			" fast and secure persistence "
	server storage: SWSIXXStorage new.			" slow dump-out using xml "
	server storage: nil.						" no persistence "

To set how often your wiki should get stored, use the following command to specify the delay in seconds:

	server storage delay: 60 * 60.

Other storage implementations might provide additional properties. To test your configuration you might want to execute the following code to force a snapshot right now:

	server storage snapshot.

Callback Cache
SmallWiki supports different cache implementation to remember callback-blocks assigned to links and form-elements. Depending on the speed and the memory available on your machine, you might want to change the settings or choose another implementation.

	server callback: SWExpiringCache new.
	server callback check: 100.			" check culling every n-th write-access "
	server callback ttl: 30 * 60.				" seconds to live "

	server callback: SWFifoCache new.
	server callback max: 5000.			" maximal count of objects "
	server callback shrink: 100.			" release rate of objects "

Security
By default there are two default users created during installation: the anonymous and the admin. The anonymous user will be used by default and has got the permissions to add new pages to the wiki and edit existing ones. Evaluate the following expression to inspect and modify this user:

	server userAt: ''anonymous''

In contrast the administrator has to log-in manually and has got all the available permissions, like deleting files, modifying the history, changing the look-and-feel, evaluating Smalltalk expressions on the server, etc. If you use SmallWiki in a production environment, it is very important that you change the default admin-password with the following expression:

	(server userAt: ''admin'')
		password: ''smallwiki''

To create a new role having the permissions to remove and copy a page, evaluate the following expression:

	studentRole := SWBasicRole name: ''student''.
	studentRole add: SWPage permissionRemove.
	studentRole add: SWPage permissionCopy.

To create a new user and assign the anonymous- and the student-role evaluate try the following code:

	anonymousRole := server roles detect: [ :each |
		each name = ''anonymous'' ].
	user := SWUser username: ''stud1'' password: ''power''.
	user add: anonymousRole.
	user add: studentRole.
	server userAdd: user.

Global Options
There are only a few global options to set. One of those options is to tell SmallWiki if it should pretty print the generated html or not. Note that pretty printing is good for development, but for production I suggest to disable it as the engine is faster and the resulting files get smaller:

	SWHtmlWriteStream prettyPrint: true.
	SWHtmlWriteStream prettyPrint: false.'' runs: (Core.RunArray runs: #(9 143 7 154 1 48 1 1 72 1 63 1 51 1 28 1 1 7 217 1 35 2 31 1 34 2 27 1 25 2 18 1 106 1 32 171 1 25 1 1 15 251 2 36 1 29 44 32 23 32 1 30 29 31 28 2 9 289 28 368 51 107 123 103 202 1 14 296 72) values: ((Core.Array new: 69) at: 1 put: #(#large #bold); at: 3 put: #bold; at: 5 put: (#color -> Graphics.ColorValue darkGreen); at: 6 put: (#color -> Graphics.ColorValue navy); at: 7 put: (#color -> Graphics.ColorValue darkMagenta); at: 8 put: (#color -> Graphics.ColorValue darkGreen); at: 10 put: (#color -> Graphics.ColorValue darkGreen); at: 11 put: (#color -> Graphics.ColorValue navy); at: 12 put: (#color -> Graphics.ColorValue darkMagenta); at: 14 put: (#color -> Graphics.ColorValue darkGreen); at: 15 put: (#color -> Graphics.ColorValue navy); at: 16 put: (#color -> Graphics.ColorValue darkMagenta); at: 18 put: #bold; at: 20 put: (#color -> Graphics.ColorValue darkGreen); at: 21 put: (#color -> Graphics.ColorValue navy); at: 22 put: (#color -> Graphics.ColorValue darkMagenta); at: 23 put: (#color -> Graphics.ColorValue darkGreen); at: 24 put: (#color -> Graphics.ColorValue darkMagenta); at: 25 put: (#color -> Graphics.ColorValue navy); at: 26 put: (#color -> Graphics.ColorValue darkMagenta); at: 27 put: (#color -> Graphics.ColorValue darkGreen); at: 28 put: (#color -> Graphics.ColorValue darkMagenta); at: 29 put: (#color -> Graphics.ColorValue navy); at: 30 put: (#color -> Graphics.ColorValue darkMagenta); at: 31 put: (#color -> Graphics.ColorValue darkGreen); at: 32 put: (#color -> Graphics.ColorValue darkMagenta); at: 34 put: (#color -> Graphics.ColorValue darkGreen); at: 35 put: (#color -> Graphics.ColorValue navy); at: 37 put: (#color -> Graphics.ColorValue darkGreen); at: 38 put: (#color -> Graphics.ColorValue navy); at: 39 put: (#color -> Graphics.ColorValue darkGreen); at: 41 put: #bold; at: 43 put: (#color -> Graphics.ColorValue darkGreen); at: 44 put: (#color -> Graphics.ColorValue navy); at: 45 put: (#color -> Graphics.ColorValue darkGreen); at: 46 put: (#color -> Graphics.ColorValue navy); at: 47 put: (#color -> Graphics.ColorValue darkGreen); at: 48 put: (#color -> Graphics.ColorValue navy); at: 49 put: (#color -> Graphics.ColorValue darkGreen); at: 50 put: (#color -> Graphics.ColorValue navy); at: 51 put: (#color -> Graphics.ColorValue darkGreen); at: 52 put: (#color -> Graphics.ColorValue navy); at: 53 put: (#color -> Graphics.ColorValue darkGreen); at: 54 put: (#color -> Graphics.ColorValue navy); at: 55 put: (#color -> Graphics.ColorValue darkGreen); at: 57 put: #bold; at: 59 put: (#color -> Graphics.ColorValue navy); at: 61 put: (#color -> Graphics.ColorValue navy); at: 63 put: (#color -> Graphics.ColorValue navy); at: 65 put: (#color -> Graphics.ColorValue navy); at: 67 put: #bold; at: 69 put: (#color -> Graphics.ColorValue navy); yourself)))';
	openAsMorphLabel: 'SmallWiki'.! !

!SWServer methodsFor: 'accessing' stamp: ''!
storage: aStorage
	storage isNil
		ifFalse: [ storage server: nil ].
	storage := aStorage.
	storage isNil
		ifFalse: [ storage server: self ]! !

!SWServer class methodsFor: 'server' stamp: ''!
startOn: anInteger
	^self
		startOn: anInteger
		host: self defaultHost
		ip: self defaultIp! !

!SWServer methodsFor: 'updating' stamp: ''!
update: anAspectSymbol with: aParameter
	storage isNil
		ifFalse: [ storage changed: aParameter ]! !

!SWServer methodsFor: 'serving-private' stamp: 'chbu 10/27/2003 20:18'!
emitException: anException on: html 
	| context exception |
	context := anException signalerContext.
	exception := anException copyForDebugging.
	html html: 
			[html head: [html title: anException description].
			html body: 
					[html heading: anException description level: 1.
					html 
						anchorWithAction: [exception defaultAction]
						to: '/'
						do: 'Open Debugger'.
					[context notNil] whileTrue: 
							[self emitContext: context on: html.
							context := context sender]]]! !

!SWServer methodsFor: 'accessing-server' stamp: ''!
ip: aString
	ip := aString! !

!SWServer methodsFor: 'configuration-pages' stamp: 'chbu 10/22/2003 18:55'!
defaultPageSyntax
	^(SWPage title: 'Syntax')
		document: (SWDocument new
			add: (SWHeader newText: 'Paragraphs' level: 1);
			add: (SWParagraph new
				add: (SWText newText: 'As carriage returns are preserved, simply add a newline to begin a new paragraph.');
				yourself);
			add: (SWHeader newText: 'Headers' level: 1);
			add: (SWParagraph new
				add: (SWText newText: 'A line starting with <code>!!</code>s becomes a header line.');
				yourself);
			add: (SWHeader newText: 'Horizontal Line' level: 1);
			add: (SWParagraph new
				add: (SWText newText: 'A line starting with <code>_</code> (underscore) becomes a horizontal line. This is often used to separate topics.');
				yourself);
			add: (SWHeader newText: 'Lists' level: 1);
			add: (SWParagraph new
				add: (SWText newText: 'Using lines starting with <code>#</code>s and <code>-</code>s, creates a list: A block of lines, where each line starts with <code>-</code> is transformed into a bulleted list, where each line is an entry. A block of lines, where each line starts with <code>#</code> is transformed into an ordered list, where each line is an entry.');
				yourself);	
			add: (SWHeader newText: 'Tables' level: 1);
			add: (SWParagraph new
				add: (SWText newText: 'To create a table, start off the lines with <code>|</code> and separate the elements with <code>|</code>s. Each new line represents a new row of the table.');
				yourself);	
			add: (SWHeader newText: 'Preformatted' level: 1);
			add: (SWParagraph new
				add: (SWText newText: 'To create a preformatted section, begin each line with <code>=</code>. A preformatted section uses equally spaced text so that spacing is preserved. ');
				yourself);
			add: (SWHeader newText: 'Links' level: 1);	
			add: (SWParagraph new 
				add: (SWText newText: 'To create a link, put it between <code>*</code>s. There are three different types of links:');
				yourself);
			add: (SWOrderedList new
				add: (SWListItem new
					add: (SWText newText: '<b>Internal Link:</b> If the item exists in the Smallwiki (e.g. <code>*Title of Item*</code>), a link to that item shows up when the page is saved. In case the item does not already exist, the link shows up with a create-button next to it; click on it to create the new item.');
					yourself);
				add: (SWListItem new
					add: (SWText newText: '<b>External Link:</b> If the link is a valid url (e.g. <code>*http://www.google.ch*</code>), a link to that external page shows up.');
					yourself);
				add: (SWListItem new
					add: (SWText newText: '<b>Mail Link:</b> If the link is an email address (e.g. <code>*self@mail.me.com*</code>), a link to mail that person shows up, but it is obfuscated to prevent robots from collecting.');
					yourself);
				yourself);
			add: (SWParagraph new
				add: (SWText newText: 'You can also alias all these links using <code>&gt;</code>. So, you can create a link like this: <code>*Alias>Reference*</code>. The link will show up as <code>Alias</code>, but link to <code>Reference</code>. For images, the alias text will become the alternate text for the image.');
				yourself);	
			add: (SWHeader newText: 'HTML' level: 1);
			add: (SWParagraph new
				add: (SWText newText: 'Use any HTML anywhere you want. Some useful HTML tags are:');
				add: (SWUnorderedList new
					add: (SWListItem new
						add: (SWText newText: 'To make something bold, surround it by <code>&lt;b&gt;</code> and <code>&lt;/b&gt;</code>.');
						yourself);
					add: (SWListItem new
						add: (SWText newText: 'To make something italic, surround it by <code>&lt;i&gt;</code> and <code>&lt;/i&gt;</code>.');
						yourself);
					add: (SWListItem new
						add: (SWText newText: 'To make something underlined, surround it by <code>&lt;u&gt;</code> and <code>&lt;/u&gt;</code>.');
						yourself);
					yourself);
				yourself);
			yourself);
		yourself! !

!SWServer methodsFor: 'configuration-pages' stamp: 'chbu 10/22/2003 18:18'!
defaultPageIntroduction
	^SWPage
		title: 'Introduction'
		parse: 'The term <i>Wiki</i> usually means the collaborative software used to create, edit and manage hypertext pages on the web. A Wiki enables the users to author their documents using a simple markup language within their preferred webbrowser.

Translated from the Hawaiian language <i>Wiki wiki</i> means <i>fast</i> and that is exactly what the collaborative editing process of a WikiWiki Web is all about: Everybody should be able to create and update pages, without the need of user-name and password to login. However there are wiki vandals around that abuse the general public access and make it necessary to protect the content with security mechanism.

!!Problem
There are more than 150 Wiki implementations available and most of them are open source. There are even a few implementations available written in different Smalltalk dialects, so why did we create a new one?

All the implementations we had a look at have major flaws in extensibility: they don''t provide a proper object oriented-design that is covered by unit tests. Moreover they all keep the content of the pages within strings, which makes it painful to render and search the wiki. These wikis haven''t been designed for extensibility!!

The existing Smalltalk wikis are old and it seems that the developers don''t want to touch their running systems. Both, WikiWorks and SqueakWiki, have some of their domain code within external files, what makes the source hard to understand as it is not possible to use the editing and debugging facilities of the Smalltalk environment.

!!Solution
As stated in the previous section, all the current wiki implementations have problems with extensibility and the design. We didn''t want to make the same mistakes and put together the following basic requirements at the very beginning of the project:

-<b>Object-Oriented Design:</b> SmallWiki provides an object oriented domain model. As an example, the content of the pages is parsed and stored as a tree of different entities representing text, links, tables, lists, etc.
-<b>Extensibility:</b> Everything in SmallWiki can be extended: page types, storage mechanism, actions, security mechanism, web-server, etc. Plug-ins can be shared within the community and loaded independently of each other into the system.
-<b>Open Source:</b> SmallWiki is released under the MIT license which grants unrestricted rights to copy, modify, and redistribute as long as the original copyright and license terms are retained.
-<b>Test Suites:</b> SmallWiki is heavily tested. There are more than 200 unit tests included with the core of SmallWiki. This makes it easy to change and verify the code and comes in extremely useful when porting SmallWiki to other Smalltalk dialects or when writing extensions.

To learn more about the technical details of SmallWiki please read *smallwiki.pdf>http://www.iam.unibe.ch/~scg/smallwiki/smallwiki.pdf*.'! !

!SWServer methodsFor: 'accessing-server' stamp: ''!
port: anInteger
	port := anInteger! !

Object subclass: #SWVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Visitor'!

!SWVisitor commentStamp: '<historical>' prior: 0!
I am an abstract visitor to be used on all subclasses of WikiItem. I provide a standard implementation of all accept messages to make it possible to walk automatically through all the entities of a wiki. Subclasses should override these messages in a appropriate way, to visit only the nodes the need and to walk through the structure more efficiently.
!

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptUnorderedList: aUnorderedList
	self visitCollection: aUnorderedList children! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptOrderedList: anOrderedList
	self visitCollection: anOrderedList children! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptLink: aLink
	self acceptText: aLink! !

!SWVisitor methodsFor: 'visiting-structure' stamp: ''!
acceptProperties: aPropertyManager! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptTableRow: aTableRow
	self visitCollection: aTableRow children! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptHeader: aHeader
	self acceptText: aHeader! !

!SWVisitor methodsFor: 'visiting-structure' stamp: ''!
acceptStructure: aStructure
	self visit: aStructure properties! !

!SWVisitor methodsFor: 'initialize' stamp: ''!
initialize! !

!SWVisitor methodsFor: 'visiting-structure' stamp: ''!
acceptResource: aResource
	self acceptStructure: aResource! !

!SWVisitor methodsFor: 'visiting' stamp: ''!
visitCollection: aCollection separatedBy: aBlock
	aCollection
		do: [ :child | child accept: self ]
		separatedBy: aBlock! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptLinkExternal: aLinkExternal
	self acceptLink: aLinkExternal! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptPreformatted: aPreformatted
	self visitCollection: aPreformatted children! !

SWVisitor subclass: #SWVisitorPluggable
	instanceVariableNames: 'block structure'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Visitor'!

!SWVisitorPluggable methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:17'!
block: aBlockContext
	block := aBlockContext! !

!SWVisitorPluggable class methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:18'!
block: aBlock start: aWikiItem
	^self new
		block: aBlock;
		visit: aWikiItem;
		yourself! !

SWVisitorPluggable subclass: #SWVisitorStructure
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Visitor'!

!SWVisitorStructure methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:19'!
acceptStructure: aStructure
	super acceptStructure: aStructure.
	self evaluateOn: aStructure! !

!SWVisitorStructure methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:19'!
acceptPage: aPage
	self acceptStructure: aPage.! !

!SWVisitorPluggable methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:18'!
evaluateOn: aWikiItem
	self block numArgs == 1
		ifTrue: [ self block value: aWikiItem ]
		ifFalse: [ self block value: aWikiItem value: self structure ]! !

!SWVisitorPluggable methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:18'!
structure
	^structure! !

!SWVisitorPluggable methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:17'!
acceptStructure: aStructure
	self structure: aStructure.
	super acceptStructure: aStructure! !

!SWVisitorPluggable methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:18'!
structure: aStructure
	structure := aStructure! !

SWVisitorPluggable subclass: #SWVisitorText
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Visitor'!

!SWVisitorText methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:19'!
acceptLink: aLink
	self evaluateOn: aLink title! !

!SWVisitorText methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:19'!
acceptText: aText
	self evaluateOn: aText text! !

!SWVisitorText methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:19'!
acceptStructure: aStructure
	super acceptStructure: aStructure.
	self evaluateOn: aStructure title! !

!SWVisitorPluggable class methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:18'!
block: aBlock
	^self new
		block: aBlock;
		yourself! !

!SWVisitorPluggable methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:17'!
block
	^block! !

SWVisitorPluggable subclass: #SWVisitorLinkInternal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Visitor'!

!SWVisitorLinkInternal methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:18'!
acceptLinkInternal: anInternalLink
	self evaluateOn: anInternalLink! !

!SWVisitor methodsFor: 'visiting-structure' stamp: ''!
acceptFolder: aFolder
	self acceptPage: aFolder.
	self visitCollection: aFolder children! !

!SWVisitor methodsFor: 'visiting' stamp: ''!
visitCollection: aCollection
	aCollection 
		do: [ :child | child accept: self ]! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptLinkInternalBroken: aLinkInternal
	self acceptLinkInternal: aLinkInternal! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptLinkInternal: aLinkInternal
	self acceptLink: aLinkInternal! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptTable: aTable
	self visitCollection: aTable children! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptLinkMailTo: aLinkMailTo
	self acceptLink: aLinkMailTo! !

!SWVisitor methodsFor: 'visiting' stamp: ''!
visit: aWikiItem
	aWikiItem isNil 
		ifFalse: [ aWikiItem accept: self ]! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptText: aText! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptListItem: aListItem
	self visitCollection: aListItem children! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptTableCell: aTableCell
	self visitCollection: aTableCell children! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptHorizontalRule: aHorizontalRule! !

!SWVisitor methodsFor: 'visiting-structure' stamp: ''!
acceptPage: aPage
	self acceptStructure: aPage.
	self visit: aPage document! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptParagraph: aParagraph
	self visitCollection: aParagraph children! !

!SWVisitor class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptDocument: aDocument
	self visitCollection: aDocument children! !

SWVisitor subclass: #SWVisitorRenderer
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Visitor'!

!SWVisitorRenderer methodsFor: 'accessing' stamp: ''!
contents
	^stream contents! !

!SWVisitorRenderer class methodsFor: 'instance creation' stamp: ''!
render: aWikiItem
	^self new
		visit: aWikiItem;
		yourself! !

!SWVisitorRenderer methodsFor: 'initialization' stamp: 'chbu 11/19/2003 19:44'!
initialize
	super initialize.
	self stream: self defaultWriteStream.! !

!SWVisitorRenderer class methodsFor: 'instance creation' stamp: ''!
render: aWikiItem on: aStream
	^self new
		stream: aStream;
		visit: aWikiItem;
		yourself! !

SWVisitorRenderer subclass: #SWVisitorRendererWiki
	instanceVariableNames: 'nesting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Visitor'!

!SWVisitorRendererWiki methodsFor: 'private' stamp: ''!
emitCrWithoutDuplicates
	"Avoid emitting multiple new-line characters into the stream of the receiver. Any new-line
	characters at the beginning of the document are also avoided."

	(stream contents isEmpty or: [ stream contents last = Character cr ])
		ifFalse: [ stream cr ]! !

!SWVisitorRendererWiki methodsFor: 'visiting-document' stamp: ''!
acceptPreformatted: aPreformatted
	self emitCr.
	stream nextPut: $=.
	self 
		visitCollection: aPreformatted children
		separatedBy: [ stream cr; nextPut: $= ]! !

!SWVisitorRendererWiki methodsFor: 'visiting-document' stamp: ''!
acceptListItem: aListItem
	self emitCrWithoutDuplicates.
	stream nextPutAll: nesting.
	super acceptListItem: aListItem! !

!SWVisitorRendererWiki methodsFor: 'private' stamp: ''!
nest: aCharacter do: aBlock
	nesting addLast: aCharacter.
	aBlock ensure: [ nesting removeLast ]! !

!SWVisitorRendererWiki methodsFor: 'private' stamp: ''!
emitCr
	"Force emitting a new-line into the stream of the receiver, except at the beginning 
	of a document."

	stream contents isEmpty
		ifFalse: [ stream cr ]! !

!SWVisitorRendererWiki methodsFor: 'visiting-document' stamp: ''!
acceptHeader: aHeader
	self emitCr.
	1 to: aHeader level do: [ :index | stream nextPut: $!! ].
	stream nextPutAll: aHeader text! !

!SWVisitorRendererWiki methodsFor: 'visiting-document' stamp: ''!
acceptLink: aLink
	stream nextPut: $*.
	aLink text isNil
		ifFalse: [ stream nextPutAll: aLink text; nextPut: $> ].
	stream nextPutAll: aLink reference.
	stream nextPut: $*! !

!SWVisitorRendererWiki methodsFor: 'visiting-document' stamp: ''!
acceptUnorderedList: anUnorderedList
	self emitCr.
	self nest: $- do: [ super acceptUnorderedList: anUnorderedList ]! !

!SWVisitorRendererWiki methodsFor: 'visiting-document' stamp: ''!
acceptOrderedList: anOrderedList
	self emitCr.
	self nest: $# do: [ super acceptOrderedList: anOrderedList ]! !

!SWVisitorRendererWiki methodsFor: 'initialization' stamp: ''!
initialize
	super initialize.
	nesting := OrderedCollection new.! !

!SWVisitorRendererWiki methodsFor: 'visiting-document' stamp: ''!
acceptParagraph: aParagraph
	self emitCr.
	self	visitCollection: aParagraph children.! !

!SWVisitorRendererWiki methodsFor: 'visiting-document' stamp: ''!
acceptTableRow: aTableRow
	self emitCr.
	self visitCollection: aTableRow children.! !

!SWVisitorRendererWiki methodsFor: 'visiting-document' stamp: ''!
acceptCode: aCode
	stream nextPut: $[; nextPutAll: aCode code; nextPut: $]! !

!SWVisitorRendererWiki methodsFor: 'visiting-document' stamp: ''!
acceptHorizontalRule: aHorizontalRule
	self emitCr.
	stream nextPut: $_! !

!SWVisitorRendererWiki methodsFor: 'visiting-document' stamp: ''!
acceptTableCell: aTableRow
	stream nextPut: $|.
	self visitCollection: aTableRow children! !

!SWVisitorRendererWiki methodsFor: 'visiting-document' stamp: ''!
acceptText: aText
	aText text do: [ :each |
		each = $*
			ifTrue: [ stream nextPut: $\ ].
		stream nextPut: each ]! !

!SWVisitorRenderer methodsFor: 'visiting-structure' stamp: ''!
acceptFolder: aFolder
	self acceptPage: aFolder.! !

!SWVisitorRenderer methodsFor: 'accessing' stamp: ''!
stream: aStream
	stream := aStream! !

SWVisitorRenderer subclass: #SWVisitorRendererHtml
	instanceVariableNames: 'action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Visitor'!

!SWVisitorRendererHtml methodsFor: 'private' stamp: 'chbu 10/22/2003 18:02'!
creatableStructures
	^SWStructure allStructures
		select: [ :class | action hasPermission: class permissionAdd ]! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: ''!
acceptOrderedList: anOrderedList
	stream orderedList: [ self visitCollection: anOrderedList children ]! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: ''!
acceptParagraph: aParagraph
	aParagraph children isEmpty ifFalse: [ 
		stream paragraph: [ self visitCollection: aParagraph children ] ]! !

!SWVisitorRendererHtml methodsFor: 'accessing' stamp: ''!
action
	^action! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: ''!
acceptHorizontalRule: aHorizontalRule
	stream horizontalRule! !

!SWVisitorRendererHtml methodsFor: 'private' stamp: ''!
emitCreateTargetFor: aLinkInternal
	| structures |
	stream nextPutAll: ' ('.
	structures := self creatableStructures.
	structures isEmpty 
		ifTrue: [ stream nextPutAll: 'unauthorized to add structures' ]
		ifFalse: [
			structures 
				do: [ :each |
					stream 
						anchorWithAction: [ :context | context createChildFromClass: each titled: aLinkInternal reference creator: action structure ]
						to: action url do: each title ]
				separatedBy: [ stream nextPutAll: ', ' ] ].
	stream nextPutAll: ') '! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: ''!
acceptText: aText
	stream text: aText text! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: 'chbu 10/22/2003 18:02'!
acceptCode: aCode 
	[ stream render: (aCode evaluateWith: action) ]
		on: Exception
		do: [ :exception | exception return: (stream spanNamed: #error with: exception description) ]! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: ''!
acceptLink: aLink
	stream anchorWithUrl: aLink url do: aLink title! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: ''!
acceptLinkInternalBroken: anInternalLink
	stream spanNamed: #error with: anInternalLink title.
	anInternalLink isComposedLink 
		ifTrue: [ stream nextPutAll: ' (unable to create non-local structure)' ]
		ifFalse: [ self emitCreateTargetFor: anInternalLink ]! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: ''!
acceptHeader: aHeader
	stream heading: aHeader text level: aHeader level + 1! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: ''!
acceptListItem: aListItem
	stream listItem: [ self visitCollection: aListItem children ]! !

!SWVisitorRendererHtml class methodsFor: 'instance creation' stamp: ''!
render: aWikiItem with: anAction
	^self new
		action: anAction;
		visit: aWikiItem;
		yourself! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: ''!
acceptTableCell: aTableCell
	stream tableData: [ self visitCollection: aTableCell children ]! !

!SWVisitorRendererHtml methodsFor: 'private' stamp: 'chbu 11/19/2003 19:43'!
emitEmbeddedResource: aLinkInternal
	| target |
	target := aLinkInternal target.
	target data isEmpty ifFalse: [
		target isImage 
			ifTrue: [ ^stream image: (SWMimeView urlFor: target) alt: aLinkInternal title ].
		target isAudio | target isVideo
			ifTrue: [ ^stream embed: target title to: (SWMimeView urlFor: target) mime: target mime ].
		^stream anchorWithUrl: (SWMimeView urlFor: target) do: target title ].
	^stream anchorWithUrl: target url do: target title! !

!SWVisitorRendererHtml class methodsFor: 'instance creation' stamp: ''!
render: aWikiItem with: anAction on: aStream
	^self new
		action: anAction;
		stream: aStream;
		visit: aWikiItem;
		yourself! !

!SWVisitorRendererHtml methodsFor: 'configuration' stamp: 'chbu 10/22/2003 18:02'!
defaultWriteStream
	^SWHtmlWriteStream on: String new! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: ''!
acceptTable: aTable
	stream attributeAt: 'border' put: '0'.
	stream table: [ self visitCollection: aTable children ]! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: ''!
acceptUnorderedList: anUnorderedList
	stream unorderedList: [ self visitCollection: anUnorderedList children ]! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: ''!
acceptLinkInternal: aLinkInternal
	aLinkInternal target isEmbedded
		ifTrue: [ self emitEmbeddedResource: aLinkInternal ]
		ifFalse: [ super acceptLinkInternal: aLinkInternal ]! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: ''!
acceptTableRow: aTableRow
	stream tableRow: [ self visitCollection: aTableRow children ]! !

!SWVisitorRendererHtml methodsFor: 'accessing' stamp: ''!
action: anAction
	action := anAction! !

!SWVisitorRendererHtml methodsFor: 'visiting-document' stamp: ''!
acceptPreformatted: aPreformatted
	stream preformatted: [
		self visitCollection: aPreformatted children separatedBy: [ stream cr ] ]! !

!SWVisitorRenderer methodsFor: 'configuration' stamp: ''!
defaultWriteStream
	^WriteStream on: String new! !

!SWVisitor methodsFor: 'visiting-document' stamp: ''!
acceptCode: aCode! !

Object subclass: #SWCache
	instanceVariableNames: 'dataTable currentKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Server'!

!SWCache methodsFor: 'initialization' stamp: ''!
initialize
	currentKey := self defaultStartKey.
	self reset.! !

SWCache subclass: #SWExpiringCache
	instanceVariableNames: 'ageTable ttl check step'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Server'!

!SWExpiringCache methodsFor: 'accessing' stamp: ''!
ttl
	^ttl! !

!SWExpiringCache methodsFor: 'private' stamp: ''!
ageTable
	^ageTable! !

!SWExpiringCache methodsFor: 'private' stamp: ''!
step
	^step! !

!SWExpiringCache methodsFor: 'private' stamp: ''!
step: anInteger
	step := anInteger! !

!SWExpiringCache methodsFor: 'accessing' stamp: ''!
check: anInteger
	check := anInteger! !

!SWExpiringCache methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: '; ttl: '; print: self ttl.! !

!SWExpiringCache methodsFor: 'accessing' stamp: ''!
check
	^check! !

!SWExpiringCache methodsFor: 'private' stamp: 'chbu 10/22/2003 09:30'!
now
	^TimeStamp now
		asSeconds! !

!SWExpiringCache methodsFor: 'private' stamp: ''!
ageTable: aCollection
	ageTable := aCollection! !

!SWExpiringCache methodsFor: 'initialize' stamp: ''!
initialize
	super initialize.
	self step: self defaultStartStep.
	self ttl: self defaultTimeToLive.
	self check: self defaultCheck.! !

!SWExpiringCache methodsFor: 'configuration' stamp: ''!
defaultTimeToLive
	^30 * 60! !

!SWExpiringCache methodsFor: 'private' stamp: ''!
removeExpired
	| stamp old |
	step := self defaultStartStep.
	stamp := self now - ttl.
	old := self ageTable select: [ :association |
		association value <= stamp ].
	old keys do: [ :key |
		dataTable removeKey: key.
		ageTable removeKey: key ]! !

!SWExpiringCache methodsFor: 'configuration' stamp: ''!
defaultStartStep
	^1! !

!SWExpiringCache methodsFor: 'configuration' stamp: ''!
defaultCheck
	^100! !

!SWExpiringCache methodsFor: 'initialize' stamp: ''!
reset
	super reset.
	self ageTable: self defaultAgeTable.! !

!SWExpiringCache methodsFor: 'accessing' stamp: ''!
ttl: anInteger
	ttl := anInteger! !

!SWExpiringCache methodsFor: 'configuration' stamp: ''!
defaultAgeTable
	^Dictionary new! !

!SWExpiringCache methodsFor: 'accessing-entries' stamp: ''!
put: anObject
	step >= check
		ifTrue: [ self removeExpired ]
		ifFalse: [ step := step + 1 ].
	dataTable at: self next put: anObject.
	ageTable at: currentKey put: self now.
	^currentKey! !

SWCache subclass: #SWFifoCache
	instanceVariableNames: 'max shrink'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Server'!

!SWFifoCache methodsFor: 'private' stamp: ''!
shrinkTable
	| keys |
	keys := dataTable keys asSortedCollection.
	1 to: shrink do: [ :index |
		dataTable removeKey: (keys at: index) ].! !

!SWFifoCache methodsFor: 'accessing' stamp: ''!
max: anInteger
	max := anInteger! !

!SWFifoCache methodsFor: 'accessing' stamp: ''!
max
	^max! !

!SWFifoCache methodsFor: 'configuration' stamp: ''!
defaultShrink
	^100! !

!SWFifoCache methodsFor: 'configuration' stamp: ''!
defaultMax
	^5000! !

!SWFifoCache methodsFor: 'accessing' stamp: ''!
shrink: anInteger
	shrink := anInteger! !

!SWFifoCache methodsFor: 'accessing' stamp: ''!
shrink
	^shrink! !

!SWFifoCache methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: '; max: '; print: self max.
	aStream nextPutAll: '; shrink: '; print: self shrink.! !

!SWFifoCache methodsFor: 'accessing-data' stamp: ''!
put: anObject
	dataTable size > max 
		ifTrue: [ self shrinkTable ].
	dataTable at: self next put: anObject.
	^currentKey! !

!SWFifoCache methodsFor: 'initialize' stamp: ''!
initialize
	super initialize.
	self max: self defaultMax.
	self shrink: self defaultShrink.! !

!SWCache methodsFor: 'initialization' stamp: ''!
reset
	dataTable := self defaultDataTable.! !

!SWCache methodsFor: 'accessing' stamp: ''!
isEmpty
	^dataTable isEmpty! !

!SWCache methodsFor: 'accessing' stamp: ''!
size
	^dataTable size! !

!SWCache methodsFor: 'accessing-entries' stamp: 'chbu 11/19/2003 19:39'!
get: aKey
	^dataTable 
		at: aKey 
		ifAbsent: [ nil ]! !

!SWCache methodsFor: 'configuration' stamp: ''!
defaultStartKey
	^0! !

!SWCache methodsFor: 'configuration' stamp: ''!
defaultDataTable
	^Dictionary new! !

!SWCache methodsFor: 'accessing-entries' stamp: ''!
put: anObject
	^self subclassResponsibility! !

!SWCache methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' size: '; print: self size.! !

!SWCache class methodsFor: 'instance-creation' stamp: ''!
new
	^super new
		initialize;
		yourself! !

!SWCache methodsFor: 'private' stamp: ''!
next
	^currentKey := currentKey + 1! !

Error subclass: #SWDuplicatedStructure
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Structure'!

!SWDuplicatedStructure methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:24'!
messageText
	^'Invalid or duplicated structure name.'! !

Error subclass: #SWUnauthorizedError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Server'!

!SWUnauthorizedError methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:22'!
messageText
	^'Unauthorized to execute the requested action.'! !

SWWikiItem subclass: #SWPropertyManager
	instanceVariableNames: 'properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Server'!

!SWPropertyManager commentStamp: '<historical>' prior: 0!
I am a property-manager to hold tags and additional properties to different entities inside the wiki. I respond basically to the dictionary protocol, but I might get visited with a Visitor.
!

!SWPropertyManager methodsFor: 'accessing' stamp: ''!
properties: aDictionary
	properties := aDictionary! !

!SWPropertyManager methodsFor: 'copying' stamp: 'chbu 10/23/2003 18:05'!
postCopy
	super postCopy.
	properties := properties deepCopy! !

!SWPropertyManager methodsFor: 'testing' stamp: ''!
isEmpty
	^properties isEmpty! !

!SWPropertyManager methodsFor: 'removing' stamp: ''!
remove: aKey
	"Remove a property from the receiver with aKey. Return the removed property or
	nil if there is no such element."

	^self 
		remove: aKey 
		ifAbsent: [ nil ]! !

!SWPropertyManager methodsFor: 'initialization' stamp: ''!
initialize
	super initialize.
	properties := Dictionary new! !

!SWPropertyManager methodsFor: 'accessing' stamp: ''!
properties
	^properties! !

!SWPropertyManager methodsFor: 'accessing-properties' stamp: ''!
at: aKey ifAbsent: anExceptionBlock
	"Return the value of the property with aName. If there is no such property,
	anExceptionBlock is evaluated."

	^properties
		at: aKey
		ifAbsent: anExceptionBlock! !

!SWPropertyManager methodsFor: 'visiting' stamp: ''!
accept: aVisitor
	aVisitor acceptProperties: self! !

!SWPropertyManager methodsFor: 'accessing-properties' stamp: ''!
at: aKey put: aValue
	"Updates the property aKey with aValue. If the property does not exist a default
	one is created. The updated or added property is returned."

	^properties 
		at: aKey 
		put: aValue! !

!SWPropertyManager methodsFor: 'removing' stamp: ''!
remove: aKey ifAbsent: anExceptionBlock
	"Remove a property from the receiver with aKey. Return the removed property or
	anExceptionBlock is evaluated if there is no such element."

	^properties
		removeKey: aKey
		ifAbsent: anExceptionBlock! !

!SWPropertyManager methodsFor: 'enumerating' stamp: ''!
do: aBlock
	"Evaluate aBlock with each of the receiver's properties (key and value) as 
	the argument."

	properties 
		keysAndValuesDo: aBlock! !

!SWPropertyManager methodsFor: 'testing' stamp: ''!
includes: aKey
	"Test if the receiver has got a property with aKey."

	^properties includesKey: aKey! !

!SWPropertyManager methodsFor: 'testing' stamp: ''!
size
	^properties size! !

!SWPropertyManager methodsFor: 'accessing-properties' stamp: ''!
at: aKey
	"Return the value of the property with aKey. If there is no such property,
	nil is returned."

	^self 
		at: aKey 
		ifAbsent: [ nil ]! !

SWWikiItem subclass: #SWStructure
	instanceVariableNames: 'title parent predecessor timestamp version properties roles dependents id'
	classVariableNames: 'Permissions Actions'
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Structure'!

SWStructure class
	instanceVariableNames: 'actions permissions'!

!SWStructure commentStamp: '<historical>' prior: 0!
The structure is the basic entity of SmallWiki, representing the model of a single page. A structure is identified by exactly one URL and is usually included in a composite-tree of other structures. The three concrete subclasses of \texttt{Structure} are: \texttt{Page} and \texttt{Resource} as components and the \texttt{Folder} as composite. In fact \texttt{Structure} should not only be the subclass of \texttt{WikiItem}, but also of \texttt{Model}. As the visiting aspect, however, is far more important, the messages provided in \texttt{Model} have been copied from this system class.

A structure provides basic navigational accessors to its parents, children and sisters in the wiki-tree. The basic serving is done with the chain-of-responsibilities design-pattern in the serving protocol. The resolving protocol provides messages to look-up other structure-items using their name.

All the structures have a title, a back-reference to their parent and might contain user defined properties, what is something like a dictionary containing symbols as key and any other objects as values. Structures are versioned automatically using a reference pointing to the previous version of the same page.  Make sure to override the message #postCopy to make it work correctly.

!

!SWStructure methodsFor: 'accessing' stamp: ''!
roles: aCollection
	roles := aCollection! !

!SWStructure methodsFor: 'accessing' stamp: ''!
version
	"Return the version-number of the receiver, the numbering starts with version 0."

	^version! !

!SWStructure methodsFor: 'testing' stamp: ''!
isSuccessor
	"Answer whether the receiver has got a previous version in the history."

	^predecessor notNil! !

!SWStructure class methodsFor: 'accessing-security' stamp: 'chbu 10/27/2003 20:05'!
unregisterPermission: aPermission
	^self permissions remove: aPermission! !

!SWStructure class methodsFor: 'accessing' stamp: 'chbu 10/24/2003 17:31'!
title
	"Returns the title of this component. Do override this message to specify
	a differnt name."
	| title |

	title := self name asString.
	^title copyFrom: 3 to: title size! !

!SWStructure methodsFor: 'versions' stamp: ''!
nextVersionBecome: aStructure
	"This message creates a copy of the receiver and puts aStructure into the history. 
	References pointing to the receiver will be still valid, as the current version stays
	the same object all the time. Right now the new version must be of the same class
	than the receiver, else an exception is thrown."

	| previous |
	self class == aStructure class
		ifFalse: [ self error: 'Unable to version class changes in history.' ].
	previous := self copy.
	1 to: aStructure class instSize do: [ :index |
		self instVarAt: index put: (aStructure instVarAt: index) ].
	self predecessor: previous.
	self version: previous version + 1.! !

!SWStructure methodsFor: 'resolving' stamp: ''!
privateParsePath: aStringPath
	| readStream pathCollection identifier |
	pathCollection := OrderedCollection new.
	readStream := aStringPath readStream.
	[ readStream atEnd ] whileFalse: [
		identifier := readStream upTo: $/.
		pathCollection add: identifier asWikiIdentifier ].
	^pathCollection! !

!SWStructure methodsFor: 'serving' stamp: ''!
process: aRequest
	"Process a basic request. First the security information contained in the request
	is updated, then it is decided if the request should be handled by the current 
	component or one of its children."

	self processSecurity: aRequest.
	^aRequest urlAtEnd
		ifTrue: [ self processSelf: aRequest ]
		ifFalse: [ self processChild: aRequest ]! !

!SWStructure class methodsFor: 'accessing-actions' stamp: 'chbu 10/27/2003 20:06'!
unregisterAction: anActionClass
	^self actions remove: anActionClass! !

!SWStructure class methodsFor: 'instance creation' stamp: ''!
version: aStructure
	"Create a new version of the given structure. At the moment the new version is
	an instance of the same class as aStructure, but that will change in the future."

	^aStructure
		nextVersion! !

!SWStructure methodsFor: 'testing' stamp: ''!
isComposite
	"Return true if the structure has got the possiblity to hold children."

	^false! !

!SWStructure methodsFor: 'properties-inherited' stamp: ''!
propertyAt: aKey put: aValue
	"Set the property aKey to aValue in the receiver. This message does the same
	as #localPropertyAt:put: and is here simply for conveniance."

	^self 
		localPropertyAt: aKey 
		put: aValue! !

!SWStructure methodsFor: 'accessing-calculated' stamp: ''!
versions
	"Return an ordered-collection containing the receiver and all its older versions."

	| versions |
	versions := OrderedCollection with: self.
	[ versions last predecessor isNil ]
		whileFalse: [ versions addLast: versions last predecessor ].
	^versions! !

!SWStructure methodsFor: 'notification' stamp: 'chbu 10/24/2003 16:29'!
changed: anAspectSymbol with: aParameter
	super changed: anAspectSymbol with: aParameter.
	self isRoot
		ifFalse: [ self parent changed: anAspectSymbol with: aParameter ]! !

!SWStructure methodsFor: 'accessing' stamp: ''!
predecessor
	"Return the previous version in the history of the receiver. If there is no history information
	available nil is returned."

	^predecessor! !

!SWStructure class methodsFor: 'initialize' stamp: 'chbu 10/27/2003 20:04'!
initialize
	" SWStructure initialize "

	self initializeActions.
	self initializePermissions.! !

!SWStructure methodsFor: 'versions' stamp: ''!
versionRestore: anInteger
	"Restore the version anInteger. In other words: the version anInteger will become the
	current one, but all the other modifications are still kept in history."

	| structure |
	structure := self versionNumber: anInteger.
	structure isNil
		ifFalse: [ self nextVersionBecome: structure ]! !

SWStructure subclass: #SWPage
	instanceVariableNames: 'document'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Structure'!

!SWPage commentStamp: '<historical>' prior: 0!
A page is the most important and probably the most used class of the Structure hierarchy. As a sole entity it  contains a composite of documents modeling the contents of the page that the user entered using the wiki-syntax. When initializing the instance a default document will be created to make the user aware of the newly created page.
!

!SWPage methodsFor: 'configuration' stamp: 'chbu 10/22/2003 17:18'!
defaultVisitor
	"Return the default visitor used when rendering this document to html."

	^SWVisitorRendererHtml! !

!SWPage methodsFor: 'initialize' stamp: ''!
initialize
	super initialize.
	document := self defaultDocument! !

!SWPage class methodsFor: 'accessing-permissions' stamp: ''!
permissionCode
	^self permissionAt: 'Page Code'! !

!SWPage methodsFor: 'visiting' stamp: ''!
accept: aVisitor
	aVisitor acceptPage: self! !

!SWPage class methodsFor: 'initialize' stamp: 'chbu 10/24/2003 17:14'!
initializePermissions
	" SWPage initializePermissions "

	self registerPermission: (SWPermission name: 'Page Add').
	self registerPermission: (SWPermission name: 'Page Remove').
	self registerPermission: (SWPermission name: 'Page Copy').
	self registerPermission: (SWPermission name: 'Page Move').
	self registerPermission: (SWPermission name: 'Page Edit').
	self registerPermission: (SWPermission name: 'Page View').
	self registerPermission: (SWPermission name: 'Page History').
	self registerPermission: (SWPermission name: 'Page Template').
	self registerPermission: (SWPermission name: 'Page Code').! !

!SWPage class methodsFor: 'instance-creation' stamp: 'chbu 10/22/2003 16:59'!
title: aTitleString parse: aWikiString
	| document |
	document := SWWikiParser parse: aWikiString readStream for: SWAction new.
	^self new
		title: aTitleString;
		document: document;
		yourself! !

!SWPage class methodsFor: 'configuration' stamp: 'chbu 10/22/2003 09:58'!
defaultViewAction
	^SWPageView! !

!SWPage methodsFor: 'configuration' stamp: 'chbu 10/22/2003 09:36'!
defaultDocument
	"Return the default document used when a new page is created."

	^SWDocument new
		add: (SWParagraph new
			add: (SWText newText: 'Edit your page ...');
			yourself);
		yourself! !

!SWPage methodsFor: 'accessing' stamp: ''!
document
	"Return the current document of the page."

	^document! !

SWPage subclass: #SWFolder
	instanceVariableNames: 'children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Structure'!

!SWFolder commentStamp: '<historical>' prior: 0!
The folder groups a number of children. \texttt{Folder} is a subclass of \texttt{Page}, therefor they also contain a document that might be used to describe the contents.!

!SWFolder methodsFor: 'children-structure' stamp: ''!
copy: aStructure
	"This message is basically the same as #add: but it creates a copy of the structure 
	and makes sure that the title is unique within the receiver before adding."

	| unique child |
	unique := self uniqueTitle: aStructure title.
	child := aStructure copy.
	child title: unique.
	self add: child.
	^child! !

!SWFolder class methodsFor: 'initialize' stamp: 'chbu 10/22/2003 15:45'!
initializeActions
	super initializeActions.
	self registerAction: SWFolderEdit.! !

!SWFolder methodsFor: 'configuration' stamp: 'chbu 10/22/2003 09:32'!
defaultDocument
	"Return the default document used when a new folder is created."

	^SWDocument new
		add: (SWParagraph new
			add: (SWText newText: 'Edit a description of your folder and add some children ...');
			yourself);
		add: (SWParagraph new);
		add: (SWParagraph new
			add: (SWCode newCode: self defaultChildrenListCode);
			yourself);
		yourself! !

!SWFolder methodsFor: 'children-structure' stamp: 'chbu 10/22/2003 09:50'!
add: aStructure
	"Add aStructure as a child to the receiver. A DuplicatedStructure exception is raised in case there is already a child with the same name."

	(self includes: aStructure title)
		ifTrue: [ SWDuplicatedStructure signal: aStructure title ].
	aStructure parent: self.
	children add: aStructure.
	^aStructure! !

!SWFolder methodsFor: 'children-structure' stamp: 'chbu 11/12/2003 23:29'!
remove: aStructure
	"Remove aStructure from the list of children of the receiver. In case there is no such 
	child an exception is raised."

	aStructure parent: nil.
	^(children includes: aStructure) 
		ifTrue:	[ children remove: aStructure ]
		ifFalse:	[ self class notFound signal ]! !

!SWFolder methodsFor: 'navigation' stamp: ''!
next: aStructure 
	"Return the next node of the receiver's child aStructure."

	^[ children after: aStructure ] 
		on: Error 
		do: [ :err | err return: nil ]! !

!SWFolder methodsFor: 'children-accessing' stamp: 'chbu 11/19/2003 19:42'!
includes: aString
	"Return true if the receiver contains a child with the id aString."

	self 
		at: aString
		ifAbsent: [ ^false ].
	^true! !

!SWFolder methodsFor: 'configuration' stamp: ''!
defaultChildrenListCode
	^' "If you are not logged-in as administrator, please remove everything between the square-brackets before saving the page!!"
structure children isEmpty ifFalse: [ 
	html unorderedList: [ 
		structure children do: [ :each | 
			html listItem: [ html anchorWithUrl: each url do: each title ] ] ] ]. 
nil'! !

!SWFolder class methodsFor: 'initialize' stamp: 'chbu 11/19/2003 23:01'!
initialize
	" SWFolder initialize "

	super initialize! !

!SWFolder methodsFor: 'serving' stamp: ''!
processChild: aRequest
	| child |
	child := self at: aRequest urlNext.
	^child isNil 
		ifFalse: [ child process: aRequest ]
		ifTrue: [ self notFound: aRequest ]! !

!SWFolder methodsFor: 'navigation' stamp: ''!
previous: aStructure 
	"Return the previous node of the receiver's child aStructure."

	^[ children before: aStructure ] 
		on: Error 
		do: [ :err | err return: nil ]! !

!SWFolder methodsFor: 'accessing' stamp: ''!
children
	"Return a collection of all the children of the receiver."

	^children! !

!SWFolder methodsFor: 'children-accessing' stamp: 'chbu 11/12/2003 20:07'!
uniqueTitle: aString
	"Proposes an unique name for a child within the receiver using aString base name."

	| number unique |
	number := 1. unique := aString.
	[ self includes: unique ] whileTrue: [
		unique := aString , number asString.
		number := number + 1 ].
	^unique! !

!SWFolder class methodsFor: 'initialize' stamp: 'chbu 10/24/2003 17:14'!
initializePermissions
	" SWFolder initializePermissions "

	self registerPermission: (SWPermission name: 'Folder Add').
	self registerPermission: (SWPermission name: 'Folder Remove').
	self registerPermission: (SWPermission name: 'Folder Copy').
	self registerPermission: (SWPermission name: 'Folder Move').
	self registerPermission: (SWPermission name: 'Folder Edit').
	self registerPermission: (SWPermission name: 'Folder View').
	self registerPermission: (SWPermission name: 'Folder History').
	self registerPermission: (SWPermission name: 'Folder Template').
	self registerPermission: (SWPermission name: 'Folder Code').! !

!SWFolder methodsFor: 'visiting' stamp: ''!
accept: aVisitor
	aVisitor acceptFolder: self! !

!SWFolder methodsFor: 'resolving' stamp: ''!
privateResolveChild: aString
	^self at: aString! !

!SWFolder methodsFor: 'children-accessing' stamp: 'chbu 11/19/2003 19:42'!
at: aString ifAbsent: aBlock
	"Return child of the receiver with id aString or nil if absent."

	| lookupKey |
	lookupKey := aString asWikiIdentifier.
	^children
		detect: [ :child | child id = lookupKey ]
		ifNone: aBlock! !

!SWFolder methodsFor: 'copying' stamp: ''!
postCopy
	super postCopy.
	children := children copy.! !

!SWFolder methodsFor: 'testing' stamp: ''!
isComposite
	^true! !

!SWFolder methodsFor: 'configuration' stamp: ''!
defaultAddTarget
	^self! !

!SWFolder methodsFor: 'initialize' stamp: ''!
initialize
	super initialize.
	children := self defaultChildrenCollection! !

!SWFolder methodsFor: 'children-accessing' stamp: ''!
at: aString
	"Return child of the receiver with id aString or nil if absent."

	^self 
		at: aString
		ifAbsent: [ nil ]! !

!SWFolder methodsFor: 'configuration' stamp: ''!
defaultChildrenCollection
	"When changing this message, you should modify the following messages: 
	#at:ifAbsent:, #add:, #remove: and #children."

	^OrderedCollection new! !

!SWPage class methodsFor: 'initialize' stamp: 'chbu 11/19/2003 23:01'!
initialize
	" SWPage initialize "

	super initialize! !

!SWPage class methodsFor: 'initialize' stamp: 'chbu 10/22/2003 15:44'!
initializeActions
	super initializeActions.
	self registerAction: SWPageView.
	self registerAction: SWPageEdit.
	self registerAction: SWPageHistory.! !

!SWPage class methodsFor: 'configuration' stamp: 'chbu 10/22/2003 09:58'!
defaultEditAction
	^SWPageEdit! !

!SWPage methodsFor: 'copying' stamp: ''!
postCopy
	super postCopy.
	document := document copy! !

!SWPage methodsFor: 'accessing' stamp: ''!
document: aDocument
	document := aDocument! !

!SWStructure methodsFor: 'as yet unclassified' stamp: 'chbu 11/19/2003 19:52'!
stylesheet
	| stylesheet |
	^self 
		propertyAt: #internalStylesheet
		ifAbsent: [ 
			self stylesheet: (stylesheet := SWTemplate defaultStylesheet).
			stylesheet ]! !

!SWStructure methodsFor: 'accessing' stamp: ''!
roles
	"Return a collection of roles which are applied when the receiver processes a query. All the roles of the current user are replaced with the corresponding ones returned by this message."

	^roles! !

!SWStructure methodsFor: 'versions' stamp: ''!
versionRevert: anInteger
	"Revert to the version anInteger in history. All the newer modifications will be
	lost."

	| structure |
	structure := self versionNumber: anInteger.
	structure isNil ifFalse: [ 
		self nextVersionBecome: structure.
		self predecessor: structure predecessor ]! !

!SWStructure methodsFor: 'as yet unclassified' stamp: 'chbu 10/22/2003 09:20'!
templates: aCollection
	aCollection isNil
		ifTrue: [ self localProperties remove: #internalTemplates ]
		ifFalse: [ self localProperties at: #internalTemplates put: aCollection ]! !

!SWStructure methodsFor: 'accessing' stamp: ''!
id
	"Return the id of the receiver, that is a string build from the title of the structure. The id is used to identify a structure within its parent and therefore has to be unique."

	"The lazy initialization should be removed, it is just here for compatibility"
	id isNil 
		ifTrue: [ id := title asWikiIdentifier ]. 
	^id! !

!SWStructure methodsFor: 'properties-local' stamp: ''!
localPropertyAt: aKey ifAbsent: anExceptionBlock
	"Return the value of the property of the receiver with aKey. If there is no such 
	property, anExceptionBlock is evaluated."

	^properties 
		at: aKey 
		ifAbsent: anExceptionBlock! !

!SWStructure methodsFor: 'resolving' stamp: ''!
resolveTo: aStringPath
	"Start the resolving-process at the receiver with the resolving-algorithm depending on the count of identifiers in aPathString: 

\begin{enumerate}
	\item if the first character of the path is a separator the look-up is started in the root-node and processed downwards. 
	\item if the first character is not a separator ...
	\begin{enumerate}
		\item and an empty path is given, the receiver is returned.
		\item and a path with exactly one entry is given, a child with that name is looked for.
		\item and a path with one or more entries is given, a look-up is started in the parent of the receiver. 
	\end{enumerate}
\end{enumerate}

The first matching structure is returned or nil if no appropriate item could be located in the tree. To see a bunch of examples about the use of this message, have a look at the tests in protocol \texttt{resting-resolving} of the class \texttt{StructureTests}."

	| path |
	path := self privateParsePath: aStringPath.
	^path isEmpty
		ifFalse: [ self privateResolve: path ]
		ifTrue: [ self ]! !

!SWStructure methodsFor: 'private' stamp: ''!
myDependents: aDependentsList
	dependents := aDependentsList! !

!SWStructure class methodsFor: 'configuration' stamp: ''!
defaultViewAction
	"Return the action that should be executed when nothing else is specified. Do
	override this message in all your own components."

	^self subclassResponsibility! !

!SWStructure methodsFor: 'as yet unclassified' stamp: 'chbu 10/22/2003 09:20'!
stylesheet: aString
	aString isNil
		ifTrue: [ self localProperties remove: #internalStylesheet ]
		ifFalse: [ self localProperties at: #internalStylesheet put: aString ]! !

!SWStructure methodsFor: 'serving-errors' stamp: 'chbu 10/22/2003 17:48'!
notFound: aRequest
	self processAction: (SWErrorNotFound 
		request: aRequest structure: self)! !

!SWStructure methodsFor: 'properties-local' stamp: ''!
localProperties
	"Return the properties of the receiver."

	^properties! !

!SWStructure methodsFor: 'versions' stamp: ''!
versionTruncate: anInteger
	"Truncate all the history information behind the version anInteger."

	| structure |
	structure := self versionNumber: anInteger.
	structure isNil
		ifFalse: [ structure predecessor: nil ]! !

!SWStructure class methodsFor: 'accessing-permissions' stamp: ''!
permissionMove
	^self permissionAt: '* Move'! !

!SWStructure methodsFor: 'testing' stamp: ''!
isEmbedded
	"Return true if the structure should be embedded into documents
	referencing the receiver."

	^false! !

!SWStructure class methodsFor: 'accessing-actions' stamp: 'chbu 10/27/2003 20:06'!
registerAction: anActionClass
	self actions add: anActionClass! !

!SWStructure methodsFor: 'serving' stamp: ''!
processSecurity: aRequest
	"Update the roles of the current user according to the current role configuration. 
	See #updateRoles: in the \texttt{User} class for additional information."

	aRequest updateRoles: roles! !

!SWStructure methodsFor: 'configuration' stamp: ''!
defaultAddTarget
	"Return the target where to put new children. The default implementation returns the
	parent of the receiver, but structures that contain children usually want to return self."

	^self parent! !

!SWStructure methodsFor: 'properties-inherited' stamp: ''!
propertyAt: aKey
	"Return the value of the property of the receiver with aKey. If there is no such property
	defined, the look-up is retried in the parent of the receiver."

	^self 
		propertyAt: aKey 
		ifAbsent: [ nil ]! !

!SWStructure class methodsFor: 'accessing-permissions' stamp: ''!
permissionTemplate
	^self permissionAt: '* Template'! !

!SWStructure class methodsFor: 'accessing-permissions' stamp: ''!
permissionView
	^self permissionAt: '* View'! !

!SWStructure class methodsFor: 'accessing-security' stamp: 'chbu 10/27/2003 20:05'!
registerPermission: aPermission
	self permissions add: aPermission! !

!SWStructure class methodsFor: 'initialize' stamp: ''!
initializePermissions
	"Permissions are usually not inherited form the supperclasses, therefor you don't need
	to call super when overriding this message."! !

!SWStructure methodsFor: 'accessing-navigation' stamp: ''!
first
	"Return the first node of the receiver's parent."

	^self isRoot
		ifTrue: [ self ]
		ifFalse: [ parent children first ]! !

!SWStructure methodsFor: 'properties-inherited' stamp: ''!
propertyAt: aKey ifAbsent: anExceptionBlock
	"Return the value of the property of the receiver with aKey. If there is no 
	such property defined, the look-up is retried in the parent of the receiver. 
	If no such property could be found, anExceptionBlock is evaluated."

	^properties 
		at: aKey 
		ifAbsent: [
			self isRoot
				ifTrue: [ anExceptionBlock value ]
				ifFalse: [ parent propertyAt: aKey ifAbsent: anExceptionBlock ] ]! !

!SWStructure class methodsFor: 'accessing-security' stamp: 'chbu 10/27/2003 20:05'!
permissionAt: aString ifAbsent: aBlock
	^self permissions 
		detect: [ :permission | aString match: permission name ]
		ifNone: aBlock! !

!SWStructure methodsFor: 'resolving' stamp: ''!
privateResolvePath: aCollection
	"Resolve a whole path starting at the receiver. If there is an error matching
	the path, nil is returned."

	| structure pathStream  |
	structure := self.
	pathStream := aCollection readStream.
	[ pathStream atEnd or: [ structure isNil ] ] 
		whileFalse: [ structure := structure privateResolveChild: pathStream next ].
	^structure! !

!SWStructure methodsFor: 'as yet unclassified' stamp: 'chbu 11/19/2003 19:40'!
templates
	| templates |
	^self 
		propertyAt: #internalTemplates
		ifAbsent: [ 
			self templates: (templates := SWTemplate defaultTemplates).
			templates ]! !

!SWStructure methodsFor: 'accessing' stamp: ''!
version: anInteger
	version := anInteger! !

!SWStructure class methodsFor: 'instance creation' stamp: ''!
title: aString
	"Create a new structure with aString as title."

	^self new
		title: aString;
		yourself! !

!SWStructure methodsFor: 'accessing-calculated' stamp: ''!
root
	"Return the root node of the receiver."

	| structure |
	structure := self.
	[ structure isRoot ]
		whileFalse: [ structure := structure parent ].
	^structure! !

!SWStructure methodsFor: 'visiting' stamp: ''!
accept: aVisitor
	aVisitor acceptStructure: self! !

!SWStructure methodsFor: 'serving-errors' stamp: 'chbu 10/22/2003 17:48'!
unauthorized: aRequest
	self processAction: (SWErrorUnauthorized 
		request: aRequest structure: self)! !

!SWStructure methodsFor: 'copying' stamp: 'chbu 10/22/2003 17:48'!
postCopy
	"Copy a selection of the instance-variables and update the time-stamp of the receiver. Subclasses should override this message to do a deep-copy of their data and call super. This is the key-message to make the versioning mechanism work properly."

	super postCopy.
	self breakDependents.
	timestamp := TimeStamp now.
	properties := properties copy.
	roles := roles copy.! !

!SWStructure class methodsFor: 'accessing-permissions' stamp: ''!
permissionHistory
	^self permissionAt: '* History'! !

!SWStructure class methodsFor: 'reflection' stamp: ''!
allStructures
	^self allSubclasses
		asSortedCollection: [ :x :y | x name < y name ]! !

!SWStructure methodsFor: 'accessing' stamp: ''!
predecessor: aStructure
	predecessor := aStructure! !

!SWStructure methodsFor: 'accessing' stamp: ''!
parent: aStructure
	parent := aStructure.! !

!SWStructure methodsFor: 'serving' stamp: 'chbu 10/22/2003 17:44'!
processAction: anAction 
	"Executes the action on myself and catch basic errors."

	^[ anAction execute ] 
		on: SWUnauthorizedError
		do: [ :error | error return: (self unauthorized: anAction request) ]! !

!SWStructure methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' title: '; print: title! !

!SWStructure methodsFor: 'accessing-calculated' stamp: ''!
url
	"Return a unix-path string representing the URL of the receiver. The URL is unique
	within a wiki-tree and contains the id of the receiver and all its parents ids,
	except for the root."

	^self parents
		removeFirst;
		inject: '/' into: [ :output :structure | output , structure id , '/' ]! !

!SWStructure methodsFor: 'accessing' stamp: 'chbu 11/19/2003 19:40'!
title: aString
	"Set the title of the receiver. If the receiver is already integrated into a wiki-tree use #renameTo: instead that assures that the title is unique and that all the links are updated properly."

	id := aString asWikiIdentifier.	
	title := aString.! !

!SWStructure class methodsFor: 'initialize' stamp: 'chbu 10/27/2003 20:05'!
initializeActions
	"When creating a subclass of \texttt{Structure} you usuallay want to inherit all 
	the common actions from the base class, therefor don't forget calling super
	when overriding this message."

	self actions
		add: SWLogin;
		add: SWLogout;
		add: SWSearch;
		add: SWTemplateEdit;
		add: SWRecentChanges;
		add: SWPreviousStructure;
		add: SWParentStructure;
		add: SWNextStructure.! !

!SWStructure class methodsFor: 'accessing-security' stamp: ''!
permissionAt: aString
	^self 
		permissionAt: aString
		ifAbsent: [ nil ]! !

!SWStructure methodsFor: 'versions' stamp: ''!
versionNumber: anInteger
	"Return the version anInteger of the receiver or nil if not present."

	^self versions
		detect: [ :item | item version = anInteger ]
		ifNone: [ self error: 'Invalid version number' ].! !

SWStructure subclass: #SWResource
	instanceVariableNames: 'data mimetype embed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Structure'!

!SWResource commentStamp: '<historical>' prior: 0!
A resource might contain any data, like images, videos, sound, pdf or zip files. In fact it can be anything that you want to include within your pages or you want to provide as a possibility to download. 

The mime-type of the data is used to determine how the given resource should be rendered. As an example images and videos should be displayed inside the html document, whereas zip-files are only references as a link to allow the user to download the file.!

!SWResource methodsFor: 'accessing' stamp: ''!
mimetype: aString
	mimetype := aString! !

!SWResource methodsFor: 'configuration' stamp: ''!
defaultData
	^ByteArray new! !

!SWResource methodsFor: 'storage' stamp: ''!
generateFilename
	^String streamContents: [ :stream |
		self parents
			do: [ :structure | stream nextPutAll: structure id ]
			separatedBy: [ stream nextPut: $_ ].
		stream nextPut: $..
		stream nextPutAll: version printString ]! !

!SWResource class methodsFor: 'initialize' stamp: 'chbu 10/24/2003 17:15'!
initializePermissions
	" SWResource initializePermissions "

	self registerPermission: (SWPermission name: 'Resource Add').
	self registerPermission: (SWPermission name: 'Resource Remove').
	self registerPermission: (SWPermission name: 'Resource Copy').
	self registerPermission: (SWPermission name: 'Resource Move').
	self registerPermission: (SWPermission name: 'Resource Edit').
	self registerPermission: (SWPermission name: 'Resource View').
	self registerPermission: (SWPermission name: 'Resource History').
	self registerPermission: (SWPermission name: 'Resource Template').! !

!SWResource methodsFor: 'accessing' stamp: ''!
embed: aBoolean
	embed := aBoolean! !

!SWResource methodsFor: 'configuration' stamp: ''!
defaultMimetype
	^String new! !

!SWResource class methodsFor: 'configuration' stamp: 'chbu 10/22/2003 09:59'!
defaultViewAction
	^SWResourceEdit! !

!SWResource methodsFor: 'accessing' stamp: ''!
embed
	^embed! !

!SWResource methodsFor: 'testing' stamp: ''!
isAudio
	"Return true if the mimetype of the receiver is audio-data. This message will match 
	types like: audio/basic, audio/tone, audio/mpeg, etc."

	^'audio/*' match: mimetype! !

!SWResource methodsFor: 'storage' stamp: 'svp 11/11/2003 14:42'!
loadBinaryData: aFilename
	| stream |
	(stream := FileStream readOnlyFileNamed: aFilename)
		ifNil: [^self error: 'problem loading resource'].
	[ self data: stream contents ]
		ensure: [ stream close ]! !

!SWResource methodsFor: 'storage' stamp: 'svp 11/11/2003 14:40'!
saveBinaryData: aFilename
	| stream |

	(stream := FileStream forceNewFileNamed: aFilename) ifNil: 
		[self error: 'error saving resource'].
	[ stream nextPutAll: self data ]
		ensure: [ stream close ]! !

!SWResource methodsFor: 'testing' stamp: ''!
isVideo
	"Return true if the mimetype of the receiver is video-data. This message will match 
	types like: video/mpeg, video/quicktime, etc."

	^'video/*' match: mimetype! !

!SWResource methodsFor: 'testing' stamp: ''!
isText
	"Return true if the mimetype of the receiver is text-data. This message will match 
	types like: text/plain, text/html, text/sgml, text/css, text/xml, text/richtext, etc."

	^'text/*' match: mimetype! !

!SWResource methodsFor: 'testing' stamp: ''!
isApplication
	"Return true if the mimetype of the receiver is application-data. This message will
	match types like: application/octet-stream, application/oda, application/postscript,
	application/zip, application/pdf, etc."

	^'application/*' match: mimetype! !

!SWResource methodsFor: 'accessing' stamp: 'svp 11/10/2003 16:08'!
data: aByteArray	
	data := aByteArray asString! !

!SWResource class methodsFor: 'initialize' stamp: 'chbu 11/19/2003 23:02'!
initialize
	" SWResource initialize "

	super initialize! !

!SWResource methodsFor: 'accessing' stamp: ''!
stream
	^data readStream! !

!SWResource methodsFor: 'visiting' stamp: ''!
accept: aVisitor
	aVisitor acceptResource: self! !

!SWResource methodsFor: 'configuration' stamp: ''!
defaultEmbed
	^true! !

!SWResource methodsFor: 'storage' stamp: 'svp 11/11/2003 14:41'!
sixxInstVarNamed: instVarName put: aValue
	(instVarName = #data)
		ifTrue: [ self loadBinaryData: aValue ]	
		ifFalse: [ super sixxInstVarNamed: instVarName put: aValue ]! !

!SWResource methodsFor: 'testing' stamp: ''!
isImage
	"Return true if the mimetype of the receiver is image-data. This message will match 
	types like: image/jpeg, image/gif, image/png, image/tiff, etc."

	^'image/*' match: mimetype! !

!SWResource class methodsFor: 'initialize' stamp: 'chbu 10/22/2003 15:42'!
initializeActions
	super initializeActions.
	self registerAction: SWResourceEdit.
	self registerAction: SWResourceHistory.! !

!SWResource methodsFor: 'testing' stamp: ''!
isEmbedded
	"Return true if the resource of the receiver should be embedded into the
	desired context. Return false if the resource should be simply linked."

	^embed! !

!SWResource methodsFor: 'accessing' stamp: ''!
data
	^data! !

!SWResource methodsFor: 'storage' stamp: 'svp 11/11/2003 14:10'!
sixxContentOn: aStream indent: level context: dictionary
	| filename instVars index |

	" emit special data field "
	filename := (dictionary at: #directory) fullNameFor: self generateFilename.
	self saveBinaryData: filename.
	filename sixxOn: aStream name: 'data' indent: level + 1 context: dictionary.

	" emith all the other fields "
	instVars := self class allInstVarNames.
	instVars do: [ :name |
		name = 'data' ifFalse: [
			index := instVars indexOf: name. 
			(self instVarAt: index)
				sixxOn: aStream name: name indent: level + 1 context: dictionary ] ]! !

!SWResource methodsFor: 'accessing' stamp: ''!
stream: aStream
	self data: aStream contents! !

!SWResource methodsFor: 'accessing' stamp: ''!
mimetype
	^mimetype! !

!SWResource methodsFor: 'initialize' stamp: ''!
initialize
	super initialize.

	data := self defaultData.
	mimetype := self defaultMimetype.
	embed := self defaultEmbed! !

!SWStructure methodsFor: 'properties-local' stamp: ''!
localPropertyAt: aKey
	"Return the value of the property of the receiver with aKey. If there is no such 
	property, nil is returned."

	^self 
		localPropertyAt: aKey 
		ifAbsent: [ nil ]! !

!SWStructure class methodsFor: 'accessing-permissions' stamp: ''!
permissionAdd
	^self permissionAt: '* Add'! !

!SWStructure methodsFor: 'properties-inherited' stamp: 'chbu 10/22/2003 09:48'!
properties
	"Return a property manager including the values of all inherited 
	properties. Changes to the returned object does not change the
	receivers property manager."

	^self parents
		inject: SWPropertyManager new
		into: [ :manager :structure |
			structure localProperties do: [ :key :value | manager at: key put: value ].
			manager ]! !

!SWStructure methodsFor: 'testing' stamp: ''!
isRoot
	"Answer whether the receiver is the root node. The root is a folder by default and the only structure having no parent in the wiki-tree."

	^parent isNil! !

!SWStructure methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:23'!
renameTo: aString
	"Rename the title of the receiver. If the receiver is a child of another structure the title is checked to be unique, in case of a problem a \texttt{DuplicatedStructure} exception is thrown."

	" check if anything needs to be done "
	self title = aString
		ifTrue: [ ^self ].

	" check if there is a conflict "
	(aString isEmpty or: [ self parent notNil and: [ self parent includes: aString ] ])
		ifTrue: [ ^SWDuplicatedStructure signal: aString ].

	" process links pointing to the receiver (should be enhanced to fix
	  the whole path correctly, currently only leaves are adapted) "
	SWVisitorLinkInternal
		block: [ :link |
			(self == link target and: [ link reference notEmpty ])
				ifTrue: [
					link reference: (String streamContents: [ :stream |
						stream nextPutAll: (link reference copyFrom: 1 to: link reference size - self title size).
						stream nextPutAll: aString ]) ] ]
		start: self root.

	" rename structure "
	self title: aString! !

!SWStructure methodsFor: 'versions' stamp: ''!
nextVersion
	"Copy the receiver to be used in the history and return a the receiver."

	^self nextVersionBecome: self! !

!SWStructure methodsFor: 'accessing-navigation' stamp: ''!
previous
	"Return the previous node of the receiver's parent."

	^self isRoot
		ifTrue: [ nil ]
		ifFalse: [ parent previous: self ]! !

!SWStructure class methodsFor: 'accessing-permissions' stamp: ''!
permissionRemove
	^self permissionAt: '* Remove'! !

!SWStructure methodsFor: 'accessing' stamp: ''!
timestamp
	"Return a timestamp with the latest modification-time of the receiver."

	^timestamp! !

!SWStructure methodsFor: 'notification' stamp: ''!
changed: anAspectSymbol
	self changed: anAspectSymbol with: self! !

!SWStructure methodsFor: 'accessing-calculated' stamp: ''!
parents
	"Return an ordered collection with all the structures from the root up to and including 
	the receiver of the message."

	| collection structure |
	structure := self.
	collection := OrderedCollection new.
	[ collection addFirst: structure.
	  structure := structure parent.
	  structure isNil ] whileFalse.
	^collection! !

!SWStructure methodsFor: 'properties-local' stamp: ''!
localPropertyAt: aKey put: aValue
	"Set the property aKey to aValue in the receiver."

	^properties 
		at: aKey 
		put: aValue! !

!SWStructure methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:23'!
references
	^self referencesStartingAt: self root! !

!SWStructure methodsFor: 'resolving' stamp: ''!
privateResolveChild: aString
	"As we usually don't have children, return nil. Subclasses with children might want to
	override this message."

	^nil! !

!SWStructure methodsFor: 'accessing-navigation' stamp: ''!
last
	"Return the last node of the receiver's parent."

	^self isRoot
		ifTrue: [ self ]
		ifFalse: [ parent children last ]! !

!SWStructure class methodsFor: 'accessing-permissions' stamp: ''!
permissionCopy
	^self permissionAt: '* Copy'! !

!SWStructure methodsFor: 'accessing' stamp: ''!
title
	^title! !

!SWStructure class methodsFor: 'configuration' stamp: ''!
defaultEditAction
	"Return the action that should be executed when one wants to set-up or
	edit the structure. Overide this message to specify a specific action in
	your own components."

	^self defaultViewAction! !

!SWStructure class methodsFor: 'instance creation' stamp: ''!
parent: aStructure
	"Create a new structure with aStructure as parent and a default title."

	| child |
	child := self title: self title.
	aStructure add: child.
	^child! !

!SWStructure class methodsFor: 'accessing-permissions' stamp: ''!
permissionEdit
	^self permissionAt: '* Edit'! !

!SWStructure methodsFor: 'serving' stamp: ''!
processChild: aRequest
	"The default structure has got no children, therefore we process a 
	not-found message."
	
	self notFound: aRequest! !

!SWStructure methodsFor: 'resolving' stamp: ''!
privateShouldResolveChild: aCollection
	^aCollection size = 1 
		and: [ self isRoot
		or: [ (self privateResolveChild: aCollection first) notNil ] ]! !

!SWStructure methodsFor: 'serving' stamp: 'chbu 10/27/2003 20:15'!
processSelf: aRequest
	"Look for an action that might be executed on the current structure. If there is no 
	action given, the default one is executed."

	| actionName actionClass |
	actionName := aRequest fieldAt: 'action'.
	actionClass := actionName isNil
		ifFalse: [ Smalltalk
			at: actionName asSymbol
			ifAbsent: [ self class defaultViewAction ] ]
		ifTrue: [ self class defaultViewAction ].
	self processAction: (actionClass request: aRequest structure: self)! !

!SWStructure methodsFor: 'accessing-navigation' stamp: ''!
next
	"Return the next node of the receiver's parent."

	^self isRoot
		ifTrue: [ nil ]
		ifFalse: [ parent next: self ]! !

!SWStructure methodsFor: 'resolving' stamp: ''!
privateResolve: aCollection 
	^aCollection first isEmpty
		ifTrue: [ self root privateResolvePath: (aCollection removeFirst; yourself) ]
		ifFalse: [
			(self privateShouldResolveChild: aCollection)
				ifTrue: [ self privateResolveChild: aCollection first ]
				ifFalse: [ 
					self isRoot 
						ifTrue: [ nil ]
						ifFalse: [ self parent privateResolvePath: aCollection ] ] ]! !

!SWStructure class methodsFor: 'accessing-security' stamp: 'chbu 10/27/2003 20:06'!
permissions
	^permissions ifNil: [permissions := Set new]! !

!SWStructure methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:23'!
referencesStartingAt: aStructure
	| collection |
	collection := Set new.
	SWVisitorLinkInternal
		block: [ :link :structure |
			self == link target
				ifTrue: [ collection add: structure ] ]
		start: aStructure.
	^collection! !

!SWStructure class methodsFor: 'accessing-actions' stamp: 'chbu 10/27/2003 20:06'!
actions
	^actions ifNil: [actions := Set new]! !

!SWStructure methodsFor: 'accessing' stamp: ''!
timestamp: aTimestamp
	timestamp := aTimestamp! !

!SWStructure methodsFor: 'accessing' stamp: ''!
parent
	"Return the parent of the receiver. In the case the receiver is the root node
	nil is returned."

	^parent! !

!SWStructure methodsFor: 'private' stamp: ''!
myDependents
	^dependents! !

!SWStructure methodsFor: 'private' stamp: 'svp 11/11/2003 14:06'!
sixxInstVarsOn: aStream indent: level context: dictionary
	| instVars ind val |
	aStream cr.
	instVars := self class allInstVarNames.
	ind := 0.
	instVars do: 
		[:nm | 
		ind := ind + 1.
		nm = 'dependents'
			ifTrue: [val := nil]
			ifFalse: [val := self instVarAt: ind].
		val
			sixxOn: aStream
			name: nm
			indent: level
			context: dictionary]! !

!SWStructure methodsFor: 'initialize' stamp: 'chbu 10/22/2003 09:32'!
initialize
	super initialize.

	properties := SWPropertyManager new.
	timestamp := TimeStamp now.
	version := 0! !

WriteStream subclass: #SWHtmlWriteStream
	instanceVariableNames: 'stack attributes keepTight server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Server'!

SWHtmlWriteStream class
	instanceVariableNames: 'SWPrettyPrint SWEscapeTable'!

!SWHtmlWriteStream commentStamp: '<historical>' prior: 0!
Creating valid XHTML is an error prone task when using string concatenation. SmallWiki follows the design of Seaside \cite{Seaside} and implements the class \texttt{HtmlWriteStream}. This class subclasses \texttt{WriteStream} and it provides a lot of additional messages to append text and XHTML elements to the document being rendered.

The following example could be part of the message #renderContent within the \texttt{Action} hierarchy to render a simple user-interface:

=html heading: 'Title' level: 1.
=html paragraph: [
=    html text: 'Click '.
=    html 
=        anchor: 'here' 
=        to: self url 
=        callback: [ :action | action doSomething ] ]

The code produces something like the following output:

=<h1>Title<h1>
=<p>
=  Click <a href="/?action=ActionClass&callback=31415">here</a>
=</p>

The first message #heading:level: produces a simple section heading of level 1. The message #paragraph: is similar to the one of the heading, but in this example we do not pass a string but block: everything done within that block will be put inside the  paragraph tags. This mechanism assures that all tags are closed properly and that always valid XHTML is generated. The message #text: does basically the same as #nextPutAll:, additionally it escapes the given string to make sure the code can be displayed correctly within the web-browser.

The message #anchor:to:callback: is probably the most interesting one as it is used to generate an anchor with an assigned call-back block. The \texttt{anchor:} argument obviously renders the things that should be rendered as the content of the link. The \texttt{to:} argument specifies the place where the callback should be handled: usually this is within the same action, but occasionally you might need to specify something else. The \texttt{callback:} argument is evaluated when clicking the link. As an argument the block receives the action that is executing the callbacks, note that this is not necessarily the same action that rendered the link and that is referenced using the keyword \texttt{self}.

\texttt{HtmlWriteStream} doesn't emit any unnecessary spaces into the output stream, what makes investigation in the HTML code somehow difficult. For this purpose, there is the possibility to enable the included pretty-printer, but keep in mind that this slows down the rendering process and might have unwanted effects on the output in the web-browser.

=HtmlWriteStream prettyPrint: true

To see a more advanced examples about html-rendering and callbacks, have a look at the action-class \texttt{CallbackDemo}, that is part of the examples-bundle. Point your web-browser to \texttt{http://localhost:8080/?action=CallbackDemo}, play around with the user-interface and have a look at the implementation.!

!SWHtmlWriteStream methodsFor: 'html-form' stamp: ''!
hiddenInputWithValue: anObject callback: aBlock
	self 
		inputWithType: 'hidden' 
		callback: [ :action :value :mime |
			action 
				performCallback: aBlock 
				withArguments: (Array with: action with: anObject with: mime) ]! !

!SWHtmlWriteStream methodsFor: 'html-form' stamp: ''!
form: anObject
	self attributeAt: 'method' put: 'post'.
	self tag: 'form' do: anObject.! !

!SWHtmlWriteStream methodsFor: 'callbacks' stamp: 'chbu 11/12/2003 20:07'!
registerAnchorCallback: aBlock to: anUrl
	| prefix callbackAction |
	callbackAction:= aBlock isBlock
		ifTrue: [aBlock copy fixTemps]
		ifFalse: [aBlock].
	prefix := (anUrl includes: $?)
		ifTrue: [ '&' ]
		ifFalse: [ '?' ]. 
	^anUrl , prefix , 'callback=' , (self callback put: callbackAction) asString
! !

!SWHtmlWriteStream methodsFor: 'attributes' stamp: 'chbu 11/12/2003 20:07'!
attributeAt: key put: value
	(value == false) 
		ifTrue: [ ^self ].
	attributes isNil 
		ifTrue: [ attributes := WriteStream on: String new ].
	attributes nextPut: $ ; nextPutAll: key asString.
	(value == true) 
		ifFalse: [ attributes nextPutAll: '="'; nextPutAll: value asString; nextPut: $" ]! !

!SWHtmlWriteStream methodsFor: 'html-table' stamp: ''!
tableData: anObject
	self tag: 'td' do: anObject keepTight: true! !

!SWHtmlWriteStream methodsFor: 'html-form' stamp: ''!
passwordInputWithValue: anObject callback: aBlock
	self inputWithType: 'password' callback: aBlock value: anObject! !

!SWHtmlWriteStream methodsFor: 'html-table' stamp: ''!
tableRowWith: aFirstBlock with: aSecondBlock
	self tableRow: [ 
		self tableData: aFirstBlock.
		self tableData: aSecondBlock ]! !

!SWHtmlWriteStream methodsFor: 'html-form' stamp: 'chbu 11/12/2003 20:07'!
selectFromList: aCollection selected: aSelectedObject callback: aCallbackBlock
	self 
		selectFromList: aCollection 
		selected: aSelectedObject 
		callback: aCallbackBlock 
		labels: [ :object | object asString ]! !

!SWHtmlWriteStream methodsFor: 'html-form' stamp: ''!
fileUploadWithCallback: aBlock
	self inputWithType: 'file' named: (self registerFormCallback: aBlock)! !

!SWHtmlWriteStream methodsFor: 'private' stamp: ''!
identTag
	self class isPrettyPrint ifTrue: [
		self identTagBy: 2 * stack size ]! !

!SWHtmlWriteStream class methodsFor: 'settings' stamp: 'chbu 10/27/2003 20:09'!
escapeTable
	^SWEscapeTable ifNil: [ SWEscapeTable := Dictionary new
		at: $ put: '&iexcl;';
		at: $ put: '&cent;';
		at: $ put: '&pound;';
		at: $ put: '&curren;';
		at: $ put: '&yen;';
		at: $ put: '&brvbar;';
		at: $ put: '&sect;';
		at: $ put: '&uml;';
		at: $ put: '&copy;';
		at: $ put: '&ordf;';
		at: $ put: '&laquo;';
		at: $ put: '&not;';
		at: $  put: '&shy;';
		at: $ put: '&reg;';
		at: $ put: '&macr;';
		at: $ put: '&deg;';
		at: $ put: '&plusmn;';
		at: $ put: '&sup2;';
		at: $ put: '&sup3;';
		at: $ put: '&acute;';
		at: $ put: '&micro;';
		at: $ put: '&para;';
		at: $ put: '&middot;';
		at: $ put: '&cedil;';
		at: $ put: '&sup1;';
		at: $ put: '&ordm;';
		at: $ put: '&raquo;';
		at: $ put: '&frac14;';
		at: $ put: '&frac12;';
		at: $ put: '&frac34;';
		at: $ put: '&iquest;';
		at: $ put: '&Agrave;';
		at: $ put: '&Aacute;';
		at: $ put: '&Acirc;';
		at: $ put: '&Atilde;';
		at: $ put: '&Auml;';
		at: $ put: '&Aring;';
		at: $ put: '&AElig;';
		at: $ put: '&Ccedil;';
		at: $ put: '&Egrave;';
		at: $ put: '&Eacute;';
		at: $ put: '&Ecirc;';
		at: $ put: '&Euml;';
		at: $ put: '&Igrave;';
		at: $ put: '&Iacute;';
		at: $ put: '&Icirc;';
		at: $ put: '&Iuml;';
		at: $ put: '&ETH;';
		at: $ put: '&Ntilde;';
		at: $ put: '&Ograve;';
		at: $ put: '&Oacute;';
		at: $ put: '&Ocirc;';
		at: $ put: '&Otilde;';
		at: $ put: '&Ouml;';
		at: $ put: '&times;';
		at: $ put: '&Oslash;';
		at: $ put: '&Ugrave;';
		at: $ put: '&Uacute;';
		at: $ put: '&Ucirc;';
		at: $ put: '&Uuml;';
		at: $ put: '&Yacute;';
		at: $ put: '&THORN;';
		at: $ put: '&szlig;';
		at: $ put: '&agrave;';
		at: $ put: '&aacute;';
		at: $ put: '&acirc;';
		at: $ put: '&atilde;';
		at: $ put: '&auml;';
		at: $ put: '&aring;';
		at: $ put: '&aelig;';
		at: $ put: '&ccedil;';
		at: $ put: '&egrave;';
		at: $ put: '&eacute;';
		at: $ put: '&ecirc;';
		at: $ put: '&euml;';
		at: $ put: '&igrave;';
		at: $ put: '&iacute;';
		at: $ put: '&icirc;';
		at: $ put: '&iuml;';
		at: $ put: '&eth;';
		at: $ put: '&ntilde;';
		at: $ put: '&ograve;';
		at: $ put: '&oacute;';
		at: $ put: '&ocirc;';
		at: $ put: '&otilde;';
		at: $ put: '&ouml;';
		at: $ put: '&divide;';
		at: $ put: '&oslash;';
		at: $ put: '&ugrave;';
		at: $ put: '&uacute;';
		at: $ put: '&ucirc;';
		at: $ put: '&uuml;';
		at: $ put: '&yacute;';
		at: $ put: '&thorn;';
		at: $ put: '&yuml;';
		yourself
	]! !

!SWHtmlWriteStream methodsFor: 'html-head' stamp: ''!
link: relation to: url title: title
	self attributeAt: 'rel' put: relation.
	self attributeAt: 'href' put: url.
	self attributeAt: 'title' put: title.
	self tag: 'link'! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
break
	self tag: 'br'! !

!SWHtmlWriteStream class methodsFor: 'settings' stamp: 'chbu 10/27/2003 20:10'!
isPrettyPrint
	^SWPrettyPrint ifNil: [SWPrettyPrint := false]! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
anchorWithAction: aBlock to: anUrlString do: anObject 
	self anchorWithUrl: (self registerAnchorCallback: aBlock to: anUrlString) do: anObject! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
unorderedList: anObject
	self tag: 'ul' do: anObject! !

!SWHtmlWriteStream methodsFor: 'html-css' stamp: ''!
cssClass: aString 
	self attributeAt: #class put: aString! !

!SWHtmlWriteStream class methodsFor: 'private' stamp: ''!
on: aCollection
	^(super on: aCollection)
		initialize;
		yourself! !

!SWHtmlWriteStream methodsFor: 'html-block' stamp: ''!
head: anObject
	self tag: #head do: anObject! !

!SWHtmlWriteStream methodsFor: 'html-block' stamp: ''!
paragraph: aBlock
	self tag: 'p' do: aBlock! !

!SWHtmlWriteStream methodsFor: 'private' stamp: ''!
openTag: aString
	self openTag: aString andClose: false! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
comment: anObject
	self nextPutAll: '<!!--'; render: anObject; nextPutAll: '-->'! !

!SWHtmlWriteStream methodsFor: 'html-table' stamp: ''!
layoutTable: aBlock
	self attributeAt: 'cellspacing' put: 0.
	self attributeAt: 'cellpadding' put: 0.
	self attributeAt: 'border' put: 0.
	self table: aBlock! !

!SWHtmlWriteStream methodsFor: 'html-css' stamp: ''!
span: anObject
	self keepTight; tag: 'span' do: anObject keepTight: true! !

!SWHtmlWriteStream methodsFor: 'initialize' stamp: ''!
initialize
	stack := OrderedCollection new.
	keepTight := false.! !

!SWHtmlWriteStream methodsFor: 'private-form' stamp: ''!
inputWithType: aTypeString named: aNameString
	self attributeAt: 'name' put: aNameString.
	self inputWithType: aTypeString.! !

!SWHtmlWriteStream methodsFor: 'html-block' stamp: ''!
preformatted: anObject
	self tag: 'pre' do: anObject! !

!SWHtmlWriteStream methodsFor: 'accessing' stamp: ''!
server: aServer
	server := aServer! !

!SWHtmlWriteStream methodsFor: 'private' stamp: ''!
keepTight
	keepTight := true! !

!SWHtmlWriteStream methodsFor: 'private' stamp: ''!
closeAllTags
	[ stack isEmpty ] 
		whileFalse: [ self closeTag ]! !

!SWHtmlWriteStream methodsFor: 'html-css' stamp: ''!
cssName: aString 
	self attributeAt: #id put: aString! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
horizontalRule
	self tag: 'hr'! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
heading: anObject level: aNumber
	self tag: 'h', aNumber printString do: anObject keepTight: true! !

!SWHtmlWriteStream methodsFor: 'callbacks' stamp: 'chbu 11/12/2003 20:07'!
registerFormCallback: aBlock
	| callbackAction |
	callbackAction := aBlock isBlock
		ifTrue: [aBlock copy fixTemps]
		ifFalse: [aBlock].
	^'form-callback-' , (self callback put: callbackAction) asString! !

!SWHtmlWriteStream methodsFor: 'html' stamp: 'chbu 11/19/2003 19:39'!
image: anUrlString alt: aString
	self attributeAt: 'alt' put: aString.
	self image: anUrlString! !

!SWHtmlWriteStream methodsFor: 'html-primitive' stamp: ''!
tag: aString do: anObject
	self tag: aString do: anObject keepTight: false! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
image: anUrlString alt: aString size: aPoint 
	self attributeAt: 'width' put: aPoint x.
	self attributeAt: 'height' put: aPoint y.
	self image: anUrlString alt: aString! !

!SWHtmlWriteStream methodsFor: 'html-form' stamp: ''!
submitButton
	self inputWithType: 'submit'! !

!SWHtmlWriteStream class methodsFor: 'settings' stamp: 'chbu 10/22/2003 09:57'!
prettyPrint: aBoolean
	SWPrettyPrint := aBoolean! !

!SWHtmlWriteStream methodsFor: 'accessing' stamp: ''!
escapeTable
	^self class
		escapeTable! !

!SWHtmlWriteStream methodsFor: 'html-table' stamp: ''!
tableRowWith: aBlock span: aNumber
	self tableRow: [ 
		self attributeAt: 'colspan' put: aNumber. 
		self tableData: aBlock ]! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
embed: anObject to: anUrlString
	self attributeAt: 'data' put: anUrlString.
	self attributeAt: 'type' put: 'video/quicktime'.
	self tag: 'object' do: anObject! !

!SWHtmlWriteStream methodsFor: 'accessing' stamp: ''!
server
	^server! !

!SWHtmlWriteStream methodsFor: 'html-form' stamp: ''!
submitButtonWithAction: aBlock text: aString
	self inputWithType: 'submit' callback: aBlock value: aString! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
emphasis: aBlock
	self tag: 'em' do: aBlock.! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
embed: anObject to: anUrlString mime: aMimeString
	self attributeAt: 'data' put: anUrlString.
	self attributeAt: 'type' put: aMimeString.
	self tag: 'object' do: anObject! !

!SWHtmlWriteStream methodsFor: 'html-css' stamp: ''!
divNamed: aString with: anObject
	self cssName: aString; div: anObject! !

!SWHtmlWriteStream methodsFor: 'html-head' stamp: ''!
style: anObject
	self attributeAt: 'type' put: 'text/css'.
	self tag: 'style' do: anObject! !

!SWHtmlWriteStream methodsFor: 'private-form' stamp: ''!
inputWithType: aTypeString
	self attributeAt: 'type' put: aTypeString.
	self tag: 'input'.! !

!SWHtmlWriteStream methodsFor: 'html-form' stamp: ''!
textAreaWithValue: aString callback: aBlock
	self attributeAt: 'name' put: (self registerFormCallback: aBlock).
	self tag: 'textarea' do: aString keepTight: true.! !

!SWHtmlWriteStream methodsFor: 'html-head' stamp: ''!
title: anObject
	self tag: 'title' do: anObject keepTight: true! !

!SWHtmlWriteStream methodsFor: 'html-head' stamp: ''!
meta: aNameString content: aContentString
	self attributeAt: 'name' put: aNameString.
	self attributeAt: 'content' put: aContentString.
	self tag: 'meta'.! !

!SWHtmlWriteStream methodsFor: 'html-primitive' stamp: ''!
tag: aString
	self openTag: aString andClose: true! !

!SWHtmlWriteStream methodsFor: 'html-table' stamp: ''!
tableHeading: aBlock
	self tag: 'th' do: aBlock keepTight: true! !

!SWHtmlWriteStream methodsFor: 'html-form' stamp: ''!
label: aBlock
	self tag: 'label' do: aBlock keepTight: true! !

!SWHtmlWriteStream methodsFor: 'html-primitive' stamp: 'chbu 11/12/2003 20:12'!
render: anObject
	anObject swRenderOn: self! !

!SWHtmlWriteStream methodsFor: 'html-form' stamp: ''!
submitButtonWithAction: aBlock
	self inputWithType: 'submit' callback: aBlock! !

!SWHtmlWriteStream methodsFor: 'private' stamp: ''!
closeTag
	| tag |
	tag := stack removeLast.
	self identTag; nextPut: $<; nextPut: $/; nextPutAll: tag; nextPut: $>! !

!SWHtmlWriteStream methodsFor: 'html-form' stamp: 'chbu 11/12/2003 20:07'!
checkboxWithValue: aBoolean callback: aBlock
	| checkedValue |
	checkedValue := ValueHolder new contents: false; yourself.
	self attributeAt: 'checked' put: aBoolean.
	self inputWithType: 'checkbox' callback: [ :action :value :mime | checkedValue contents: true ].
	self hiddenInputWithValue: aBoolean asString callback: [ :action :value :mime |
		action performCallback: aBlock withArguments: (Array with: action with: checkedValue contents with: mime) ].! !

!SWHtmlWriteStream methodsFor: 'html-table' stamp: ''!
table: anObject
	self tag: 'table' do: anObject! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
anchorWithUrl: anUrlString do: anObject 
	self attributeAt: 'href' put: anUrlString.
	self keepTight; tag: 'a' do: anObject keepTight: true! !

!SWHtmlWriteStream methodsFor: 'accessing' stamp: ''!
callback
	^server callback! !

!SWHtmlWriteStream methodsFor: 'html-block' stamp: ''!
html: anObject
	self tag: #html do: anObject! !

!SWHtmlWriteStream methodsFor: 'html-head' stamp: ''!
metaEncoding: aString
	self attributeAt: 'http-equiv' put: 'Content-Type'.
	self attributeAt: 'content' put: aString.
	self tag: 'meta'! !

!SWHtmlWriteStream methodsFor: 'html-table' stamp: ''!
tableRowWith: aBlock
	self tableRow: [ self tableData: aBlock ]! !

!SWHtmlWriteStream methodsFor: 'html-primitive' stamp: ''!
tag: aString do: anObject keepTight: aBoolean
	self openTag: aString.
	self render: anObject.
	aBoolean 
		ifTrue: [ self keepTight ].
	self closeTag.! !

!SWHtmlWriteStream methodsFor: 'html-head' stamp: ''!
doctype: aString
	self nextPutAll: '<!!DOCTYPE html PUBLIC '.
	self nextPutAll: aString.
	self nextPutAll: '>'! !

!SWHtmlWriteStream methodsFor: 'html-form' stamp: ''!
selectFromList: aCollection selected: aSelectedObject callback: aCallbackBlock labels: aLabelsBlock
	| selectName |
	selectName := self registerFormCallback: [ :action :value :mime |
		action performCallback: aCallbackBlock withArguments: (Array with: action with: (aCollection at: value asNumber) with: mime) ].
	self attributeAt: 'name' put: selectName.
	self tag: 'select' do: [
		aCollection do: [ :object |
			self attributeAt: 'selected' put: (aSelectedObject = object).
			self attributeAt: 'value' put: (aCollection indexOf: object).
			self tag: 'option' do: [ 
				self text: (aLabelsBlock value: object); keepTight ] ] ]! !

!SWHtmlWriteStream methodsFor: 'private' stamp: ''!
identTagBy: aNumber
	keepTight
		ifTrue: [ keepTight := false ] 
		ifFalse: [
			self nextPut: Character cr.
			aNumber timesRepeat: [
				self nextPut: $ ] ]! !

!SWHtmlWriteStream methodsFor: 'html-table' stamp: ''!
tableRow: anObject
	self tag: 'tr' do: anObject! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
space
	self nextPutAll: '&nbsp;'! !

!SWHtmlWriteStream methodsFor: 'html-form' stamp: ''!
submitButton: aString
	self inputWithType: 'submit' named: nil value: aString! !

!SWHtmlWriteStream methodsFor: 'private' stamp: ''!
openTag: aString andClose: aBoolean
	self identTag.
	aBoolean 
		ifFalse: [ stack add: aString ].
	self nextPut: $<; nextPutAll: aString.
	attributes isNil ifFalse: [
		self nextPutAll: attributes contents.
		attributes := nil ].
	aBoolean
		ifTrue: [ self nextPutAll: ' /' ].
	self nextPut: $>! !

!SWHtmlWriteStream methodsFor: 'accessing' stamp: ''!
contents
	self closeAllTags.
	^super contents! !

!SWHtmlWriteStream methodsFor: 'private-form' stamp: ''!
inputWithType: aTypeString callback: aBlock value: aValueString 
	self
		inputWithType: aTypeString
		named: (self registerFormCallback: aBlock)
		value: aValueString! !

!SWHtmlWriteStream methodsFor: 'html-form' stamp: ''!
textInputWithValue: anObject callback: aBlock
	self
		inputWithType: 'text'
		callback: aBlock
		value: anObject! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
heading: aString
	self heading: aString level: 1! !

!SWHtmlWriteStream methodsFor: 'private-form' stamp: ''!
inputWithType: aTypeString callback: aBlock
	self
		inputWithType: aTypeString
		named: (self registerFormCallback: aBlock)! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
image: anUrlString
	self attributeAt: 'src' put: anUrlString.
	self tag: 'img'! !

!SWHtmlWriteStream methodsFor: 'html-css' stamp: ''!
spanClass: aString with: anObject
	self keepTight; cssClass: aString; span: anObject! !

!SWHtmlWriteStream methodsFor: 'html-form' stamp: ''!
formMultipart: anObject
	self attributeAt: 'enctype' put: 'multipart/form-data'.
	self form: anObject.! !

!SWHtmlWriteStream methodsFor: 'attributes' stamp: ''!
attributes: aCollection
	aCollection keysAndValuesDo: [ :key :value |
		self attributeAt: key put: value ]! !

!SWHtmlWriteStream methodsFor: 'html-css' stamp: ''!
divClass: aString with: anObject
	self cssClass: aString; div: anObject! !

!SWHtmlWriteStream methodsFor: 'html-css' stamp: ''!
spanNamed: aString with: anObject
	self keepTight; cssName: aString; span: anObject! !

!SWHtmlWriteStream methodsFor: 'html-block' stamp: ''!
body: anObject
	self tag: #body do: anObject! !

!SWHtmlWriteStream methodsFor: 'private-dnu' stamp: ''!
doesNotUnderstand: aMessage
	| argumentCount |
	argumentCount := aMessage arguments size.
	argumentCount = 0 ifTrue: [ ^self tag: aMessage selector ].
	argumentCount = 1 ifTrue: [ ^self tag: (aMessage selector copyFrom: 1 to: aMessage selector size - 1) do: aMessage arguments first ].
	^super doesNotUnderstand: aMessage! !

!SWHtmlWriteStream methodsFor: 'html-primitive' stamp: ''!
text: aString
	| table escape |
	table := self escapeTable.
	aString do: [ :character |
		escape := table at: character ifAbsent: [ nil ].
		escape isNil
			ifTrue: [ self nextPut: character ]
			ifFalse: [ self nextPutAll: escape ] ]! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
orderedList: anObject
	self tag: 'ol' do: anObject! !

!SWHtmlWriteStream methodsFor: 'private-form' stamp: ''!
inputWithType: aTypeString named: aNameString value: aValueString
	self attributeAt: 'value' put: aValueString.
	self inputWithType: aTypeString named: aNameString.! !

!SWHtmlWriteStream methodsFor: 'html-css' stamp: ''!
div: anObject
	self tag: 'div' do: anObject! !

!SWHtmlWriteStream methodsFor: 'html' stamp: ''!
listItem: anObject
	self tag: 'li' do: anObject keepTight: true! !

TestCase subclass: #SWDocumentTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Tests'!

!SWDocumentTests methodsFor: 'testing-component' stamp: 'chbu 10/25/2003 20:32'!
testLinkInternalBroken
	| item |
	self deny: self defaultBrokenLink isNil.
	self assert: self defaultBrokenLink isBroken.
	self deny: self defaultBrokenLink isComposedLink.
	self deny: self defaultBrokenLink hasTarget.
	self assert: self defaultBrokenLink class == SWLinkInternal.

	item := self defaultBrokenLink.
	self assert: item text isNil.
	self assert: item resolver title = self defaultPage title.
	self assert: item target isNil.
	self assert: item title = self defaultAnotherString.
	self assert: item url isEmpty.

	item := self defaultBrokenLink text: self defaultString.
	self deny: item text isNil.
	self assert: item resolver title = self defaultPage title.
	self assert: item target isNil.
	self assert: item title = self defaultString.
	self assert: item url isEmpty.! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:36'!
defaultPage
	^(SWPage parent: (SWFolder title: 'SmallWiki'))
		title: self defaultString;
		yourself! !

!SWDocumentTests methodsFor: 'configuration-constants' stamp: ''!
defaultEmail
	^'renggli@student.unibe.ch'! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testCode
	self deny: self defaultCode isNil.
	self deny: self defaultCode text isNil.! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testUnorderedList
	self deny: self defaultUnorderedList isNil.
	self deny: self defaultUnorderedList children first isNil! !

!SWDocumentTests methodsFor: 'testing-component' stamp: 'chbu 10/25/2003 20:33'!
testLinkExternal
	| item |
	self deny: self defaultExternalLink isNil.
	self deny: self defaultExternalLink isBroken.
	self assert: self defaultExternalLink class == SWLinkExternal.

	item := self defaultExternalLink.
	self assert: item text isNil.
	self assert: item url = self defaultUrl.
	self assert: item title = self defaultUrl.
	self assert: item reference = self defaultUrl.

	item := self defaultExternalLink text: self defaultAnotherString.
	self assert: item text = self defaultAnotherString.
	self assert: item url = self defaultUrl.
	self assert: item title = self defaultAnotherString.
	self assert: item reference = self defaultUrl.! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testText
	self deny: self defaultText isNil.
	self assert: self defaultText text = self defaultString! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:35'!
defaultExternalLink
	^SWLink newTo: self defaultUrl from: nil! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testTextHasText
	| item |
	item := self defaultText.
	self assert: item hasText.

	item := self defaultText text: ''.
	self deny: item hasText.

	item := self defaultText text: nil.
	self deny: item hasText.! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testTable
	self deny: self defaultTable isNil.
	self deny: self defaultTable children first isNil.
	self assert: self defaultTable columns = 1.
	self assert: self defaultTable rows = 1! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:38'!
defaultTable
	^SWTable new
		add: self defaultTableRow;
		yourself! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:37'!
defaultInternalLink
	^SWLink newTo: self defaultString from: self defaultPage! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:36'!
defaultMailToLink
	^SWLink newTo: self defaultEmail from: nil! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:38'!
defaultUnorderedList
	^SWUnorderedList new
		add: self defaultListItem;
		yourself! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:31'!
defaultCode
	^SWCode new! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:35'!
defaultHeader
	^SWHeader newText: self defaultString level: 1! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testHeader
	self deny: self defaultHeader isNil.
	self assert: self defaultHeader text = self defaultString.
	self assert: self defaultHeader level = 1! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testParagraph
	self deny: self defaultParagraph isNil.
	self deny: self defaultParagraph children first isNil! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testOrderedList
	self deny: self defaultOrderedList isNil.
	self deny: self defaultOrderedList children first isNil! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:38'!
defaultPreformatted
	^SWPreformatted new
		add: self defaultText;
		yourself! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:37'!
defaultParagraph
	^SWParagraph new
		add: self defaultText;
		yourself! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 17:08'!
defaultAction
	^SWPageEdit request: self defaultRequest structure: self defaultPage! !

!SWDocumentTests methodsFor: 'testing-component' stamp: 'svp 11/7/2003 00:58'!
testCodeSyntax
	self 
		should: [ self defaultCode code: '['; evaluateWith: self defaultAction ] 
		raise: Error.
	self 
		should: [ self defaultCode code: ';'; evaluateWith: self defaultAction ] 
		raise: Error.
	self 
		should: [ self defaultCode code: '1+'; evaluateWith: self defaultAction ] 
		raise: Error.! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:32'!
defaultRequest
	^SWRequest server: (SWServer new)! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testTableRow
	self deny: self defaultTableRow isNil.
	self deny: self defaultTableRow children first isNil.
	self assert: self defaultTableRow columns = 1.
	self assert: self defaultTableRow rows = 1! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testTableCell
	self deny: self defaultTableCell isNil.
	self deny: self defaultTableCell children first isNil! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testPreformatted
	self deny: self defaultPreformatted isNil.
	self deny: self defaultPreformatted children first isNil.! !

!SWDocumentTests methodsFor: 'testing' stamp: 'chbu 10/22/2003 09:31'!
testChildren
	| comp |
	comp := SWDocumentComposite new.
	self deny: comp children isNil.
	self assert: comp children isEmpty.

	comp add: #first.
	self assert: comp children size = 1.
	comp addAll: #(#second #third).
	self assert: comp children size = 3.
	self assert: comp children first = #first.
	self assert: comp children last = #third! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testHorizontalRule
	self deny: self defaultHorizontalRule isNil! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testCodeVariables
	| action |
	action := self defaultAction.

	self assert: (self defaultCode code: ''; evaluateWith: action) isNil.
	self assert: (self defaultCode code: 'self'; evaluateWith: action) == action.
	self assert: (self defaultCode code: 'structure'; evaluateWith: action) == action structure.
	self assert: (self defaultCode code: 'request'; evaluateWith: action) == action request.
	self assert: (self defaultCode code: 'response'; evaluateWith: action) == action request response.
	self assert: (self defaultCode code: 'html'; evaluateWith: action) == action request response stream.

	self assert: (self defaultCode code: '^nil'; evaluateWith: action) isNil.
	self assert: (self defaultCode code: '^self'; evaluateWith: action) == action.
	self assert: (self defaultCode code: '^structure'; evaluateWith: action) == action structure.
	self assert: (self defaultCode code: '^request'; evaluateWith: action) == action request.
	self assert: (self defaultCode code: '^response'; evaluateWith: action) == action request response.
	self assert: (self defaultCode code: '^html'; evaluateWith: action) == action request response stream.! !

!SWDocumentTests methodsFor: 'configuration-constants' stamp: ''!
defaultUrl
	^'http://renggli.freezope.org'! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:36'!
defaultOrderedList
	^SWOrderedList new
		add: self defaultListItem;
		yourself! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:37'!
defaultBrokenLink
	^SWLink newTo: self defaultAnotherString from: self defaultPage! !

!SWDocumentTests methodsFor: 'configuration-constants' stamp: ''!
defaultAnotherString
	^'Lukas Renggli'! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:37'!
defaultText
	^SWText newText: self defaultString! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:36'!
defaultListItem
	^SWListItem new
		add: self defaultText;
		yourself! !

!SWDocumentTests methodsFor: 'testing-component' stamp: 'chbu 10/25/2003 20:29'!
testCodeSemantic
	self 	
		should: [ self defaultCode code: 'self subclassResponsibility'; evaluateWith: self defaultAction ] 
		raise: "SubclassResponsibility"Error.
	self 
		should: [ self defaultCode code: 'self foo'; evaluateWith: self defaultAction ] 
		raise: MessageNotUnderstood.
	self 
		should: [ self defaultCode code: '1/0'; evaluateWith: self defaultAction ] 
		raise: "ZeroDivide"Error.! !

!SWDocumentTests methodsFor: 'testing' stamp: 'chbu 10/22/2003 09:35'!
testEnumerating
	| comp a b |
	comp := SWDocumentComposite new
		add: #first;
		addAll: #(#second #third);
		yourself.

	a := 0.
	comp children do: [ :item | a := a + 1 ].
	self assert: a = 3.

	a := 0. b := 0.
	comp children do: [ :item | a := a + 1 ] separatedBy: [ b := b + 1 ].
	self assert: a = 3.
	self assert: b = 2! !

!SWDocumentTests methodsFor: 'configuration-constants' stamp: ''!
defaultString
	^'SmallWiki'! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testListItem
	self deny: self defaultListItem isNil.
	self deny: self defaultListItem children first isNil! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:35'!
defaultHorizontalRule
	^SWHorizontalRule new! !

!SWDocumentTests methodsFor: 'testing-component' stamp: 'chbu 10/25/2003 20:32'!
testLinkInternal
	| item |
	self deny: self defaultInternalLink isNil.
	self deny: self defaultInternalLink isBroken.
	self deny: self defaultInternalLink isComposedLink.
	self assert: self defaultInternalLink hasTarget.
	self assert: self defaultInternalLink class == SWLinkInternal.

	item := self defaultInternalLink.
	self assert: item text isNil.
	self assert: item resolver title = self defaultPage title.
	self assert: item target title = self defaultPage title.
	self assert: item title = self defaultPage title.
	self assert: item url = self defaultPage url.

	item := self defaultInternalLink text: self defaultAnotherString.
	self deny: item text isNil.
	self assert: item resolver title = self defaultPage title.
	self assert: item target title = self defaultPage title.
	self assert: item title = self defaultAnotherString.
	self assert: item url = self defaultPage url.! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:38'!
defaultTableCell
	^SWTableCell new
		add: self defaultText;
		yourself! !

!SWDocumentTests methodsFor: 'testing-component' stamp: ''!
testDocument
	self deny: self defaultDocument isNil.
	self deny: self defaultDocument children first isNil! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:38'!
defaultTableRow
	^SWTableRow new
		add: self defaultTableCell;
		yourself! !

!SWDocumentTests methodsFor: 'configuration-pagecomponent' stamp: 'chbu 10/22/2003 09:35'!
defaultDocument
	^SWDocument new
		add: self defaultParagraph;
		yourself! !

!SWDocumentTests methodsFor: 'testing-component' stamp: 'chbu 10/25/2003 20:33'!
testLinkMailTo
	| item |
	self deny: self defaultMailToLink isNil.
	self deny: self defaultMailToLink isBroken.
	self assert: self defaultMailToLink class == SWLinkMailTo.

	item := self defaultMailToLink.
	self assert: item text isNil.
	self assert: item url = 'mailto:&#114;&#101;&#110;&#103;&#103;&#108;&#105;&#64;&#115;&#116;&#117;&#100;&#101;&#110;&#116;&#46;&#117;&#110;&#105;&#98;&#101;&#46;&#99;&#104;'.
	self assert: item title = self defaultEmail.
	self assert: item reference = self defaultEmail.

	item := self defaultMailToLink text: self defaultString.
	self assert: item text = self defaultString.
	self assert: item url = 'mailto:&#114;&#101;&#110;&#103;&#103;&#108;&#105;&#64;&#115;&#116;&#117;&#100;&#101;&#110;&#116;&#46;&#117;&#110;&#105;&#98;&#101;&#46;&#99;&#104;'.
	self assert: item title = self defaultString.
	self assert: item reference = self defaultEmail.! !

TestCase subclass: #SWStorageTests
	instanceVariableNames: 'storage server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Tests'!

!SWStorageTests methodsFor: 'testing-testing' stamp: ''!
testTestingValid
	self deny: storage isValid.

	server storage: storage.
	self assert: storage isValid.! !

!SWStorageTests methodsFor: 'configuration' stamp: 'chbu 10/22/2003 09:46'!
defaultStorage
	^SWSnapshotStorage new! !

!SWStorageTests methodsFor: 'testing-accessing' stamp: ''!
testDelay
	self deny: storage delay isNil.
	self assert: storage delay > 0.
	
	storage delay: 60 * 60 * 24.
	self assert: storage delay = (60 * 60 * 24).! !

!SWStorageTests methodsFor: 'testing-testing' stamp: ''!
testTestingExpired
	self assert: storage isExpired.! !

!SWStorageTests methodsFor: 'testing-accessing' stamp: 'chbu 10/22/2003 17:44'!
testLastsnapshot
	self assert: storage lastsnapshot asSeconds < TimeStamp now asSeconds! !

!SWStorageTests methodsFor: 'testing-accessing' stamp: ''!
testServer
	server storage: storage.
	self assert: storage isThreadRunning.
	self assert: server storage == storage.
	
	server storage: nil.
	self deny: storage isThreadRunning.
	self assert: server storage isNil.! !

!SWStorageTests methodsFor: 'testing-testing' stamp: ''!
testTestingChanged
	self deny: storage isChanged.

	storage changed.
	self assert: storage isChanged.! !

!SWStorageTests methodsFor: 'testing-accessing' stamp: 'chbu 10/22/2003 17:44'!
testLastchange
	self assert: storage lastchange asSeconds < TimeStamp now asSeconds! !

!SWStorageTests methodsFor: 'configuration' stamp: 'chbu 10/22/2003 09:46'!
defaultServer
	^SWServer new! !

!SWStorageTests methodsFor: 'testing-testing' stamp: ''!
testTestingThread
	self deny: storage isThreadRunning.

	server storage: storage.
	self assert: storage isThreadRunning.! !

!SWStorageTests methodsFor: 'running' stamp: ''!
tearDown
	storage stopThread! !

!SWStorageTests methodsFor: 'testing-testing' stamp: ''!
testTestingSnapshot
	self deny: storage isSnapshotNeeded.

	server storage: storage.
	self deny: storage isSnapshotNeeded.

	storage changed.
	self assert: storage isSnapshotNeeded.! !

!SWStorageTests methodsFor: 'testing' stamp: 'chbu 10/22/2003 20:16'!
testSnapshot
	self shouldnt: [ storage threadSnapshot ] raise: "SubclassResponsibility"Error.

	server storage: storage.
	storage stopThread.
	self shouldnt: [ storage threadSnapshot ] raise: "SubclassResponsibility"Error.

	storage changed.
	self should: [ storage threadSnapshot ] raise: "SubclassResponsibility"Error.! !

!SWStorageTests methodsFor: 'testing' stamp: ''!
testInitialized
	self deny: storage isValid.
	self deny: storage isChanged.! !

!SWStorageTests methodsFor: 'running' stamp: ''!
setUp
	storage := self defaultStorage.
	server := self defaultServer.! !

TestCase subclass: #SWCacheTests
	instanceVariableNames: 'fifo exp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Tests'!

!SWCacheTests methodsFor: 'testing' stamp: ''!
testFifoAccessors
	fifo max: 1111.
	self assert: fifo max = 1111.
	fifo shrink: 2222.
	self assert: fifo shrink = 2222.! !

!SWCacheTests methodsFor: 'testing' stamp: 'chbu 11/19/2003 19:46'!
testFifoShrink
	fifo max: 100; shrink: 10.

	" put 100 elements "
	1 to: 100 do: [ :index | fifo put: index ].
	self assert: fifo size = 100.

	" check the 100 elements twice (new behaviour) "
	1 to: 100 do: [ :index | self assert: (fifo get: index) = index ].
	1 to: 100 do: [ :index | self assert: (fifo get: index) = index ].

	" put 10 more elements "
	101 to: 110 do: [ :index | fifo put: index ].
	self assert: fifo size = 100.
	1 to: 10 do: [ :index | self assert: (fifo get: index) = nil ].
	111 to: 110 do: [ :index | self assert: (fifo get: index) = index ].! !

!SWCacheTests methodsFor: 'testing' stamp: 'chbu 11/12/2003 20:07'!
testExpData
	1 to: 1000 do: [ :index |
		self assert: (exp put: index asString) = index ].
	1 to: 1000 do: [ :index |
		self assert: (exp get: index) = index asString ].! !

!SWCacheTests methodsFor: 'testing' stamp: 'chbu 11/12/2003 20:07'!
testFifoData
	1 to: 1000 do: [ :index |
		self assert: (fifo put: index asString) = index ].
	1 to: 1000 do: [ :index |
		self assert: (fifo get: index) = index asString ].! !

!SWCacheTests methodsFor: 'testing' stamp: 'chbu 11/19/2003 19:46'!
testExpExpired
	exp ttl: 1; check: 1. 

	" put 100 elements "
	1 to: 100 do: [ :index | exp put: index ].
	self assert: exp size = 100.

	" check the 100 elements twice (new behaviour) "
	1 to: 100 do: [ :index | self assert: (exp get: index) = index ].
	1 to: 100 do: [ :index | self assert: (exp get: index) = index ].

	" wait one second "
	(Delay forSeconds: 1) wait.

	" put 10 more elements "
	101 to: 110 do: [ :index | exp put: index ].
	self assert: exp size = 10.
	1 to: 100 do: [ :index | self assert: (exp get: index) isNil ].
	101 to: 110 do: [ :index | self assert: (exp get: index) = index ].! !

!SWCacheTests methodsFor: 'running' stamp: 'chbu 10/22/2003 09:29'!
setUp
	fifo := SWFifoCache new.
	exp := SWExpiringCache new.! !

!SWCacheTests methodsFor: 'testing' stamp: ''!
testExpAccessors
	exp ttl: 1111.
	self assert: exp ttl = 1111.
	exp check: 2222.
	self assert: exp check = 2222.! !

TestCase subclass: #SWResponseTests
	instanceVariableNames: 'response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Tests'!

!SWResponseTests methodsFor: 'testing-accessing' stamp: ''!
testHeaders
	response headerAt: 'key' put: 'value'.
	self assert: (response headers at: 'key') = 'value'! !

!SWResponseTests methodsFor: 'testing-accessing' stamp: 'chbu 10/22/2003 17:32'!
testStatus
	self deny: response status isNil.
	self assert: response status = SWResponse statusOk.

	response status: SWResponse statusNotFound.
	self deny: response status isNil.
	self assert: response status = SWResponse statusNotFound! !

!SWResponseTests methodsFor: 'testing-callbacks' stamp: 'svp 11/11/2003 16:11'!
testCallbacksEvaluationAnchor
	| anchor request |
	anchor := ValueHolder new contents: false; yourself.
	response stream 
		anchorWithAction: [anchor contents: true]
		to: ''
		do: 'Anchor'.
	self deny: response server callback isEmpty.
	self assert: response server callback size = 1.
	request := SWRequest server: response server.
	request fields: response headers.
	self deny: anchor contents.
	(SWAction request: request structure: request server root) executeCallback.
	self deny: anchor contents.
	request fields: ((Dictionary new)
				at: 'callback' put: '1';
				yourself).
	(SWAction request: request structure: request server root) executeCallback.
	self assert: anchor contents! !

!SWResponseTests methodsFor: 'testing-action' stamp: 'chbu 10/22/2003 17:33'!
testUnauthorized
	response unauthorized.

	self assert: (response headers includesKey: 'WWW-Authenticate').
	self assert: ('basic realm="*"' match: (response headers at: 'WWW-Authenticate')).
	self assert: response status = SWResponse statusUnauthorized! !

!SWResponseTests methodsFor: 'testing-accessing' stamp: ''!
testType
	self deny: response type isNil.
	self assert: response type = 'text/html'.

	response status: 'plain/text'.
	self deny: response status isNil.
	self assert: response status = 'plain/text'! !

!SWResponseTests methodsFor: 'testing-action' stamp: 'chbu 10/22/2003 17:33'!
testRedirect
	response redirectTo: 'http://renggli.freezope.org'.

	self assert: (response headers at: 'Location') =  'http://renggli.freezope.org'.
	self assert: response status = SWResponse statusFound! !

!SWResponseTests methodsFor: 'testing-callbacks' stamp: 'chbu 11/12/2003 20:06'!
testCallbacksEvaluationForm
	| submits request posts |
	submits := Array new: 100.
	posts := Dictionary new.
	1 to: 100 do: [ :index |
		response stream 
			submitButtonWithAction: [ :action :value | 
				submits at: value put: true.
				"evaluation order wrong if assertion failure here"
				1 to: value do: [ :item | 
					self assert: (submits at: item) ].
				value + 1 to: 100 do: [ :item |
					self assert: (submits at: item) isNil ] ].  
		posts at: 'form-callback-' , index asString put: index asNumber ].	

	self deny: response server callback isEmpty.
	self assert: response server callback size = 100.

	request := SWRequest server: response server.
	request fields: response headers.
	self assert: (submits allSatisfy: [ :item | item isNil ]).
	(SWAction request: request structure: request server root)
		executeCallback.
	self assert: (submits allSatisfy: [ :item | item isNil ]).

	request fields: posts.
	(SWAction request: request structure: request server root)
		executeCallback.
	self assert: (submits allSatisfy: [ :item | item = true ]).! !

!SWResponseTests methodsFor: 'testing-accessing' stamp: 'chbu 10/22/2003 17:32'!
testStream
	self deny: response stream isNil.
	self assert: response stream class = SWHtmlWriteStream.

	response stream: (WriteStream on: String new).
	self deny: response stream isNil.
	self assert: response stream class = WriteStream.! !

!SWResponseTests methodsFor: 'running' stamp: 'chbu 10/22/2003 17:33'!
setUp
	response := SWResponse 
		server: SWServer new! !

TestCase subclass: #SWRequestTests
	instanceVariableNames: 'server request'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Tests'!

!SWRequestTests methodsFor: 'testing-fields' stamp: ''!
testFieldsAbsent
	self assert: (request fieldAt: 'unknown') isNil.
	self assert: (request fieldAt: 'unknown' ifAbsent: [ 'nothing' ]) = 'nothing'.
	self deny: (request fieldIncludes: 'unknown').
	self assert: request fields isEmpty! !

!SWRequestTests methodsFor: 'testing-authentication' stamp: ''!
testAuthenticationInvalid
	request headers at: 'AUTHORIZATION' put: 'Basic aW52YWxpZA=='.
	request updateAuthentication.

	self assert: request isAuthenticated not.
	self assert: request username = 'invalid'.
	self assert: request password isNil.
	self assert: request user == request server userAnonymous.! !

!SWRequestTests methodsFor: 'testing-url' stamp: ''!
testUrlRoot
	request url: '/'.

	self assert: request url size = 1.
	self assert: request urlAtEnd.! !

!SWRequestTests methodsFor: 'testing-cookies' stamp: ''!
testCookiesPresent
	request cookies at: 'somekey' put: 'value'.

	self assert: (request cookieAt: 'somekey') = 'value'.
	self assert: (request cookieAt: 'somekey' ifAbsent: [ nil ]) = 'value'.
	self assert: (request cookieIncludes: 'somekey').
	self assert: request cookies size = 1! !

!SWRequestTests methodsFor: 'testing-url' stamp: ''!
testUrlLong
	request url: '/world/europe/switzerland/berne/university/iam/scg'.

	self deny: request url isEmpty.
	self assert: request urlAtEnd not.
	self assert: request urlNext = 'world'.
	self assert: request urlAtEnd not.
	self assert: request urlNext = 'europe'.
	self assert: request urlAtEnd not.
	self assert: request urlNext = 'switzerland'.
	self assert: request urlAtEnd not.
	self assert: request urlNext = 'berne'.
	self assert: request urlAtEnd not.
	self assert: request urlNext = 'university'.
	self assert: request urlAtEnd not.
	self assert: request urlNext = 'iam'.
	self assert: request urlAtEnd not.
	self assert: request urlNext = 'scg'.
	self assert: request urlAtEnd! !

!SWRequestTests methodsFor: 'testing-cookies' stamp: ''!
testCookiesAbsent
	self assert: (request cookieAt: 'unknown') isNil.
	self assert: (request cookieAt: 'unknown' ifAbsent: [ 'nothing' ]) = 'nothing'.
	self deny: (request cookieIncludes: 'unknown').
	self assert: request cookies isEmpty! !

!SWRequestTests methodsFor: 'testing' stamp: 'chbu 10/22/2003 17:30'!
testResponse
	self deny: request response isNil.
	self assert: request response class = SWResponse! !

!SWRequestTests methodsFor: 'running' stamp: 'chbu 10/22/2003 09:41'!
setUp
	server := SWServer new
		userAdd: (SWUser username: 'renggli' password: 'something');
		yourself.
	request := (SWRequest server: server)
		cookies: Dictionary new;
		fields: Dictionary new;	
		headers: Dictionary new;
		yourself! !

!SWRequestTests methodsFor: 'testing-headers' stamp: ''!
testHeadersAbsent
	self assert: (request headerAt: 'unknown') isNil.
	self assert: (request headerAt: 'unknown' ifAbsent: [ 'nothing' ]) = 'nothing'.
	self deny: (request headerIncludes: 'unknown').
	self assert: request headers isEmpty! !

!SWRequestTests methodsFor: 'testing-authentication' stamp: ''!
testAuthenticationUnknown
	request headers at: 'AUTHORIZATION' put: 'Digest realm="renggli@student.unibe.ch", nonce="dcd98b7102dd2f0e8b11d0f600bfb0c093", opaque="5ccc069c403ebaf9f0171e9517f40e41"'.
	request updateAuthentication.

	self assert: request isAuthenticated not.
	self assert: request username isNil.
	self assert: request password isNil.
	self assert: request user == request server userAnonymous.! !

!SWRequestTests methodsFor: 'testing-authentication' stamp: ''!
testAuthenticationCookie
	request cookies at: 'authorization' put: 'renggli:sometxing'.
	request updateAuthentication.

	self assert: request isAuthenticated.
	self assert: request username = 'renggli'.
	self assert: request password = 'sometxing'.
	self assert: request user == request server userAnonymous.! !

!SWRequestTests methodsFor: 'testing-url' stamp: ''!
testUrlEmpty
	request url: String new.

	self assert: request url isEmpty.
	self assert: request urlAtEnd! !

!SWRequestTests methodsFor: 'testing-authentication' stamp: ''!
testAuthenticationInvalidPassword
	request headers at: 'AUTHORIZATION' put: 'Basic cmVuZ2dsaTpzb21ldHhpbmc='.
	request updateAuthentication.

	self assert: request isAuthenticated.
	self assert: request username = 'renggli'.
	self assert: request password = 'sometxing'.
	
	self assert: request user == request server userAnonymous.! !

!SWRequestTests methodsFor: 'testing-authentication' stamp: ''!
testAuthenticationTest
	self deny: request isAuthenticated.
	self assert: request user == request server userAnonymous.! !

!SWRequestTests methodsFor: 'testing-fields' stamp: ''!
testFieldsPresent
	request fields at: 'somekey' put: 'value'.

	self assert: (request fieldAt: 'somekey') = 'value'.
	self assert: (request fieldAt: 'somekey' ifAbsent: [ nil ]) = 'value'.
	self assert: (request fieldIncludes: 'somekey').
	self assert: request fields size = 1! !

!SWRequestTests methodsFor: 'testing-headers' stamp: ''!
testHeadersPresent
	request headers at: 'SOMEKEY' put: 'value'.

	self assert: (request headerAt: 'somekey') = 'value'.
	self assert: (request headerAt: 'someKey') = 'value'.
	self assert: (request headerAt: 'SomeKey') = 'value'.
	self assert: (request headerAt: 'somekey' ifAbsent: [ nil ]) = 'value'.
	self assert: (request headerAt: 'someKey' ifAbsent: [ nil ]) = 'value'.
	self assert: (request headerAt: 'SomeKey' ifAbsent: [ nil ]) = 'value'.
	self assert: (request headerIncludes: 'somekey').
	self assert: (request headerIncludes: 'someKey').
	self assert: (request headerIncludes: 'SomeKey').
	self assert: request headers size = 1! !

!SWRequestTests methodsFor: 'testing-authentication' stamp: ''!
testAuthenticationValid
	request headers at: 'AUTHORIZATION' put: 'Basic cmVuZ2dsaTpzb21ldGhpbmc='.
	request updateAuthentication.

	self assert: request isAuthenticated.
	self assert: request username = 'renggli'.
	self assert: request password = 'something'.
	
	self assert: request user username = 'renggli'.
	self assert: request user ~~ request server userAnonymous.! !

TestCase subclass: #SWSecurityTests
	instanceVariableNames: 'permissionEdit permissionView permissionHistory roleAdmin roleAnybody userLukas userUnknown'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Tests'!

!SWSecurityTests methodsFor: 'testing-permissions' stamp: ''!
testPermissionCopy
	self assert: permissionView copy == permissionView.
	self assert: permissionEdit copy == permissionEdit.
	self assert: permissionHistory copy == permissionHistory.! !

!SWSecurityTests methodsFor: 'running' stamp: 'chbu 10/22/2003 09:43'!
setUp

	"create permissions"
	permissionView := SWPermission name: 'Page View'.
	permissionEdit := SWPermission name: 'Page Edit'.
	permissionHistory := SWPermission name: 'Page History'.

	"create roles"
	roleAdmin := (SWBasicRole name: 'Administrator')
		add: permissionView;
		add: permissionEdit;
		add: permissionHistory;
		yourself.
	roleAnybody := (SWBasicRole name: 'Anybody')
		add: permissionView;
		yourself.
	
	"create users"
	userLukas := (SWUser username: 'lukas' password: 'something')
		add: roleAnybody;
		add: roleAdmin;
		yourself.
	userUnknown := SWUser anonymous
		add: roleAnybody;
		yourself! !

!SWSecurityTests methodsFor: 'testing-users' stamp: 'chbu 10/22/2003 09:45'!
testUserAsserting
	self shouldnt: [ userLukas assertPermission: permissionView ] raise: SWUnauthorizedError.
	self shouldnt: [ userLukas assertPermission: permissionEdit ] raise: SWUnauthorizedError.
	self shouldnt: [ userLukas assertPermission: permissionHistory ] raise: SWUnauthorizedError.

	self shouldnt: [ userUnknown assertPermission: permissionView ] raise: SWUnauthorizedError.
	self should: [ userUnknown assertPermission: permissionEdit ] raise: SWUnauthorizedError.
	self should: [ userUnknown assertPermission: permissionHistory ] raise: SWUnauthorizedError.! !

!SWSecurityTests methodsFor: 'testing-roles' stamp: 'chbu 10/22/2003 09:44'!
testRoleComparing
	self assert: (SWBasicRole name: 'Administrator') = roleAdmin.
	self assert: (SWBasicRole name: 'Anybody') = roleAnybody.

	self assert: (SWBasicRole name: 'Administrator') hash = roleAdmin hash.
	self assert: (SWBasicRole name: 'Anybody') hash = roleAnybody hash.! !

!SWSecurityTests methodsFor: 'testing-roles' stamp: ''!
testRoleCopy
	self assert: roleAdmin copy ~~ roleAdmin.
	self assert: roleAdmin copy name == roleAdmin name.
	self assert: roleAdmin copy permissions ~~ roleAdmin permissions.

	self assert: roleAnybody copy ~~ roleAnybody.
	self assert: roleAnybody copy name == roleAnybody name.
	self assert: roleAnybody copy permissions ~~ roleAnybody permissions.! !

!SWSecurityTests methodsFor: 'testing-roles' stamp: ''!
testRolePermissions
	self assert: roleAdmin permissions size = 3.
	self assert: roleAnybody permissions size = 1.

	roleAdmin add: permissionView.
	self assert: roleAdmin permissions size = 3.

	roleAnybody add: permissionEdit.
	self assert: roleAnybody permissions size = 2.

	roleAnybody remove: permissionView.
	self assert: roleAnybody permissions size = 1.
	
	roleAnybody remove: permissionView.
	self assert: roleAnybody permissions size = 1.! !

!SWSecurityTests methodsFor: 'testing-users' stamp: 'chbu 10/22/2003 09:44'!
testUserUpdateRoles
	| roles newUserUnknown |
	roles := OrderedCollection 
		with: ((SWBasicRole name: 'Admin')
			add: permissionView;
			yourself)
		with: ((SWBasicRole name: 'Anybody')
			add: permissionView;
			add: permissionHistory;
			yourself).
	newUserUnknown := userUnknown updateRoles: roles.

	self assert: newUserUnknown ~~ userUnknown.
	self assert: (newUserUnknown hasPermission: permissionView).
	self assert: (newUserUnknown hasPermission: permissionHistory).
	self deny: (newUserUnknown hasPermission: permissionEdit).! !

!SWSecurityTests methodsFor: 'testing-users' stamp: ''!
testUserCopy
	self assert: userLukas copy ~~ userLukas.
	self assert: userLukas copy roles ~~ userLukas roles.
	self assert: userLukas copy username == userLukas username.

	self assert: userUnknown copy ~~ userUnknown.
	self assert: userUnknown copy roles ~~ userUnknown roles.
	self assert: userUnknown copy username == userUnknown username.! !

!SWSecurityTests methodsFor: 'testing-permissions' stamp: 'chbu 10/22/2003 09:45'!
testPermissionAsserting
	self shouldnt: [ permissionView assertPermission: permissionView ] raise: SWUnauthorizedError.
	self shouldnt: [ permissionEdit assertPermission: permissionEdit ] raise: SWUnauthorizedError.
	self shouldnt: [ permissionHistory assertPermission: permissionHistory ] raise: SWUnauthorizedError.

	self should: [ permissionView assertPermission: permissionEdit ] raise: SWUnauthorizedError.
	self should: [ permissionView assertPermission: permissionHistory ] raise: SWUnauthorizedError.
	self should: [ permissionHistory assertPermission: permissionView ] raise: SWUnauthorizedError.
	self should: [ permissionHistory assertPermission: permissionEdit ] raise: SWUnauthorizedError.
	self should: [ permissionEdit assertPermission: permissionView ] raise: SWUnauthorizedError.
	self should: [ permissionEdit assertPermission: permissionHistory ] raise: SWUnauthorizedError.! !

!SWSecurityTests methodsFor: 'testing-users' stamp: ''!
testUserRoles
	self assert: userLukas roles size = 2.
	self assert: userUnknown roles size = 1.
	
	userLukas add: roleAdmin.
	self assert: userLukas roles size = 2.

	userUnknown add: roleAdmin.
	self assert: userUnknown roles size = 2.

	userUnknown remove: roleAdmin.
	self assert: userUnknown roles size = 1.

	userUnknown remove: roleAdmin.
	self assert: userUnknown roles size = 1.! !

!SWSecurityTests methodsFor: 'testing-users' stamp: ''!
testUserTesting
	self assert: (userLukas hasPermission: permissionView).
	self assert: (userLukas hasPermission: permissionEdit).
	self assert: (userLukas hasPermission: permissionHistory).

	self assert: (userUnknown hasPermission: permissionView).
	self deny: (userUnknown hasPermission: permissionEdit).
	self deny: (userUnknown hasPermission: permissionHistory).! !

!SWSecurityTests methodsFor: 'testing-permissions' stamp: ''!
testPermissionTesting
	self assert: (permissionView hasPermission: permissionView).
	self assert: (permissionEdit hasPermission: permissionEdit).
	self assert: (permissionHistory hasPermission: permissionHistory).

	self deny: (permissionView hasPermission: permissionEdit).
	self deny: (permissionView hasPermission: permissionHistory).
	self deny: (permissionHistory hasPermission: permissionView).
	self deny: (permissionHistory hasPermission: permissionEdit).
	self deny: (permissionEdit hasPermission: permissionView).
	self deny: (permissionEdit hasPermission: permissionHistory).! !

!SWSecurityTests methodsFor: 'testing-roles' stamp: ''!
testRoleTesting
	self assert: (roleAdmin hasPermission: permissionView).
	self assert: (roleAdmin hasPermission: permissionEdit).
	self assert: (roleAdmin hasPermission: permissionHistory).

	self assert: (roleAnybody hasPermission: permissionView).
	self deny: (roleAnybody hasPermission: permissionEdit).
	self deny: (roleAnybody hasPermission: permissionHistory).! !

!SWSecurityTests methodsFor: 'testing-permissions' stamp: 'chbu 10/22/2003 09:43'!
testPermissionComparing
	self assert: (SWPermission name: 'Page View') = permissionView.
	self assert: (SWPermission name: 'Page Edit') = permissionEdit.
	self assert: (SWPermission name: 'Page History') = permissionHistory.

	self assert: (SWPermission name: 'Page View') hash = permissionView hash.
	self assert: (SWPermission name: 'Page Edit') hash = permissionEdit hash.
	self assert: (SWPermission name: 'Page History') hash = permissionHistory hash.! !

!SWSecurityTests methodsFor: 'testing-roles' stamp: ''!
testRoleAccessing
	self assert: roleAdmin name = 'Administrator'.
	self assert: roleAnybody name = 'Anybody'! !

!SWSecurityTests methodsFor: 'testing-users' stamp: 'chbu 10/22/2003 09:44'!
testUserComparing
	self assert: (SWUser username: 'lukas' password: 'something') = userLukas.
	self assert: (SWUser username: 'lukas' password: 'abc') = userLukas.
	self assert: (SWUser anonymous) = userUnknown.

	self assert: (SWUser username: 'lukas' password: 'something') hash = userLukas hash.
	self assert: (SWUser username: 'lukas' password: 'abc') hash = userLukas hash.
	self assert: (SWUser anonymous) hash = userUnknown hash.! !

!SWSecurityTests methodsFor: 'testing-users' stamp: 'chbu 10/22/2003 09:44'!
testUserUpdateRole
	| newRoleAdmin newRoleAnybody newUserUnknown |

	"update with role that is not definied does not change anything"
	newRoleAdmin := (SWBasicRole name: 'Admin')
		add: permissionView;
		yourself.
	newUserUnknown := userUnknown updateRole: newRoleAdmin.

	self assert: newUserUnknown == userUnknown.
	self assert: (newUserUnknown hasPermission: permissionView).
	self deny: (newUserUnknown hasPermission: permissionHistory).
	self deny: (newUserUnknown hasPermission: permissionEdit).	

	"roles get updated if they are defined within the receiver"
	newRoleAnybody := (SWBasicRole name: 'Anybody')
		add: permissionView;
		add: permissionHistory;
		yourself.
	newUserUnknown := userUnknown updateRole: newRoleAnybody.

	self assert: newUserUnknown ~~ userUnknown.
	self assert: (newUserUnknown hasPermission: permissionView).
	self assert: (newUserUnknown hasPermission: permissionHistory).
	self deny: (newUserUnknown hasPermission: permissionEdit).! !

!SWSecurityTests methodsFor: 'testing-permissions' stamp: ''!
testPermissionAccessing
	self assert: permissionView name = 'Page View'.
	self assert: permissionEdit name = 'Page Edit'.
	self assert: permissionHistory name = 'Page History'.! !

!SWSecurityTests methodsFor: 'testing-users' stamp: ''!
testUserUsername
	self assert: userLukas username = 'lukas'.
	self assert: userUnknown username = 'anonymous'! !

!SWSecurityTests methodsFor: 'testing-roles' stamp: 'chbu 10/22/2003 09:43'!
testRoleAsserting
	self shouldnt: [ roleAdmin assertPermission: permissionView ] raise: SWUnauthorizedError.
	self shouldnt: [ roleAdmin assertPermission: permissionEdit ] raise: SWUnauthorizedError.
	self shouldnt: [ roleAdmin assertPermission: permissionHistory ] raise: SWUnauthorizedError.

	self shouldnt: [ roleAnybody assertPermission: permissionView ] raise: SWUnauthorizedError.
	self should: [ roleAnybody assertPermission: permissionEdit ] raise: SWUnauthorizedError.
	self should: [ roleAnybody assertPermission: permissionHistory ] raise: SWUnauthorizedError.! !

!SWSecurityTests methodsFor: 'testing-users' stamp: ''!
testUserPassword
	self deny: (userLukas validatePassword: nil).
	self deny: (userLukas validatePassword: String new).
	self assert: (userLukas validatePassword: 'something').

	self assert: (userUnknown validatePassword: nil).
	self assert: (userUnknown validatePassword: String new).
	self assert: (userUnknown validatePassword: 'something').! !

TestCase subclass: #SWTemplateTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Tests'!

!SWTemplateTests methodsFor: 'testing' stamp: 'chbu 10/22/2003 09:53'!
testExpansion
	| template action |
	template := SWTemplate new.
	action := SWPageEdit new.
	
	self assert: (template expand: String new for: action) = String new.
	self assert: (template expand: '%' for: action) = String new.
	self assert: (template expand: '%%' for: action) = '%'.

	self assert: (template expand: 'abc' for: action) = 'abc'.
	self assert: (template expand: 'abc%' for: action) = 'abc'.
	self assert: (template expand: 'abc%%' for: action) = 'abc%'.

	self assert: (template expand: '%a' for: action) = 'Edit'.
	self assert: (template expand: 'abc%a' for: action) = 'abcEdit'.
	self assert: (template expand: '%aabc' for: action) = 'Editabc'.
	self assert: (template expand: 'abc%aabc' for: action) = 'abcEditabc'.! !

!SWTemplateTests methodsFor: 'testing-special' stamp: 'chbu 10/22/2003 09:53'!
testingBodyAccessing
	| template |
	template := SWTemplateBodyPath new.

	template id: 'myId'.
	self assert: template id = 'myId'.

	template title: 'myTitle'.
	self assert: template title = 'myTitle'.! !

!SWTemplateTests methodsFor: 'testing-special' stamp: 'chbu 10/22/2003 09:53'!
testingHead
	| action stream |
	action := SWPageEdit new.
	stream := SWHtmlWriteStream on: String new.

	" no body rendering in head templates "
	SWTemplateHead withAllSubclasses do: [ :each |
		each new renderBodyWith: action on: stream.
		self assert: stream isEmpty ]! !

!SWTemplateTests methodsFor: 'testing' stamp: 'chbu 10/22/2003 09:52'!
testClass
	self assert: SWTemplate title isNil.
	self assert: SWTemplate isRemoveable.! !

!SWTemplateTests methodsFor: 'testing-special' stamp: 'chbu 10/22/2003 09:53'!
testingBody
	| template |
	SWTemplateBody allSubclasses do: [ :each |
		template := each new.
		self deny: template id isNil.
		self deny: template title isNil ]! !

!SWTemplateTests methodsFor: 'testing' stamp: 'chbu 10/22/2003 09:53'!
testRendering
	| template action stream |
	template := SWTemplate new.
	action := SWPageEdit new.
	stream := SWHtmlWriteStream on: String new.

	template renderHeadWith: action on: stream.
	self assert: stream  contents isEmpty.

	template renderBodyWith: action on: stream.
	self assert: stream  contents isEmpty.

	template renderConfigWith: action on: stream.
	self assert: stream  contents isEmpty.! !

TestCase subclass: #SWExtensionTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Tests'!

!SWExtensionTests methodsFor: 'testing' stamp: 'svp 11/7/2003 01:00'!
testStreamContents
	| result |
	result := String streamContents: [ :stream |
		stream nextPutAll: 'Hello World' ].
	self assert: result = 'Hello World'.

	result := result readStream upToEnd.
	self assert: result = 'Hello World'.

	result := String streamContents: [ :stream |
		nil ].
	self assert: result isEmpty.
	
	result := result readStream upToEnd.
	self assert: result isEmpty.! !

!SWExtensionTests methodsFor: 'testing' stamp: ''!
testMoveUp
	| beenHere collection |
	beenHere := false.
	collection := #( a b c ) asOrderedCollection.

	collection moveUp: #c ifError: [ self assert: false ].
	self assert: collection = #( a c b ) asOrderedCollection.
	collection moveUp: #c ifError: [ self assert: false ].
	self assert: collection = #( c a b ) asOrderedCollection.
	collection moveUp: #c ifError: [ beenHere := true ].
	self assert: collection = #( c a b ) asOrderedCollection.
	self assert: beenHere.

	beenHere := false.
	collection moveUp: #d ifError: [ beenHere := true ].
	self assert: collection = #( c a b ) asOrderedCollection.
	self assert: beenHere.! !

!SWExtensionTests methodsFor: 'testing' stamp: ''!
testMoveDown
	| beenHere collection |
	beenHere := false.
	collection := #( a b c ) asOrderedCollection.

	collection moveDown: #a ifError: [ self assert: false ].
	self assert: collection = #( b a c ) asOrderedCollection.
	collection moveDown: #a ifError: [ self assert: false ].
	self assert: collection = #( b c a ) asOrderedCollection.
	collection moveDown: #a ifError: [ beenHere := true ].
	self assert: collection = #( b c a ) asOrderedCollection.
	self assert: beenHere.

	beenHere := false.
	collection moveDown: #d ifError: [ beenHere := true ].
	self assert: collection = #( b c a ) asOrderedCollection.
	self assert: beenHere.! !

TestCase subclass: #SWServerTests
	instanceVariableNames: 'server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Tests'!

!SWServerTests methodsFor: 'testing-users' stamp: ''!
testUser
	self assert: (server userAnonymous username) = 'anonymous'.
	self assert: (server userAnonymous validatePassword: nil).
	self assert: (server userAnonymous validatePassword: String new).
	self assert: (server userAnonymous validatePassword: 'something').! !

!SWServerTests methodsFor: 'testing-accessing' stamp: 'chbu 10/22/2003 17:42'!
testCallback
	| cache |
	cache := SWFifoCache new.
	server callback: cache.
	
	self assert: (server callback) == cache.! !

!SWServerTests methodsFor: 'testing-serving' stamp: 'chbu 10/24/2003 16:14'!
testStartOnHostIp
	server := self defaultServerClass startOn: 8082 host: 'localhost'  ip: '*'.
	[ self assert: server port = 8082.
	  self assert: server host = 'localhost'.
 	  self assert: server ip = '*'.
	  self assert: server isServing ]
	ensure: [ server stop ]! !

!SWServerTests methodsFor: 'running' stamp: ''!
setUp
	server := self defaultServerClass new! !

!SWServerTests methodsFor: 'testing-namespace' stamp: 'chbu 10/22/2003 17:41'!
testCopyright
	self deny: SWSmallWiki copyrightString isNil.
	self deny: SWSmallWiki copyrightString isEmpty.! !

!SWServerTests methodsFor: 'testing-serving' stamp: 'chbu 10/22/2003 17:40'!
testStartOn
	server := self defaultServerClass startOn: 8081.
	[ self assert: server port = 8081.
	  self assert: server host = SWServer defaultHost.
 	  self assert: server ip = SWServer defaultIp.
	  self assert: server isServing ]
	ensure: [ server stop ]! !

!SWServerTests methodsFor: 'testing-namespace' stamp: 'chbu 10/22/2003 17:41'!
testVersion
	self deny: SWSmallWiki versionString isNil.
	self deny: SWSmallWiki versionString isEmpty.! !

!SWServerTests methodsFor: 'testing-namespace' stamp: 'chbu 10/22/2003 17:40'!
testAuthor
	self deny: SWSmallWiki authorString isNil.
	self deny: SWSmallWiki authorString isEmpty.! !

!SWServerTests methodsFor: 'testing-accessing' stamp: ''!
testCallbacks
	self deny: server callback isNil.
	self assert: server callback isEmpty.! !

!SWServerTests methodsFor: 'testing-namespace' stamp: 'chbu 10/22/2003 17:41'!
testLicense
	self deny: SWSmallWiki licenseString isNil.
	self deny: SWSmallWiki licenseString isEmpty.! !

!SWServerTests methodsFor: 'testing-accessing' stamp: ''!
testHost
	server host: 'renggli.freezope.org'.
	self assert: server host = 'renggli.freezope.org'.! !

!SWServerTests methodsFor: 'testing-accessing' stamp: ''!
testPort
	server port: 1234.
	self assert: server port = 1234.! !

!SWServerTests methodsFor: 'testing-accessing' stamp: 'svp 11/7/2003 01:24'!
testIp

	server ip: '123.456.789.123'.
	self assert: server ip = '123.456.789.123'.! !

!SWServerTests methodsFor: 'testing-accessing' stamp: 'chbu 10/22/2003 17:42'!
testRoot
	| chapter |
	chapter := SWFolder title: 'SmallWiki'.
	server root: chapter.
	
	self assert: (server root) == chapter.! !

!SWServerTests methodsFor: 'testing-users' stamp: 'chbu 10/22/2003 17:41'!
testUserAt
	| user |
	user := SWUser username: 'renggli' password: 'something'.
	server userAdd: user.

	self assert: (server userAt: 'renggli') == user.
	self assert: (server userAt: 'renggli' ifAbsent: [ nil ]) == user.

	self assert: (server userAt: 'unknown') == server userAnonymous.
	self assert: (server userAt: 'unknown' ifAbsent: [ nil ]) isNil.! !

!SWServerTests methodsFor: 'testing-users' stamp: 'chbu 10/22/2003 17:41'!
testUserRemove
	server userAdd: (SWUser username: 'r1' password: 'a').
	server userAdd: (SWUser username: 'r2' password: 'b').
	server userAdd: (SWUser username: 'r3' password: 'c').

	self assert: (server userIncludes: 'r1').
	self assert: (server userIncludes: 'r2').
	self assert: (server userIncludes: 'r3').
	self deny: (server userIncludes: 'r4').

	server userRemove: 'r1'.
	server userRemove: 'r2'.

	self deny: (server userIncludes: 'r1').
	self deny: (server userIncludes: 'r2').
	self assert: (server userIncludes: 'r3').
	self deny: (server userIncludes: 'r4').! !

!SWServerTests methodsFor: 'testing-serving' stamp: 'chbu 10/22/2003 17:40'!
testStart
	"DISABLED: If there is an instance running on the default port, we get a failure here."

	"server := self defaultServerClass start.
	[ self assert: server port = SWServer defaultPort.
	  self assert: server host = SWServer defaultHost.
 	  self assert: server ip = SWServer defaultIp.
	  self assert: server isServing ]
	ensure: [ server stop ]"! !

!SWServerTests methodsFor: 'configuration' stamp: 'svp 11/7/2003 01:12'!
defaultServerClass
	^SWKom! !

TestCase subclass: #SWPropertyTests
	instanceVariableNames: 'manager propertyTitle propertyUser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Tests'!

!SWPropertyTests methodsFor: 'testing-accessing' stamp: ''!
testRemove
	self assert: (manager remove: #title) == propertyTitle.
	self assert: (manager remove: #title) isNil.
	self deny: (manager includes: #title).

	self assert: (manager remove: #user ifAbsent: [ true ]) == propertyUser.
	self assert: (manager remove: #user ifAbsent: [ true ]).
	self deny: (manager includes: #use).! !

!SWPropertyTests methodsFor: 'testing-accessing' stamp: ''!
testAtPut
	self assert: (manager at: #title put: (propertyTitle := 'PowerBook')) == propertyTitle.
	self assert: (manager at: #title) == propertyTitle.
	self assert: (manager size) = 2.

	self assert: (manager at: #unknown put: true) == true.
	self assert: (manager at: #unknown) == true.
	self assert: (manager size) = 3.! !

!SWPropertyTests methodsFor: 'testing' stamp: ''!
testEnumerating
	| count |
	count := 0.
	manager do: [ :key :value |
		self assert: (key = #title or: [ key = #user ]).
		self assert: (value == propertyTitle or: [ value == propertyUser ]).
		count := count + 1 ].
	self assert: count = 2! !

!SWPropertyTests methodsFor: 'testing-accessing' stamp: ''!
testAt
	self assert: (manager at: #title) == propertyTitle.
	self assert: (manager at: #user) == propertyUser.
	self assert: (manager at: #unknow) == nil.

	self assert: (manager at: #title ifAbsent: [ true ]) == propertyTitle.
	self assert: (manager at: #user  ifAbsent: [ true ]) == propertyUser.
	self assert: (manager at: #unknow ifAbsent: [ true ]) == true.! !

!SWPropertyTests methodsFor: 'testing-testing' stamp: ''!
testIncludes
	self assert: (manager includes: #title).
	self assert: (manager includes: #user).

	self deny: (manager includes: #unknow).! !

!SWPropertyTests methodsFor: 'running' stamp: 'chbu 10/22/2003 09:41'!
setUp
	manager := SWPropertyManager new
		at: #title put: (propertyTitle := 'SmallWiki');
		at: #user put: (propertyUser := 'Lukas Renggli');
		yourself! !

!SWPropertyTests methodsFor: 'testing-testing' stamp: ''!
testSize
	self assert: manager size = 2! !

!SWPropertyTests methodsFor: 'testing' stamp: ''!
testCopying
	self assert: manager copy ~~ manager.
	self assert: manager copy properties ~~ manager properties.

	manager copy properties asArray with: manager properties asArray do: [ :a :b |
		self assert: a  ~~ b ]! !

TestCase subclass: #SWStructureTests
	instanceVariableNames: 'pa21 pa22 re31 re32 counter parameter fd fd1 fd11 fd111 fd12 fd2 fd3'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallWiki-Kernel-Tests'!

!SWStructureTests methodsFor: 'testing-resolving' stamp: ''!
testResolveSister
	self assert: (fd1 resolveTo: 'Folder 1') == fd1.
	self assert: (fd1 resolveTo: 'Folder 2') == fd2.
	self assert: (fd1 resolveTo: 'Folder 3') == fd3.
	self assert: (fd11 resolveTo: 'Folder 1.1') == fd11.
	self assert: (fd11 resolveTo: 'Folder 1.2') == fd12.
	self assert: (fd111 resolveTo: 'Folder 1.1.1') == fd111.
	self assert: (fd12 resolveTo: 'Folder 1.1') == fd11.
	self assert: (fd12 resolveTo: 'Folder 1.2') == fd12.
	self assert: (fd2 resolveTo: 'Folder 1') == fd1.
	self assert: (fd2 resolveTo: 'Folder 2') == fd2.
	self assert: (fd2 resolveTo: 'Folder 3') == fd3.
	self assert: (pa21 resolveTo: 'Page 2.1') == pa21.
	self assert: (pa21 resolveTo: 'Page 2.2') == pa22.
	self assert: (pa22 resolveTo: 'Page 2.1') == pa21.
	self assert: (pa22 resolveTo: 'Page 2.2') == pa22.
	self assert: (fd3 resolveTo: 'Folder 1') == fd1.
	self assert: (fd3 resolveTo: 'Folder 2') == fd2.
	self assert: (fd3 resolveTo: 'Folder 3') == fd3.
	self assert: (re31 resolveTo: 'Resource 3.1') == re31.
	self assert: (re31 resolveTo: 'Resource 3.2') == re32.
	self assert: (re32 resolveTo: 'Resource 3.1') == re31.
	self assert: (re32 resolveTo: 'Resource 3.2') == re32.
	

"

	self assert: (ch1 resolveTo: 'Folder 1.1') == ch11.
	self assert: (ch11 resolveTo: 'Folder 1.1.1') == ch111.
	self assert: (ch resolveTo: 'Folder 2') == ch2.
	self assert: (ch2 resolveTo: 'Page 2.1') == pa21.
	self assert: (ch2 resolveTo: 'Page 2.2') == pa22.
	self assert: (ch2 resolveTo: 'Page 2.3') == nil.
	self assert: (ch resolveTo: 'Folder 3') == ch3.
	self assert: (ch3 resolveTo: 'Resource 3.1') == re31.
	self assert: (ch3 resolveTo: 'Resource 3.2') == re32.
	self assert: (ch3 resolveTo: 'Resource 3.3') == nil."! !

!SWStructureTests methodsFor: 'testing-navigation' stamp: ''!
testNext
	self assert: fd next == nil.
	self assert: fd1 next == fd2.
	self assert: fd11 next == fd12.
	self assert: fd111 next == nil.
	self assert: fd12 next == nil.
	self assert: fd2 next == fd3.
	self assert: pa21 next == pa22.
	self assert: pa22 next == nil.
	self assert: fd3 next == nil.
	self assert: re31 next == re32.
	self assert: re32 next == nil.! !

!SWStructureTests methodsFor: 'utilites' stamp: ''!
requestWithUrl: anUrlString action: anActionString
	^(self requestWithUrl: anUrlString)
		fields: (Dictionary new
			at: 'action' put: anActionString;
			yourself);
		yourself! !

!SWStructureTests methodsFor: 'testing-resolving' stamp: ''!
testResolvePath
	self assert: (fd resolveTo: '/Folder 1') == fd1.
	self assert: (fd resolveTo: '/Folder 1/Folder 1.1') == fd11.
	self assert: (fd resolveTo: '/Folder 1/Folder 1.1/Folder 1.1.1') == fd111.
	self assert: (fd resolveTo: '/Folder 1/Folder 1.2') == fd12.
	self assert: (fd resolveTo: '/Folder 2') == fd2.
	self assert: (fd resolveTo: '/Folder 2/Page 2.1') == pa21.
	self assert: (fd resolveTo: '/Folder 2/Page 2.2') == pa22.
	self assert: (fd resolveTo: '/Folder 3') == fd3.
	self assert: (fd resolveTo: '/Folder 3/Resource 3.1') == re31.
	self assert: (fd resolveTo: '/Folder 3/Resource 3.2') == re32.! !

!SWStructureTests methodsFor: 'testing-versions' stamp: ''!
testVersionNumber
	10 timesRepeat: [ fd1 nextVersion ].

	0 to: 10 do: [ :version | 
		self assert: (fd1 versionNumber: version) == (fd1 versions at: 11 - version) ].

	self assert: (fd1 versionNumber: 0) == fd1 versions last.
	self assert: (fd11 versionNumber: 0) == fd11 versions last.
	self should: [ fd1 versionNumber: 11 ] raise: Error.
	self should: [ fd11 versionNumber: 1] raise: Error.! !

!SWStructureTests methodsFor: 'testing-properties' stamp: ''!
testLocalProperties
	self deny: fd localProperties isNil.
	self deny: fd localProperties isNil.
	self deny: fd1 localProperties isNil.
	self deny: fd11 localProperties isNil.
	self deny: fd111 localProperties isNil.
	self deny: fd12 localProperties isNil.
	self deny: fd2 localProperties isNil.
	self deny: pa21 localProperties isNil.
	self deny: pa22 localProperties isNil.
	self deny: fd3 localProperties isNil.
	self deny: re31 localProperties isNil.
	self deny: re32 localProperties isNil.

	self assert: fd localProperties size = 2.
	self assert: fd1 localProperties size = 1.
	self assert: fd11 localProperties size = 2.
	self assert: fd111 localProperties size = 1.

	self assert: (fd localProperties at: #level) = 0.
	self assert: (fd1 localProperties at: #level) = 1.
	self assert: (fd11 localProperties at: #level) = 2.
	self assert: (fd111 localProperties at: #level) = 3.

	self assert: (fd localProperties at: #trueTag).
	self assert: (fd1 localProperties at: #trueTag) isNil.
	self assert: (fd11 localProperties at: #trueTag) isNil.
	self assert: (fd111 localProperties at: #trueTag) isNil.

	self assert: (fd localProperties at: #falseTag) isNil.
	self assert: (fd1 localProperties at: #falseTag) isNil.
	self deny: (fd11 localProperties at: #falseTag).
	self assert: (fd111 localProperties at: #falseTag) isNil! !

!SWStructureTests methodsFor: 'testing-versions' stamp: ''!
testVersionChildren
	self assert: (fd at: 'Folder1') == fd1.
	fd1 nextVersion.
	fd1 title: 'New Folder 1'.
	self assert: (fd at: 'NewFolder1') == fd1.
	self assert: (fd1 parent) == fd.! !

!SWStructureTests methodsFor: 'testing-accessing-write' stamp: ''!
testWritePredecessor
	fd predecessor: self.
	self assert: fd predecessor == self! !

!SWStructureTests methodsFor: 'testing-testing' stamp: ''!
testIsEmbedded
	self deny: fd isEmbedded.
	self deny: fd1 isEmbedded.
	self deny: fd11 isEmbedded.
	self deny: fd111 isEmbedded.
	self deny: fd12 isEmbedded.
	self deny: fd2  isEmbedded.
	self deny: pa21 isEmbedded.
	self deny: pa22  isEmbedded.
	self deny: fd3 isEmbedded.
	self assert: re31 isEmbedded.
	self assert: re32 isEmbedded.! !

!SWStructureTests methodsFor: 'testing-notification' stamp: ''!
testChangedAndNextVersion
	fd changed.
	self assert: counter = 1.
	self assert: parameter == fd.
	
	fd nextVersion; changed.
	self assert: counter = 2.
	self assert: parameter == fd.! !

!SWStructureTests methodsFor: 'testing-creational' stamp: 'chbu 10/22/2003 09:49'!
testCreateTitle
	| folder page resource |

	folder := SWFolder title: 'Small Wiki'.
	self assert: folder title = 'Small Wiki'.
	self assert: folder id = 'SmallWiki'.
	self assert: folder parent isNil.
	self assert: folder predecessor isNil.
	self assert: folder timestamp <= TimeStamp now.
	self assert: folder version = 0.
	self assert: folder properties isEmpty.
	self assert: folder children isEmpty.

	page := SWPage title: 'Small Wiki'.
	self assert: page title = 'Small Wiki'.
	self assert: page id = 'SmallWiki'.
	self assert: page parent isNil.
	self assert: page predecessor isNil.
	self assert: page timestamp <= TimeStamp now.
	self assert: page version = 0.
	self assert: page properties isEmpty.
	self deny: page document isNil.

	resource := SWResource title: 'Small Wiki'.
	self assert: resource title = 'Small Wiki'.
	self assert: resource id = 'SmallWiki'.
	self assert: resource parent isNil.
	self assert: resource predecessor isNil.
	self assert: resource timestamp <= TimeStamp now.
	self assert: resource version = 0.
	self assert: resource properties isEmpty.
	self deny: resource data isNil.
	self deny: resource mimetype isNil.! !

!SWStructureTests methodsFor: 'testing-children' stamp: ''!
testChildrenSize
	self assert: fd children size = 3.
	self assert: fd1 children size = 2.
	self assert: fd11 children size = 1.
	self assert: fd111 children size = 0.
	self assert: fd12 children size = 0.
	self assert: fd2 children size = 2.
	self assert: fd3 children size = 2.! !

!SWStructureTests methodsFor: 'testing-children' stamp: ''!
testChildrenRemove
	self assert: (fd remove: fd1) == fd1.
	self assert: fd children size = 2.
	self assert: fd1 parent isNil.
	self deny: (fd includes: fd1 title).

	self should: [ fd remove: fd1 ] raise: Error.
	self assert: fd children size = 2.
	self assert: fd1 parent isNil.
	self deny: (fd includes: fd1 title).! !

!SWStructureTests methodsFor: 'testing-accessing' stamp: 'chbu 10/22/2003 09:51'!
testTimestamp
	self assert: fd timestamp <= TimeStamp now.
	self assert: fd1 timestamp <= TimeStamp now.
	self assert: fd11 timestamp <= TimeStamp now.
	self assert: fd111 timestamp <= TimeStamp now.
	self assert: fd12 timestamp <= TimeStamp now.
	self assert: fd2 timestamp <= TimeStamp now.
	self assert: pa21 timestamp <= TimeStamp now.
	self assert: pa22 timestamp <= TimeStamp now.
	self assert: fd3 timestamp <= TimeStamp now.
	self assert: re31 timestamp <= TimeStamp now.
	self assert: re32 timestamp <= TimeStamp now.! !

!SWStructureTests methodsFor: 'testing-properties' stamp: ''!
testPropertiesAtPut
	fd11 propertyAt: #owner put: 123.
	self assert: (fd11 propertyAt: #owner) = 123.! !

!SWStructureTests methodsFor: 'testing-accessing-write' stamp: ''!
testWriteParent
	fd parent: self.
	self assert: fd parent == self.! !

!SWStructureTests methodsFor: 'testing-accessing-calculated' stamp: ''!
testRoot
	self assert: fd root == fd.
	self assert: fd1 root == fd.
	self assert: fd11 root == fd.
	self assert: fd111 root == fd.
	self assert: fd12 root == fd.
	self assert: fd2 root == fd.
	self assert: pa21 root == fd.
	self assert: pa22 root == fd.
	self assert: fd3 root == fd.
	self assert: re31 root == fd.
	self assert: re32 root == fd.! !

!SWStructureTests methodsFor: 'testing-notification' stamp: ''!
update: anAspectSymbol with: aParameter
	counter := counter + 1.
	parameter := aParameter.! !

!SWStructureTests methodsFor: 'testing-navigation' stamp: ''!
testFirst
	self assert: fd first == fd.
	self assert: fd1 first == fd1.
	self assert: fd11 first == fd11.
	self assert: fd111 first == fd111.
	self assert: fd12 first == fd11.
	self assert: fd2 first == fd1.
	self assert: pa21 first == pa21.
	self assert: pa22 first == pa21.
	self assert: fd3 first == fd1.
	self assert: re31 first == re31.
	self assert: re32 first == re31.! !

!SWStructureTests methodsFor: 'testing-children' stamp: ''!
testChildrenAddAll
	| children |

	"check existing children"
	children := fd1 children copy.
	children do: [ :child |
		self assert: child parent == fd1.
		self assert: (fd1 includes: child id).
		self deny: (fd includes: child id) ].

	"old references have to be removed manually"
	fd1 children removeAll: children.		" carefull: right order is important "
	children do: [ :each |
		fd add: each ].
	children do: [ :child |
		self assert: child parent == fd.
		self assert: (fd includes: child id).
		self deny: (fd1 includes: child id) ].! !

!SWStructureTests methodsFor: 'testing-children' stamp: ''!
testChildrenRemoveAll

	"remove unknown children"
	self 
		should: [ fd children removeAll: fd1 children ]
		raise: Error.
	self assert: fd children size = 3.

	"remove all children"
	fd children removeAll: fd children copy.
	self assert: fd children isEmpty.! !

!SWStructureTests methodsFor: 'testing-versions' stamp: ''!
testVersionRevert
	5 timesRepeat: [ fd1 nextVersion ].
	(fd1 versionNumber: 3) propertyAt: #tag put: true.
	fd1 versionRevert: 3.

	self assert: ((fd1 versionNumber: 0) propertyAt: #tag) = nil.
	self assert: ((fd1 versionNumber: 1) propertyAt: #tag) = nil.
	self assert: ((fd1 versionNumber: 2) propertyAt: #tag) = nil.
	self assert: ((fd1 versionNumber: 6) propertyAt: #tag) = true.

	self should: [ fd1 versionNumber: 3 ] raise: Error.
	self should: [ fd1 versionNumber: 4 ] raise: Error.
	self should: [ fd1 versionNumber: 5 ] raise: Error.

	fd1 versions do: [ :folder |
		self assert: folder parent == fd ].! !

!SWStructureTests methodsFor: 'testing-versions' stamp: 'chbu 10/22/2003 09:52'!
testVersionUpdateReferences
	| link21 link22 |

	pa21 document: (link21 := SWLinkInternal newTo: pa22 title from: pa21).
	pa22 document: (link22 := SWLinkInternal newTo: pa22 title from: pa22).
	self assert: link21 target == pa22.
	self assert: link22 target == pa22.

	self assert: pa21 == pa21 nextVersion.
	self assert: link21 target == pa22.
	self assert: link22 target == pa22.
	
	self assert: pa22 == pa22 nextVersion.
	self assert: link21 target == pa22.
	self assert: link22 target == pa22.

	self assert: pa22 == pa22 nextVersion.
	self assert: link21 target == pa22.
	self assert: link22 target == pa22.! !

!SWStructureTests methodsFor: 'testing-resolving' stamp: ''!
testResolveNotFound
	| testblock |
	testblock := [ :path |
		self assert: (fd resolveTo: path) isNil.
		self assert: (fd1 resolveTo: path) isNil.
		self assert: (fd11 resolveTo: path) isNil.
		self assert: (fd111 resolveTo: path) isNil.
		self assert: (fd2 resolveTo: path) isNil.
		self assert: (pa21 resolveTo: path) isNil.
		self assert: (pa22 resolveTo: path) isNil.
		self assert: (fd3 resolveTo: path) isNil.
		self assert: (re31 resolveTo: path) isNil.
		self assert: (re31 resolveTo: path) isNil ].

	testblock value: 'unknown'.
	testblock value: 'unknown/unknown'.
	testblock value: 'unknown/unknown/unknown'.
	testblock value: 'Folder 1/unknown'.
	testblock value: 'Folder 1/Folder 1.1/unknown'.
	testblock value: 'Folder 1/Folder 1.1/Folder 1.1.1/unknown'! !

!SWStructureTests methodsFor: 'testing-accessing' stamp: ''!
testRoles
	self assert: fd roles isNil.
	self assert: fd1 roles isNil.
	self assert: fd11 roles isNil.
	self assert: fd111 roles isNil.
	self assert: fd12 roles isNil.
	self assert: fd12 roles isNil.
	self assert: pa21 roles isNil.
	self assert: pa22  roles isNil.
	self assert: fd3  roles isNil.
	self assert: re31 roles isNil.
	self assert: re32 roles isNil.! !

!SWStructureTests methodsFor: 'testing-versions' stamp: 'chbu 10/22/2003 09:51'!
testVersionNext
	self assert: fd version = 0.
	self assert: fd versions size = 1.
	self assert: fd versions first == fd. 
	self deny: fd isSuccessor.

	fd := SWFolder version: fd.
	self assert: fd version = 1.
	self assert: fd versions size = 2.
	self assert: fd versions first == fd. 
	self assert: fd isSuccessor.

	fd := fd nextVersion.
	self assert: fd version = 2.
	self assert: fd versions size = 3.
	self assert: fd versions first == fd.
	self assert: fd isSuccessor.! !

!SWStructureTests methodsFor: 'testing-creational' stamp: 'chbu 10/22/2003 09:49'!
testCreateVersion
	| folder page resource |

	folder := SWFolder title: 'Small Wiki'.
	self assert: folder title = 'Small Wiki'.
	self assert: folder id = 'SmallWiki'.
	self assert: folder parent isNil.
	self assert: folder predecessor isNil.
	self assert: folder timestamp <= TimeStamp now.
	self assert: folder version = 0.
	self assert: folder properties isEmpty.
	self assert: folder children isEmpty.

	page := SWPage title: 'Small Wiki'.
	self assert: page title = 'Small Wiki'.
	self assert: page id = 'SmallWiki'.
	self assert: page parent isNil.
	self assert: page predecessor isNil.
	self assert: page timestamp <= TimeStamp now.
	self assert: page version = 0.
	self assert: page properties isEmpty.
	self deny: page document isNil.

	resource := SWResource title: 'Small Wiki'.
	self assert: resource title = 'Small Wiki'.
	self assert: resource id = 'SmallWiki'.
	self assert: resource parent isNil.
	self assert: resource predecessor isNil.
	self assert: resource timestamp <= TimeStamp now.
	self assert: resource version = 0.
	self assert: resource properties isEmpty.
	self deny: resource data isNil.
	self deny: resource mimetype isNil.! !

!SWStructureTests methodsFor: 'testing-notification' stamp: ''!
testChanged	
	fd changed.
	self assert: counter = 1.
	self assert: parameter == fd.

	fd1 changed.
	self assert: counter = 2.
	self assert: parameter == fd1.

	fd11 changed.
	self assert: counter = 3.
	self assert: parameter == fd11.

	fd111 changed.
	self assert: counter = 4.
	self assert: parameter == fd111.! !

!SWStructureTests methodsFor: 'testing-accessing' stamp: ''!
testPredecessor
	self assert: fd predecessor isNil.
	self assert: fd1 predecessor isNil.
	self assert: fd11 predecessor isNil.
	self assert: fd111 predecessor isNil.
	self assert: fd12 predecessor isNil.
	self assert: fd2 predecessor isNil.
	self assert: pa21 predecessor isNil.
	self assert: pa22 predecessor isNil.
	self assert: fd3 predecessor isNil.
	self assert: re31 predecessor isNil.
	self assert: re32 predecessor isNil.! !

!SWStructureTests methodsFor: 'testing-serving' stamp: 'svp 11/7/2003 01:11'!
testProcessSecurity
	| request |
	fd process: (request := self requestWithUrl: '/' action: 'SWPageHistory').
	self deny: ('*Not Found*' match: request response stream contents).
	self assert: ('*Unauthorized*' match: request response stream contents).! !

!SWStructureTests methodsFor: 'testing-properties' stamp: ''!
testLocalPropertiesAtIfAbsent
	self assert: (fd localPropertyAt: #level ifAbsent: [ 123 ]) = 0.
	self assert: (fd1 localPropertyAt: #level ifAbsent: [ 123 ]) = 1.
	self assert: (fd11 localPropertyAt: #level ifAbsent: [ 123 ]) = 2.
	self assert: (fd111 localPropertyAt: #level ifAbsent: [ 123 ]) = 3.

	self assert: (fd localPropertyAt: #trueTag ifAbsent: [ 123 ]) = true.
	self assert: (fd1 localPropertyAt: #trueTag ifAbsent: [ 123 ]) = 123.
	self assert: (fd11 localPropertyAt: #trueTag ifAbsent: [ 123 ]) = 123.
	self assert: (fd111 localPropertyAt: #trueTag ifAbsent: [ 123 ]) = 123.

	self assert: (fd localPropertyAt: #falseTag ifAbsent: [ 123 ]) = 123.
	self assert: (fd1 localPropertyAt: #falseTag ifAbsent: [ 123 ]) = 123.
	self assert: (fd11 localPropertyAt: #falseTag ifAbsent: [ 123 ]) = false.
	self assert: (fd111 localPropertyAt: #falseTag ifAbsent: [ 123 ]) = 123.! !

!SWStructureTests methodsFor: 'testing-properties' stamp: ''!
testLocalPropertiesAtPut
	fd11 localPropertyAt: #owner put: 123.
	self assert: (fd11 localPropertyAt: #owner) = 123.! !

!SWStructureTests methodsFor: 'testing-utilities' stamp: ''!
testIdentifierCaracter
	self assert: $a isWikiIdentifier.
	self assert: $z isWikiIdentifier.
	self assert: $A isWikiIdentifier.
	self assert: $Z isWikiIdentifier.
	self assert: $0 isWikiIdentifier.
	self assert: $9 isWikiIdentifier.

	self assert: $. isWikiIdentifier.
	self assert: $- isWikiIdentifier.
	self assert: $_ isWikiIdentifier.

	self deny: $  isWikiIdentifier.
	self deny: $/ isWikiIdentifier.
	self deny: $: isWikiIdentifier.
	self deny: $& isWikiIdentifier.
	self deny: $? isWikiIdentifier.! !

!SWStructureTests methodsFor: 'testing-accessing-calculated' stamp: ''!
testUrl
	self assert: fd url = '/'.
	self assert: fd1 url = '/Folder1/'.
	self assert: fd11 url = '/Folder1/Folder1.1/'.
	self assert: fd111 url = '/Folder1/Folder1.1/Folder1.1.1/'.
	self assert: fd12 url = '/Folder1/Folder1.2/'.
	self assert: fd2 url = '/Folder2/'.
	self assert: pa21 url = '/Folder2/Page2.1/'.
	self assert: pa22 url = '/Folder2/Page2.2/'.
	self assert: fd3 url = '/Folder3/'.
	self assert: re31 url = '/Folder3/Resource3.1/'.
	self assert: re32 url = '/Folder3/Resource3.2/'.! !

!SWStructureTests methodsFor: 'testing-children' stamp: 'chbu 10/22/2003 09:48'!
testChildrenAdd
	| root page1 page2 |

	"empty chapter"
	root := SWFolder title: 'root'.
	self assert: root children isEmpty.

	"add one page"
	page1 := SWPage title: 'My First Page'.
	root add: page1.
	self deny: root children isEmpty.
	self assert: root children size = 1.
	self assert: page1 parent == root.
	self assert: (root at: 'My First Page') == page1.

	"add another page with same title"
	page2 := SWPage title: 'My First Page'.
	self 
		should: [ root add: page2 ] 
		raise: SWDuplicatedStructure.
	self deny: root children isEmpty.
	self assert: root children size = 1.
	self assert: page1 parent == root.
	self assert: (root at: 'My First Page') == page1.! !

!SWStructureTests methodsFor: 'testing-properties' stamp: ''!
testPropertiesAt
	self assert: (fd propertyAt: #level) = 0.
	self assert: (fd1 propertyAt: #level) = 1.
	self assert: (fd11 propertyAt: #level) = 2.
	self assert: (fd111 propertyAt: #level) = 3.

	self assert: (fd propertyAt: #trueTag) = true.
	self assert: (fd1 propertyAt: #trueTag) = true.
	self assert: (fd11 propertyAt: #trueTag) = true.
	self assert: (fd111 propertyAt: #trueTag) = true.

	self assert: (fd propertyAt: #falseTag) = nil.
	self assert: (fd1 propertyAt: #falseTag) = nil.
	self assert: (fd11 propertyAt: #falseTag) = false.
	self assert: (fd111 propertyAt: #falseTag) = false.! !

!SWStructureTests methodsFor: 'testing-accessing-calculated' stamp: ''!
testParents
	self assert: fd parents size = 1.
	self assert: fd1 parents size = 2.
	self assert: fd11 parents size = 3.
	self assert: fd111 parents size = 4.
	self assert: fd12 parents size = 3.
	self assert: fd2 parents size = 2.
	self assert: pa21 parents size = 3.
	self assert: pa22 parents size = 3.
	self assert: fd3 parents size = 2.
	self assert: re31 parents size = 3.
	self assert: re32 parents size = 3.! !

!SWStructureTests methodsFor: 'testing-resolving' stamp: ''!
testParsePath
	self assert: (fd privateParsePath: '') isEmpty.

	self assert: (fd privateParsePath: 'a') size = 1.
	self assert: (fd privateParsePath: '/') first = ''.
	self assert: (fd privateParsePath: '/') last = ''.

	self assert: (fd privateParsePath: 'a') size = 1.
	self assert: (fd privateParsePath: 'a') first = 'a'.
	self assert: (fd privateParsePath: 'a') last = 'a'.
	self assert: (fd privateParsePath: '/a') size = 2.
	self assert: (fd privateParsePath: '/a') first = ''.
	self assert: (fd privateParsePath: '/a') last = 'a'.

	self assert: (fd privateParsePath: 'a/b/c') size = 3.
	self assert: (fd privateParsePath: 'a/b/c') first = 'a'.
	self assert: (fd privateParsePath: 'a/b/c') last = 'c'.
	self assert: (fd privateParsePath: '/a/b/c') size = 4.
	self assert: (fd privateParsePath: '/a/b/c') first = ''.
	self assert: (fd privateParsePath: '/a/b/c') last = 'c'.

	self assert: (fd privateParsePath: ' a ') size = 1.
	self assert: (fd privateParsePath: ' a ') first = 'a'.
	self assert: (fd privateParsePath: ' a ') last = 'a'.
	self assert: (fd privateParsePath: '/ a ') size = 2.
	self assert: (fd privateParsePath: '/ a ') first = ''.
	self assert: (fd privateParsePath: '/ a ') last = 'a'.

	self assert: (fd privateParsePath: ' a / b / c ') size = 3.
	self assert: (fd privateParsePath: ' a / b / c ') first = 'a'.
	self assert: (fd privateParsePath: ' a / b / c ') last = 'c'.
	self assert: (fd privateParsePath: ' /a / b / c ') size = 4.
	self assert: (fd privateParsePath: ' /a / b / c ') first = ''.
	self assert: (fd privateParsePath: ' /a / b / c ') last = 'c'! !

!SWStructureTests methodsFor: 'testing-resolving' stamp: ''!
testResolveReceiver
	self assert: (fd1 resolveTo: 'Folder 1') == fd1.
	self assert: (fd11 resolveTo: 'Folder 1.1') == fd11.
	self assert: (fd111 resolveTo: 'Folder 1.1.1') == fd111.
	self assert: (fd2 resolveTo: 'Folder 2') == fd2.
	self assert: (pa21 resolveTo: 'Page 2.1') == pa21.
	self assert: (pa22 resolveTo: 'Page 2.2') == pa22.
	self assert: (fd3 resolveTo: 'Folder 3') == fd3.
	self assert: (re31 resolveTo: 'Resource 3.1') == re31.
	self assert: (re32 resolveTo: 'Resource 3.2') == re32.

	self assert: (fd1 resolveTo: '') == fd1.
	self assert: (fd11 resolveTo: '') == fd11.
	self assert: (fd111 resolveTo: '') == fd111.
	self assert: (fd2 resolveTo: '') == fd2.
	self assert: (pa21 resolveTo: '') == pa21.
	self assert: (pa22 resolveTo: '') == pa22.
	self assert: (fd3 resolveTo: '') == fd3.
	self assert: (re31 resolveTo: '') == re31.
	self assert: (re32 resolveTo: '') == re32.! !

!SWStructureTests methodsFor: 'testing-creational' stamp: 'chbu 10/24/2003 17:22'!
testCreateParent
	| folder1 |
	folder1 := SWFolder parent: fd.
"	self 
		should: [ folder2 := SWFolder parent: fd ]
		raise: SWDuplicatedStructure."

	self assert: folder1 title = 'Folder'.
	self assert: folder1 id = 'Folder'.
	self assert: folder1 parent == fd.
	self assert: folder1 predecessor isNil.
	self assert: folder1 timestamp <= TimeStamp now.
	self assert: folder1 version = 0.
	self assert: folder1 localProperties isEmpty.
	self assert: folder1 children isEmpty.! !

!SWStructureTests methodsFor: 'testing-children' stamp: ''!
testChildren
	self deny: fd children isNil.
	self deny: fd1 children isNil.
	self deny: fd11 children isNil.
	self deny: fd111 children isNil.
	self deny: fd12 children isNil.
	self deny: fd2 children isNil.
	self deny: fd3 children isNil.! !

!SWStructureTests methodsFor: 'testing-testing' stamp: ''!
testIsSuccessor
	self deny: fd isSuccessor.
	self deny: fd1 isSuccessor.
	self deny: fd11 isSuccessor.
	self deny: fd111 isSuccessor.
	self deny: fd12 isSuccessor.
	self deny: fd2  isSuccessor.
	self deny: pa21 isSuccessor.
	self deny: pa22  isSuccessor.
	self deny: fd3 isSuccessor.
	self deny: re31 isSuccessor.
	self deny: re32 isSuccessor.! !

!SWStructureTests methodsFor: 'testing-serving' stamp: ''!
testProcessChild
	| request |
	fd process: (request := self requestWithUrl: '/Folder1').
	self deny: ('*Not Found*' match: request response stream contents).
	self assert: ('*<title>Root: Folder 1</title>*' match: request response stream contents).

	fd process: (request := self requestWithUrl: '/Folder1/').
	self deny: ('*Not Found*' match: request response stream contents).
	self assert: ('*<title>Root: Folder 1</title>*' match: request response stream contents).

	fd process: (request := self requestWithUrl: '/Folder1/Folder1.1').
	self deny: ('*Not Found*' match: request response stream contents).
	self assert: ('*<title>Root: Folder 1.1</title>*' match: request response stream contents).

	fd process: (request := self requestWithUrl: '/Folder 1/Folder1.1/').
	self deny: ('*Not Found*' match: request response stream contents).
	self assert: ('*<title>Root: Folder 1.1</title>*' match: request response stream contents).

	fd process: (request := self requestWithUrl: '/Folder2/Page2.1').
	self deny: ('*Not Found*' match: request response stream contents).
	self assert: ('*<title>Root: Page 2.1</title>*' match: request response stream contents).

	fd process: (request := self requestWithUrl: '/Folder3/Resource3.1').
	self deny: ('*Not Found*' match: request response stream contents).
	self assert: ('*<title>Root: Edit Resource 3.1</title>*' match: request response stream contents).! !

!SWStructureTests methodsFor: 'testing-navigation' stamp: ''!
testLast
	self assert: fd last == fd.
	self assert: fd1 last == fd3.
	self assert: fd11 last == fd12.
	self assert: fd111 last == fd111.
	self assert: fd12 last == fd12.
	self assert: fd2 last == fd3.
	self assert: pa21 last == pa22.
	self assert: pa22 last == pa22.
	self assert: fd3 last == fd3.
	self assert: re31 last == re32.
	self assert: re32 last == re32.! !

!SWStructureTests methodsFor: 'running' stamp: ''!
tearDown
	fd removeDependent: self! !

!SWStructureTests methodsFor: 'testing-notification' stamp: ''!
testChangedAspectParam	
	fd changed: #hallo with: pa22.
	self assert: counter = 1.
	self assert: parameter == pa22.

	fd1 changed: #hallo with: pa22.
	self assert: counter = 2.
	self assert: parameter == pa22.

	fd11 changed: #hallo with: pa22.
	self assert: counter = 3.
	self assert: parameter == pa22.

	fd111 changed: #hallo with: pa22.
	self assert: counter = 4.
	self assert: parameter == pa22.! !

!SWStructureTests methodsFor: 'testing-accessing-write' stamp: ''!
testWriteVersion
	fd version: 123456.
	self assert: fd version = 123456.! !

!SWStructureTests methodsFor: 'testing-versions' stamp: ''!
testVersionTruncate
	5 timesRepeat: [ fd1 nextVersion ].
	fd1 versionTruncate: 3.

	self should: [ fd1 versionNumber: 0 ] raise: Error.
	self should: [ fd1 versionNumber: 1 ] raise: Error.
	self should: [ fd1 versionNumber: 2 ] raise: Error.
	self deny: (fd1 versionNumber: 3) isNil.
	self deny: (fd1 versionNumber: 4) isNil.
	self deny: (fd1 versionNumber: 5) isNil.

	self assert: fd1 versions size = 3.
	fd1 versions do: [ :folder |
		self assert: folder parent == fd ].! !

!SWStructureTests methodsFor: 'testing-accessing-write' stamp: 'chbu 10/22/2003 09:51'!
testWriteTimestamp
	fd timestamp: TimeStamp now.
	self assert: fd timestamp <= TimeStamp now.! !

!SWStructureTests methodsFor: 'testing-properties' stamp: ''!
testLocalPropertiesAt
	self assert: (fd localPropertyAt: #level) = 0.
	self assert: (fd1 localPropertyAt: #level) = 1.
	self assert: (fd11 localPropertyAt: #level) = 2.
	self assert: (fd111 localPropertyAt: #level) = 3.

	self assert: (fd localPropertyAt: #trueTag) = true.
	self assert: (fd1 localPropertyAt: #trueTag) = nil.
	self assert: (fd11 localPropertyAt: #trueTag) = nil.
	self assert: (fd111 localPropertyAt: #trueTag) = nil.

	self assert: (fd localPropertyAt: #falseTag) = nil.
	self assert: (fd1 localPropertyAt: #falseTag) = nil.
	self assert: (fd11 localPropertyAt: #falseTag) = false.
	self assert: (fd111 localPropertyAt: #falseTag) = nil.! !

!SWStructureTests methodsFor: 'testing-accessing' stamp: ''!
testParent
	self assert: fd parent = nil.
	self assert: fd1 parent = fd.
	self assert: fd11 parent = fd1.
	self assert: fd111 parent = fd11.
	self assert: fd12 parent = fd1.
	self assert: fd2 parent = fd.
	self assert: pa21 parent = fd2.
	self assert: pa22 parent = fd2.
	self assert: fd3 parent = fd.
	self assert: re31 parent = fd3.
	self assert: re32 parent = fd3.! !

!SWStructureTests methodsFor: 'testing-accessing-calculated' stamp: ''!
testId
	self assert: fd id = 'Root'.
	self assert: fd1 id = 'Folder1'.
	self assert: fd11 id = 'Folder1.1'.
	self assert: fd111 id = 'Folder1.1.1'.
	self assert: fd12 id = 'Folder1.2'.
	self assert: fd2 id = 'Folder2'.
	self assert: pa21 id = 'Page2.1'.
	self assert: pa22 id = 'Page2.2'.
	self assert: fd3 id ='Folder3'.
	self assert: re31 id = 'Resource3.1'.
	self assert: re32 id = 'Resource3.2'.! !

!SWStructureTests methodsFor: 'testing-accessing-write' stamp: 'chbu 11/19/2003 19:45'!
testWriteTitle

	"change title of root"
	fd title: 'My Root Page'.
	self assert: fd title = 'My Root Page'.
	self assert: fd id = 'MyRootPage'.
	self assert: fd parent isNil.

	"change title of child (this should not be done)"
	fd1 title: 'My First Page'.
	self assert: fd1 title = 'My First Page'.
	self assert: fd1 id = 'MyFirstPage'.
	self assert: fd1 parent == fd.
	self deny: (fd includes: 'Folder1').
	self assert: (fd at: 'MyFirstPage') == fd1.! !

!SWStructureTests methodsFor: 'testing-accessing' stamp: ''!
testVersion
	self assert: fd version = 0.
	self assert: fd1 version = 0.
	self assert: fd11 version = 0.
	self assert: fd111 version = 0.
	self assert: fd12 version = 0.
	self assert: fd2 version = 0.
	self assert: pa21 version = 0.
	self assert: pa22 version = 0.
	self assert: fd3 version = 0.
	self assert: re31 version = 0.
	self assert: re32 version = 0.! !

!SWStructureTests methodsFor: 'testing-properties' stamp: ''!
testProperties
	self deny: fd properties isNil.
	self deny: fd properties isNil.
	self deny: fd1 properties isNil.
	self deny: fd11 properties isNil.
	self deny: fd111 properties isNil.
	self deny: fd12 properties isNil.
	self deny: fd2 properties isNil.
	self deny: pa21 properties isNil.
	self deny: pa22 properties isNil.
	self deny: fd3 properties isNil.
	self deny: re31 properties isNil.
	self deny: re32 properties isNil.

	self assert: fd properties size = 2.
	self assert: fd1 properties size = 2.
	self assert: fd11 properties size = 3.
	self assert: fd111 properties size = 3.

	self assert: (fd properties at: #level) = 0.
	self assert: (fd1 properties at: #level) = 1.
	self assert: (fd11 properties at: #level) = 2.
	self assert: (fd111 properties at: #level) = 3.

	self assert: (fd properties at: #trueTag) = true.
	self assert: (fd1 properties at: #trueTag) = true.
	self assert: (fd11 properties at: #trueTag) = true.
	self assert: (fd111 properties at: #trueTag) = true.

	self assert: (fd properties at: #falseTag) = nil.
	self assert: (fd1 properties at: #falseTag) = nil.
	self assert: (fd11 properties at: #falseTag) = false.
	self assert: (fd111 properties at: #falseTag) = false.! !

!SWStructureTests methodsFor: 'testing-resolving' stamp: ''!
testResolveChild
	self assert: (fd resolveTo: '/Folder 1') == fd1.
	self assert: (fd1 resolveTo: 'Folder 1/Folder 1.1') == fd11.
	self assert: (fd11 resolveTo: 'Folder 1.1/Folder 1.1.1') == fd111.
	self assert: (fd1 resolveTo: 'Folder 1/Folder 1.2') == fd12.
	self assert: (fd1 resolveTo: 'Folder 1/Folder 1.3') == nil.

	self assert: (fd resolveTo: '/Folder 2') == fd2.
	self assert: (fd2 resolveTo: 'Folder 2/Page 2.1') == pa21.
	self assert: (fd2 resolveTo: 'Folder 2/Page 2.2') == pa22.
	self assert: (fd2 resolveTo: 'Folder 2/Page 2.3') == nil.

	self assert: (fd resolveTo: '/Folder 3') == fd3.
	self assert: (fd3 resolveTo: 'Folder 3/Resource 3.1') == re31.
	self assert: (fd3 resolveTo: 'Folder 3/Resource 3.2') == re32.
	self assert: (fd3 resolveTo: 'Folder 3/Resource 3.3') == nil.! !

!SWStructureTests methodsFor: 'testing-creational' stamp: ''!
testCopy
	fd11 roles: OrderedCollection new.

	self assert: fd11 copy ~~ fd11.
	self assert: fd11 copy properties ~~ fd11 properties.
	self assert: fd11 copy timestamp ~~ fd11 timestamp.
	self assert: fd11 copy roles ~~ fd11 roles.
	self assert: fd11 copy parent == fd11 parent.
	self assert: fd11 copy predecessor == fd11 predecessor.
	self assert: fd11 copy version == fd11 version.

	self assert: pa21 copy ~~ pa21.
	self assert: pa21 copy properties ~~ pa21 properties.
	self assert: pa21 copy timestamp ~~ pa21 timestamp.
	self assert: pa21 copy document  ~~ pa21 document.
	self assert: pa21 copy parent == pa21 parent.
	self assert: pa21 copy predecessor == pa21 predecessor.
	self assert: pa21 copy version == pa21 version.

	self assert: re31 copy ~~ re31.
	self assert: re31 copy properties ~~ re31 properties.
	self assert: re31 copy timestamp ~~ re31 timestamp.
	self assert: re31 copy data  == re31 data.
	self assert: re31 copy parent == re31 parent.
	self assert: re31 copy predecessor == re31 predecessor.
	self assert: re31 copy version == re31 version.! !

!SWStructureTests methodsFor: 'running' stamp: 'chbu 10/22/2003 09:47'!
setUp

	"create tree"
	(fd := SWFolder title: 'Root')
		add: ((fd1 := SWFolder title: 'Folder 1')
			add: ((fd11 := SWFolder title: 'Folder 1.1')
				add: (fd111 := SWFolder title: 'Folder 1.1.1');
				yourself);
			add: (fd12 := SWFolder title: 'Folder 1.2');
			yourself);
		add: ((fd2 := SWFolder title: 'Folder 2')
			add: (pa21 := SWPage title: 'Page 2.1');
			add: (pa22 := SWPage title: 'Page 2.2');
			yourself);
		add: ((fd3 := SWFolder title: 'Folder 3')
			add: (re31 := SWResource title: 'Resource 3.1');
			add: (re32 := SWResource title: 'Resource 3.2');
			yourself).

	"create some properites"
	fd propertyAt: #level put: 0.
	fd1 propertyAt: #level put: 1.
	fd11 propertyAt: #level put: 2.
	fd111 propertyAt: #level put: 3.
	fd propertyAt: #trueTag put: true.
	fd11 propertyAt: #falseTag put: false.

	"dependency"
	fd addDependent: self.
	counter := 0.
	parameter := nil.! !

!SWStructureTests methodsFor: 'testing-notification' stamp: ''!
testChangedAspect	
	fd changed: #hallo.
	self assert: counter = 1.
	self assert: parameter == fd.

	fd1 changed: #hallo.
	self assert: counter = 2.
	self assert: parameter == fd1.

	fd11 changed: #hallo.
	self assert: counter = 3.
	self assert: parameter == fd11.

	fd111 changed: #hallo.
	self assert: counter = 4.
	self assert: parameter == fd111.! !

!SWStructureTests methodsFor: 'testing-children' stamp: ''!
testChildrenAtIfAbsent
	self assert: (fd at: 'Folder 1' ifAbsent: [ false ]) == fd1.
	self assert: (fd1 at: 'Folder 1.1' ifAbsent: [ false ]) == fd11.
	self assert: (fd11 at: 'Folder 1.1.1' ifAbsent: [ false ]) == fd111.
	self assert: (fd1 at: 'Folder 1.2' ifAbsent: [ false ]) == fd12.
	self assert: (fd at: 'Folder 2' ifAbsent: [ false ]) == fd2.
	self assert: (fd2 at: 'Page 2.1' ifAbsent: [ false ]) == pa21.
	self assert: (fd2 at: 'Page 2.2' ifAbsent: [ false ]) == pa22.
	self assert: (fd at: 'Folder 3' ifAbsent: [ false ]) == fd3.
	self assert: (fd3 at: 'Resource 3.1' ifAbsent: [ false ]) == re31.
	self assert: (fd3 at: 'Resource 3.2' ifAbsent: [ false ]) == re32.

	self assert: (fd at: 'Unknown' ifAbsent: [ false ]) == false.! !

!SWStructureTests methodsFor: 'testing-accessing-calculated' stamp: ''!
testParentsFirst
	self assert: fd parents first == fd.
	self assert: fd1 parents first == fd.
	self assert: fd11 parents first == fd.
	self assert: fd111 parents first == fd.
	self assert: fd12 parents first == fd.
	self assert: fd2 parents first == fd.
	self assert: pa21 parents first == fd.
	self assert: pa22 parents first == fd.
	self assert: fd3 parents first == fd.
	self assert: re31 parents first == fd.
	self assert: re32 parents first == fd.! !

!SWStructureTests methodsFor: 'testing-children' stamp: ''!
testChildrenIncludes
	self assert: (fd includes: 'Folder1').
	self assert: (fd1 includes: 'Folder1.1').
	self assert: (fd11 includes: 'Folder1.1.1').
	self assert: (fd1 includes: 'Folder1.2').
	self assert: (fd includes: 'Folder2').
	self assert: (fd2 includes: 'Page2.1').
	self assert: (fd2 includes: 'Page2.2').
	self assert: (fd includes: 'Folder3').
	self assert: (fd3 includes: 'Resource3.1').
	self assert: (fd3 includes: 'Resource3.2').

	self assert: (fd includes: 'Folder  1').
	self assert: (fd includes: '    Folder 1').
	self assert: (fd includes: 'Folder 1   ').

	self deny: (fd includes: 'Unknown').
	self deny: (fd includes: 'Unknown    ').
	self deny: (fd includes: '    Unknown').! !

!SWStructureTests methodsFor: 'testing-versions' stamp: ''!
testVersionRestore
	5 timesRepeat: [ fd1 nextVersion ].
	(fd1 versionNumber: 3) propertyAt: #tag put: true.
	fd1 versionRestore: 3.

	self assert: ((fd1 versionNumber: 0) propertyAt: #tag) = nil.
	self assert: ((fd1 versionNumber: 1) propertyAt: #tag) = nil.
	self assert: ((fd1 versionNumber: 2) propertyAt: #tag) = nil.
	self assert: ((fd1 versionNumber: 3) propertyAt: #tag) = true.
	self assert: ((fd1 versionNumber: 4) propertyAt: #tag) = nil.
	self assert: ((fd1 versionNumber: 5) propertyAt: #tag) = nil.
	self assert: ((fd1 versionNumber: 6) propertyAt: #tag) = true.

	fd1 versions do: [ :folder |
		self assert: folder parent == fd ].! !

!SWStructureTests methodsFor: 'neu' stamp: 'chbu 11/19/2003 19:26'!
testWriteRename

	" rename to nil "
	self
		should: [ fd1 renameTo: nil ]
		raise: Exception.
	self assert: fd1 title = 'Folder 1'.	

	" rename to empty string "
	self
		should: [ fd1 renameTo: String new ]
		raise: SWDuplicatedStructure.
	self assert: fd1 title = 'Folder 1'.

	" rename to existing item "
	self
		should: [ fd1 renameTo: 'Folder 2' ]
		raise: SWDuplicatedStructure.
	self assert: fd1 title = 'Folder 1'.

	" renaming root is no problem "
	fd renameTo: 'New Root Name'.
	self assert: fd title = 'New Root Name'.! !

!SWStructureTests methodsFor: 'utilites' stamp: 'chbu 10/22/2003 09:49'!
requestWithUrl: anUrlString
	^(SWRequest server: SWServer new)
		headers: Dictionary new;
		fields: Dictionary new;
		cookies: Dictionary new;
		url: anUrlString;
		yourself! !

!SWStructureTests methodsFor: 'testing-testing' stamp: ''!
testIsRoot
	self assert: fd isRoot.
	self deny: fd1 isRoot.
	self deny: fd11 isRoot.
	self deny: fd111 isRoot.
	self deny: fd12 isRoot.
	self deny: fd2  isRoot.
	self deny: pa21 isRoot.
	self deny: pa22  isRoot.
	self deny: fd3 isRoot.
	self deny: re31 isRoot.
	self deny: re32 isRoot.! !

!SWStructureTests methodsFor: 'testing-serving' stamp: ''!
testProcessAction
	| request default |
	fd process: (request := self requestWithUrl: '/').
	default := request response stream contents.
	
	fd process: (request := self requestWithUrl: '/' action: 'PageView').
	self deny: ('*Not Found*' match: request response stream contents).
	self assert: default = request response stream contents.

	fd process: (request := self requestWithUrl: '/' action: 'UnknownAction').
	self deny: ('*Not Found*' match: request response stream contents).
	self assert: default = request response stream contents.! !

!SWStructureTests methodsFor: 'testing-children' stamp: ''!
testChildrenAt
	self assert: (fd at: 'Folder 1') == fd1.
	self assert: (fd1 at: 'Folder 1.1') == fd11.
	self assert: (fd11 at: 'Folder 1.1.1') == fd111.
	self assert: (fd1 at: 'Folder 1.2') == fd12.
	self assert: (fd at: 'Folder 2') == fd2.
	self assert: (fd2 at: 'Page 2.1') == pa21.
	self assert: (fd2 at: 'Page 2.2') == pa22.
	self assert: (fd at: 'Folder 3') == fd3.
	self assert: (fd3 at: 'Resource 3.1') == re31.
	self assert: (fd3 at: 'Resource 3.2') == re32.

	self assert: (fd at: 'Unknown') isNil.! !

!SWStructureTests methodsFor: 'testing-resolving' stamp: ''!
testResolvePathRoot
	self assert: (fd resolveTo: '/Folder 1/Folder 1.1/Folder 1.1.1') == fd111.
	self assert: (fd1 resolveTo: '/Folder 1/Folder 1.1/Folder 1.1.1') == fd111.
	self assert: (fd11 resolveTo: '/Folder 1/Folder 1.1/Folder 1.1.1') == fd111.
	self assert: (fd111 resolveTo: '/Folder 1/Folder 1.1/Folder 1.1.1') == fd111.
	self assert: (fd12 resolveTo: '/Folder 1/Folder 1.1/Folder 1.1.1') == fd111.
	self assert: (fd2 resolveTo: '/Folder 1/Folder 1.1/Folder 1.1.1') == fd111.
	self assert: (pa21 resolveTo: '/Folder 1/Folder 1.1/Folder 1.1.1') == fd111.
	self assert: (pa22 resolveTo: '/Folder 1/Folder 1.1/Folder 1.1.1') == fd111.
	self assert: (fd3 resolveTo: '/Folder 1/Folder 1.1/Folder 1.1.1') == fd111.
	self assert: (re31 resolveTo: '/Folder 1/Folder 1.1/Folder 1.1.1') == fd111.
	self assert: (re31 resolveTo: '/Folder 1/Folder 1.1/Folder 1.1.1') == fd111.! !

!SWStructureTests methodsFor: 'testing-serving' stamp: ''!
testProcessNotFound
	| request |
	fd process: (request := self requestWithUrl: '/Folder4').
	self assert: ('*Not Found*' match: request response stream contents).

	fd process: (request := self requestWithUrl: '/Folder2/Page2.1/Page2.1.1').
	self assert: ('*Not Found*' match: request response stream contents).

	fd process: (request := self requestWithUrl: '/Folder3/Resource3.1/Resource3.1.1').
	self assert: ('*Not Found*' match: request response stream contents).! !

!SWStructureTests methodsFor: 'testing-accessing-calculated' stamp: ''!
testParentsLast
	self assert: fd parents last == fd.
	self assert: fd1 parents last == fd1.
	self assert: fd11 parents last == fd11.
	self assert: fd111 parents last == fd111.
	self assert: fd12 parents last == fd12.
	self assert: fd2 parents last == fd2.
	self assert: pa21 parents last == pa21.
	self assert: pa22 parents last == pa22.
	self assert: fd3 parents last == fd3.
	self assert: re31 parents last == re31.
	self assert: re32 parents last == re32.! !

!SWStructureTests methodsFor: 'testing-utilities' stamp: ''!
testIdentifierString
	self assert: 'SmallWiki' asWikiIdentifier = 'SmallWiki'.
	self assert: 'Lukas Renggli' asWikiIdentifier = 'LukasRenggli'.
	self assert: 'favicon.ico' asWikiIdentifier = 'favicon.ico'.
	self assert: '/usr/sbin/apache' asWikiIdentifier = 'usrsbinapache'.
	self assert: 'apache_src-1.2beta9.zip' asWikiIdentifier = 'apache_src-1.2beta9.zip'.! !

!SWStructureTests methodsFor: 'testing-accessing' stamp: ''!
testTitle
	self assert: fd title = 'Root'.
	self assert: fd1 title = 'Folder 1'.
	self assert: fd11 title = 'Folder 1.1'.
	self assert: fd111 title = 'Folder 1.1.1'.
	self assert: fd12 title = 'Folder 1.2'.
	self assert: fd2 title = 'Folder 2'.
	self assert: pa21 title = 'Page 2.1'.
	self assert: pa22 title = 'Page 2.2'.
	self assert: fd3 title ='Folder 3'.
	self assert: re31 title = 'Resource 3.1'.
	self assert: re32 title = 'Resource 3.2'.! !

!SWStructureTests methodsFor: 'testing-accessing-calculated' stamp: ''!
testVersions
	| item |

	" test initial configuration "
	self assert: fd1 version = 0.
	self assert: fd1 versions size = 1.
	self assert: fd1 versions first = fd1.
	self assert: fd1 versions last = fd1.

	" new version "
	fd1 nextVersion.

	" test new version "
	self assert: fd1 version = 1.
	self assert: fd1 versions size = 2.
	self assert: fd1 versions first = fd1.
	self assert: fd1 versions last ~= fd1.

	" some more versions "
	18 timesRepeat: [ fd1 nextVersion ].

	" check attributes "
	1 to: 20 do: [ :index |
		item := fd1 versions at: index.
		self assert: item parent = fd.
		self assert: item version = (20 - index).
		self assert: item versions size = (21 - index).
		self assert: item versions first = item ].
	
	fd1 versions do: [ :first | 
		fd1 versions do: [ :second | 
			first version ~~ second version 
				ifTrue: [ self deny: first == second ] ] ]! !

!SWStructureTests methodsFor: 'testing-navigation' stamp: ''!
testPrevious
	self assert: fd previous == nil.
	self assert: fd1 previous == nil.
	self assert: fd11 previous == nil.
	self assert: fd111 previous == nil.
	self assert: fd12 previous == fd11.
	self assert: fd2 previous == fd1.
	self assert: pa21 previous == nil.
	self assert: pa22 previous == pa21.
	self assert: fd3 previous == fd2.
	self assert: re31 previous == nil.
	self assert: re32 previous == re31.! !

!SWStructureTests methodsFor: 'testing-serving' stamp: 'chbu 10/24/2003 16:32'!
testProcess
	| request |
	fd process: (request := self requestWithUrl: '').
	self assert: ('*<title>Root: Root</title>*' match: request response stream contents).

	fd process: (request := self requestWithUrl: '/').
	self assert: ('*<title>Root: Root</title>*' match: request response stream contents).! !

!SWStructureTests methodsFor: 'testing-testing' stamp: ''!
testIsComposite
	self assert: fd isComposite.
	self assert: fd1 isComposite.
	self assert: fd11 isComposite.
	self assert: fd111 isComposite.
	self assert: fd12 isComposite.
	self assert: fd2  isComposite.
	self deny: pa21 isComposite.
	self deny: pa22  isComposite.
	self assert: fd3 isComposite.
	self deny: re31 isComposite.
	self deny: re32 isComposite.! !

!SWStructureTests methodsFor: 'testing-accessing-write' stamp: ''!
testWriteRole
	fd roles: self.
	self assert: fd roles == self.! !

!SWStructureTests methodsFor: 'testing-properties' stamp: ''!
testPropertiesAtIfAbsent
	self assert: (fd propertyAt: #level ifAbsent: [ 123 ]) = 0.
	self assert: (fd1 propertyAt: #level ifAbsent: [ 123 ]) = 1.
	self assert: (fd11 propertyAt: #level ifAbsent: [ 123 ]) = 2.
	self assert: (fd111 propertyAt: #level ifAbsent: [ 123 ]) = 3.

	self assert: (fd propertyAt: #trueTag ifAbsent: [ 123 ]) = true.
	self assert: (fd1 propertyAt: #trueTag ifAbsent: [ 123 ]) = true.
	self assert: (fd11 propertyAt: #trueTag ifAbsent: [ 123 ]) = true.
	self assert: (fd111 propertyAt: #trueTag ifAbsent: [ 123 ]) = true.

	self assert: (fd propertyAt: #falseTag ifAbsent: [ 123 ]) = 123.
	self assert: (fd1 propertyAt: #falseTag ifAbsent: [ 123 ]) = 123.
	self assert: (fd11 propertyAt: #falseTag ifAbsent: [ 123 ]) = false.
	self assert: (fd111 propertyAt: #falseTag ifAbsent: [ 123 ]) = false.! !
SWAction initialize!
SWStructure initialize!
SWFolder initialize!
SWPage initialize!
SWResource initialize!
